@@ -129,12 +129,12 @@ syncdaemon_daemon_class_init (SyncdaemonDaemonClass *klass)
 
 	/* Register signals */
 	daemon_signals[READY_SIGNAL] = g_signal_new ("ready",
-						     G_TYPE_FROM_CLASS (klass),
-						     (GSignalFlags) G_SIGNAL_RUN_LAST,
-						     G_STRUCT_OFFSET (SyncdaemonDaemonClass, ready),
-						     NULL, NULL,
-						     g_cclosure_marshal_VOID__VOID,
-						     G_TYPE_NONE, 0);
+							 G_TYPE_FROM_CLASS (klass),
+							 (GSignalFlags) G_SIGNAL_RUN_LAST,
+							 G_STRUCT_OFFSET (SyncdaemonDaemonClass, ready),
+							 NULL, NULL,
+							 g_cclosure_marshal_VOID__VOID,
+							 G_TYPE_NONE, 0);
 	daemon_signals[CONNECTED_SIGNAL] = g_signal_new ("connected",
 							 G_TYPE_FROM_CLASS (klass),
 							 (GSignalFlags) G_SIGNAL_RUN_LAST,
@@ -143,54 +143,54 @@ syncdaemon_daemon_class_init (SyncdaemonDaemonClass *klass)
 							 g_cclosure_marshal_VOID__VOID,
 							 G_TYPE_NONE, 0);
 	daemon_signals[DISCONNECTED_SIGNAL] = g_signal_new ("disconnected",
-							    G_TYPE_FROM_CLASS (klass),
-							    (GSignalFlags) G_SIGNAL_RUN_LAST,
-							    G_STRUCT_OFFSET (SyncdaemonDaemonClass, disconnected),
-							    NULL, NULL,
-							    g_cclosure_marshal_VOID__VOID,
-							    G_TYPE_NONE, 0);
+								G_TYPE_FROM_CLASS (klass),
+								(GSignalFlags) G_SIGNAL_RUN_LAST,
+								G_STRUCT_OFFSET (SyncdaemonDaemonClass, disconnected),
+								NULL, NULL,
+								g_cclosure_marshal_VOID__VOID,
+								G_TYPE_NONE, 0);
 	daemon_signals[STATUS_CHANGED_SIGNAL] = g_signal_new ("status_changed",
-							      G_TYPE_FROM_CLASS (klass),
-							      (GSignalFlags) G_SIGNAL_RUN_LAST,
-							      G_STRUCT_OFFSET (SyncdaemonDaemonClass, status_changed),
-							      NULL, NULL,
-							      g_cclosure_marshal_VOID__OBJECT,
-							      G_TYPE_NONE, 1,
-							      G_TYPE_OBJECT);
+								  G_TYPE_FROM_CLASS (klass),
+								  (GSignalFlags) G_SIGNAL_RUN_LAST,
+								  G_STRUCT_OFFSET (SyncdaemonDaemonClass, status_changed),
+								  NULL, NULL,
+								  g_cclosure_marshal_VOID__OBJECT,
+								  G_TYPE_NONE, 1,
+								  G_TYPE_OBJECT);
 	daemon_signals[EVENT_SIGNAL] = g_signal_new ("event",
-						     G_TYPE_FROM_CLASS (klass),
-						     (GSignalFlags) G_SIGNAL_RUN_LAST,
-						     G_STRUCT_OFFSET (SyncdaemonDaemonClass, event),
-						     NULL, NULL,
-						     g_cclosure_marshal_VOID__POINTER,
-						     G_TYPE_NONE, 1,
-						     G_TYPE_POINTER);
+							 G_TYPE_FROM_CLASS (klass),
+							 (GSignalFlags) G_SIGNAL_RUN_LAST,
+							 G_STRUCT_OFFSET (SyncdaemonDaemonClass, event),
+							 NULL, NULL,
+							 g_cclosure_marshal_VOID__POINTER,
+							 G_TYPE_NONE, 1,
+							 G_TYPE_POINTER);
 	daemon_signals[ERROR_SIGNAL] = g_signal_new ("error",
-						     G_TYPE_FROM_CLASS (klass),
-						     (GSignalFlags) G_SIGNAL_RUN_LAST,
-						     G_STRUCT_OFFSET (SyncdaemonDaemonClass, error),
-						     NULL, NULL,
-						     _syncdaemon_marshal_VOID__STRING_POINTER,
-						     G_TYPE_NONE, 2,
-						     G_TYPE_STRING, G_TYPE_POINTER);
+							 G_TYPE_FROM_CLASS (klass),
+							 (GSignalFlags) G_SIGNAL_RUN_LAST,
+							 G_STRUCT_OFFSET (SyncdaemonDaemonClass, error),
+							 NULL, NULL,
+							 _syncdaemon_marshal_VOID__STRING_POINTER,
+							 G_TYPE_NONE, 2,
+							 G_TYPE_STRING, G_TYPE_POINTER);
 	daemon_signals[FOLDER_CREATED_SIGNAL] = g_signal_new ("folder_created",
-							      G_TYPE_FROM_CLASS (klass),
-							      (GSignalFlags) G_SIGNAL_RUN_LAST,
-							      G_STRUCT_OFFSET (SyncdaemonDaemonClass, folder_created),
-							      NULL, NULL,
-							      _syncdaemon_marshal_VOID__BOOLEAN_OBJECT,
-							      G_TYPE_NONE, 2,
-							      G_TYPE_BOOLEAN,
-							      G_TYPE_OBJECT);
+								  G_TYPE_FROM_CLASS (klass),
+								  (GSignalFlags) G_SIGNAL_RUN_LAST,
+								  G_STRUCT_OFFSET (SyncdaemonDaemonClass, folder_created),
+								  NULL, NULL,
+								  _syncdaemon_marshal_VOID__BOOLEAN_OBJECT,
+								  G_TYPE_NONE, 2,
+								  G_TYPE_BOOLEAN,
+								  G_TYPE_OBJECT);
 	daemon_signals[FOLDER_DELETED_SIGNAL] = g_signal_new ("folder_deleted",
-							      G_TYPE_FROM_CLASS (klass),
-							      (GSignalFlags) G_SIGNAL_RUN_LAST,
-							      G_STRUCT_OFFSET (SyncdaemonDaemonClass, folder_deleted),
-							      NULL, NULL,
-							      _syncdaemon_marshal_VOID__BOOLEAN_OBJECT,
-							      G_TYPE_NONE, 2,
-							      G_TYPE_BOOLEAN,
-							      G_TYPE_OBJECT);
+								  G_TYPE_FROM_CLASS (klass),
+								  (GSignalFlags) G_SIGNAL_RUN_LAST,
+								  G_STRUCT_OFFSET (SyncdaemonDaemonClass, folder_deleted),
+								  NULL, NULL,
+								  _syncdaemon_marshal_VOID__BOOLEAN_OBJECT,
+								  G_TYPE_NONE, 2,
+								  G_TYPE_BOOLEAN,
+								  G_TYPE_OBJECT);
 	daemon_signals[FOLDER_SUBSCRIBED_SIGNAL] = g_signal_new ("folder_subscribed",
 								 G_TYPE_FROM_CLASS (klass),
 								 (GSignalFlags) G_SIGNAL_RUN_LAST,
@@ -210,14 +210,14 @@ syncdaemon_daemon_class_init (SyncdaemonDaemonClass *klass)
 								   G_TYPE_BOOLEAN,
 								   G_TYPE_OBJECT);
 	daemon_signals[FILE_PUBLISHED_SIGNAL] = g_signal_new ("file_published",
-							      G_TYPE_FROM_CLASS (klass),
-							      (GSignalFlags) G_SIGNAL_RUN_LAST,
-							      G_STRUCT_OFFSET (SyncdaemonDaemonClass, file_published),
-							      NULL, NULL,
-							      _syncdaemon_marshal_VOID__BOOLEAN_OBJECT,
-							      G_TYPE_NONE, 2,
-							      G_TYPE_BOOLEAN,
-							      G_TYPE_OBJECT);
+								  G_TYPE_FROM_CLASS (klass),
+								  (GSignalFlags) G_SIGNAL_RUN_LAST,
+								  G_STRUCT_OFFSET (SyncdaemonDaemonClass, file_published),
+								  NULL, NULL,
+								  _syncdaemon_marshal_VOID__BOOLEAN_OBJECT,
+								  G_TYPE_NONE, 2,
+								  G_TYPE_BOOLEAN,
+								  G_TYPE_OBJECT);
 	daemon_signals[FILE_UNPUBLISHED_SIGNAL] = g_signal_new ("file_unpublished",
 								G_TYPE_FROM_CLASS (klass),
 								(GSignalFlags) G_SIGNAL_RUN_LAST,
@@ -228,23 +228,23 @@ syncdaemon_daemon_class_init (SyncdaemonDaemonClass *klass)
 								G_TYPE_BOOLEAN,
 								G_TYPE_OBJECT);
 	daemon_signals[SHARE_CREATED_SIGNAL] = g_signal_new ("share_created",
-							     G_TYPE_FROM_CLASS (klass),
-							     (GSignalFlags) G_SIGNAL_RUN_LAST,
-							     G_STRUCT_OFFSET (SyncdaemonDaemonClass, share_created),
-							     NULL, NULL,
-							     _syncdaemon_marshal_VOID__BOOLEAN_OBJECT,
-							     G_TYPE_NONE, 2,
-							     G_TYPE_BOOLEAN,
-							     G_TYPE_OBJECT);
+								 G_TYPE_FROM_CLASS (klass),
+								 (GSignalFlags) G_SIGNAL_RUN_LAST,
+								 G_STRUCT_OFFSET (SyncdaemonDaemonClass, share_created),
+								 NULL, NULL,
+								 _syncdaemon_marshal_VOID__BOOLEAN_OBJECT,
+								 G_TYPE_NONE, 2,
+								 G_TYPE_BOOLEAN,
+								 G_TYPE_OBJECT);
 	daemon_signals[SHARE_DELETED_SIGNAL] = g_signal_new ("share_deleted",
-							     G_TYPE_FROM_CLASS (klass),
-							     (GSignalFlags) G_SIGNAL_RUN_LAST,
-							     G_STRUCT_OFFSET (SyncdaemonDaemonClass, share_deleted),
-							     NULL, NULL,
-							     _syncdaemon_marshal_VOID__BOOLEAN_OBJECT,
-							     G_TYPE_NONE, 2,
-							     G_TYPE_BOOLEAN,
-							     G_TYPE_OBJECT);
+								 G_TYPE_FROM_CLASS (klass),
+								 (GSignalFlags) G_SIGNAL_RUN_LAST,
+								 G_STRUCT_OFFSET (SyncdaemonDaemonClass, share_deleted),
+								 NULL, NULL,
+								 _syncdaemon_marshal_VOID__BOOLEAN_OBJECT,
+								 G_TYPE_NONE, 2,
+								 G_TYPE_BOOLEAN,
+								 G_TYPE_OBJECT);
 	daemon_signals[DOWNLOAD_STARTED_SIGNAL] = g_signal_new ("download_started",
 								G_TYPE_FROM_CLASS (klass),
 								(GSignalFlags) G_SIGNAL_RUN_LAST,
@@ -254,13 +254,13 @@ syncdaemon_daemon_class_init (SyncdaemonDaemonClass *klass)
 								G_TYPE_NONE, 1,
 								G_TYPE_STRING);
 	daemon_signals[DOWNLOAD_FILE_PROGRESS_SIGNAL] = g_signal_new ("download_file_progress",
-								      G_TYPE_FROM_CLASS (klass),
-								      (GSignalFlags) G_SIGNAL_RUN_LAST,
-								      G_STRUCT_OFFSET (SyncdaemonDaemonClass, download_file_progress),
-								      NULL, NULL,
-								      _syncdaemon_marshal_VOID__STRING_OBJECT,
-								      G_TYPE_NONE, 2,
-								      G_TYPE_STRING, G_TYPE_OBJECT);
+									  G_TYPE_FROM_CLASS (klass),
+									  (GSignalFlags) G_SIGNAL_RUN_LAST,
+									  G_STRUCT_OFFSET (SyncdaemonDaemonClass, download_file_progress),
+									  NULL, NULL,
+									  _syncdaemon_marshal_VOID__STRING_OBJECT,
+									  G_TYPE_NONE, 2,
+									  G_TYPE_STRING, G_TYPE_OBJECT);
 	daemon_signals[DOWNLOAD_FINISHED_SIGNAL] = g_signal_new ("download_finished",
 								 G_TYPE_FROM_CLASS (klass),
 								 (GSignalFlags) G_SIGNAL_RUN_LAST,
@@ -270,37 +270,37 @@ syncdaemon_daemon_class_init (SyncdaemonDaemonClass *klass)
 								 G_TYPE_NONE, 2,
 								 G_TYPE_STRING, G_TYPE_OBJECT);
 	daemon_signals[UPLOAD_STARTED_SIGNAL] = g_signal_new ("upload_started",
-							      G_TYPE_FROM_CLASS (klass),
-							      (GSignalFlags) G_SIGNAL_RUN_LAST,
-							      G_STRUCT_OFFSET (SyncdaemonDaemonClass, upload_started),
-							      NULL, NULL,
-							      g_cclosure_marshal_VOID__STRING,
-							      G_TYPE_NONE, 1,
-							      G_TYPE_STRING);
+								  G_TYPE_FROM_CLASS (klass),
+								  (GSignalFlags) G_SIGNAL_RUN_LAST,
+								  G_STRUCT_OFFSET (SyncdaemonDaemonClass, upload_started),
+								  NULL, NULL,
+								  g_cclosure_marshal_VOID__STRING,
+								  G_TYPE_NONE, 1,
+								  G_TYPE_STRING);
 	daemon_signals[UPLOAD_FILE_PROGRESS_SIGNAL] = g_signal_new ("upload_file_progress",
-								    G_TYPE_FROM_CLASS (klass),
-								    (GSignalFlags) G_SIGNAL_RUN_LAST,
-								    G_STRUCT_OFFSET (SyncdaemonDaemonClass, upload_file_progress),
-								    NULL, NULL,
-								    _syncdaemon_marshal_VOID__STRING_OBJECT,
-								    G_TYPE_NONE, 2,
-								    G_TYPE_STRING, G_TYPE_OBJECT);
+									G_TYPE_FROM_CLASS (klass),
+									(GSignalFlags) G_SIGNAL_RUN_LAST,
+									G_STRUCT_OFFSET (SyncdaemonDaemonClass, upload_file_progress),
+									NULL, NULL,
+									_syncdaemon_marshal_VOID__STRING_OBJECT,
+									G_TYPE_NONE, 2,
+									G_TYPE_STRING, G_TYPE_OBJECT);
 	daemon_signals[UPLOAD_FINISHED_SIGNAL] = g_signal_new ("upload_finished",
-							       G_TYPE_FROM_CLASS (klass),
-							       (GSignalFlags) G_SIGNAL_RUN_LAST,
-							       G_STRUCT_OFFSET (SyncdaemonDaemonClass, upload_finished),
-							       NULL, NULL,
-							       _syncdaemon_marshal_VOID__STRING_OBJECT,
-							       G_TYPE_NONE, 2,
-							       G_TYPE_STRING, G_TYPE_OBJECT);
+								   G_TYPE_FROM_CLASS (klass),
+								   (GSignalFlags) G_SIGNAL_RUN_LAST,
+								   G_STRUCT_OFFSET (SyncdaemonDaemonClass, upload_finished),
+								   NULL, NULL,
+								   _syncdaemon_marshal_VOID__STRING_OBJECT,
+								   G_TYPE_NONE, 2,
+								   G_TYPE_STRING, G_TYPE_OBJECT);
 	daemon_signals[QUOTA_EXCEEDED_SIGNAL] = g_signal_new ("quota_exceeded",
-							      G_TYPE_FROM_CLASS (klass),
-							      (GSignalFlags) G_SIGNAL_RUN_LAST,
-							      G_STRUCT_OFFSET (SyncdaemonDaemonClass, quota_exceeded),
-							      NULL, NULL,
-							      g_cclosure_marshal_VOID__POINTER,
-							      G_TYPE_NONE, 1,
-							      G_TYPE_POINTER);
+								  G_TYPE_FROM_CLASS (klass),
+								  (GSignalFlags) G_SIGNAL_RUN_LAST,
+								  G_STRUCT_OFFSET (SyncdaemonDaemonClass, quota_exceeded),
+								  NULL, NULL,
+								  g_cclosure_marshal_VOID__POINTER,
+								  G_TYPE_NONE, 1,
+								  G_TYPE_POINTER);
 }
 
 static void
@@ -309,7 +309,7 @@ quota_exceeded_cb (DBusGProxy *proxy, GHashTable *volume_info, gpointer user_dat
 	SyncdaemonDaemon *daemon = SYNCDAEMON_DAEMON (user_data);
 
 	if (daemon != NULL)
-	        g_signal_emit_by_name (daemon, "quota_exceeded", volume_info);
+			g_signal_emit_by_name (daemon, "quota_exceeded", volume_info);
 }
 
 static gboolean
@@ -330,22 +330,22 @@ setup_daemon_interface (SyncdaemonDaemon *daemon)
 	if (daemon->priv->daemon_interface != NULL)
 		g_object_unref (daemon->priv->daemon_interface);
 	daemon->priv->daemon_interface = g_object_new (SYNCDAEMON_TYPE_INTERFACE,
-						       "daemon", daemon,
-						       NULL);
+							   "daemon", daemon,
+							   NULL);
 	proxy = syncdaemon_interface_setup_proxy (SYNCDAEMON_INTERFACE (daemon->priv->daemon_interface),
-					          SYNCDAEMON_DBUS_NAME, "/",
-					          "com.ubuntuone.SyncDaemon.SyncDaemon");
+							  SYNCDAEMON_DBUS_NAME, "/",
+							  "com.ubuntuone.SyncDaemon.SyncDaemon");
 	if (proxy != NULL) {
 		daemon->priv->ready = TRUE;
 
 		dbus_g_proxy_add_signal (DBUS_G_PROXY (proxy), "QuotaExceeded",
 					 dbus_g_type_get_map ("GHashTable",
-					                      G_TYPE_STRING,
-					                      G_TYPE_STRING),
+										  G_TYPE_STRING,
+										  G_TYPE_STRING),
 					 G_TYPE_INVALID);
 		dbus_g_proxy_connect_signal (DBUS_G_PROXY (proxy), "QuotaExceeded",
-					     G_CALLBACK (quota_exceeded_cb),
-					     daemon, NULL);
+						 G_CALLBACK (quota_exceeded_cb),
+						 daemon, NULL);
 
 		/* Now get all the interfaces so that we get the signals from them */
 		syncdaemon_daemon_get_config_interface (daemon);
@@ -365,10 +365,10 @@ setup_daemon_interface (SyncdaemonDaemon *daemon)
 
 static void
 name_owner_changed_cb (DBusGProxy *proxy,
-		       const char *name,
-		       const char *old_owner,
-		       const char *new_owner,
-		       gpointer user_data)
+			   const char *name,
+			   const char *old_owner,
+			   const char *new_owner,
+			   gpointer user_data)
 {
 	SyncdaemonDaemon *daemon = SYNCDAEMON_DAEMON (user_data);
 
@@ -402,7 +402,7 @@ syncdaemon_daemon_init (SyncdaemonDaemon *daemon)
 	daemon->priv->ready = FALSE;
 	daemon->priv->connected = FALSE;
 	daemon->priv->subinterfaces = g_hash_table_new_full (g_str_hash, g_str_equal,
-							     g_free, g_object_unref);
+								 g_free, g_object_unref);
 	daemon->priv->auth = g_object_new (SYNCDAEMON_TYPE_AUTHENTICATION, NULL);
 
 	/* Initialize DBus */
@@ -442,9 +442,9 @@ syncdaemon_daemon_init (SyncdaemonDaemon *daemon)
 				 G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
 				 G_TYPE_INVALID);
 	dbus_g_proxy_connect_signal (DBUS_G_PROXY (daemon->priv->dbus_proxy),
-				     "NameOwnerChanged",
-				     G_CALLBACK (name_owner_changed_cb),
-				     daemon, NULL);
+					 "NameOwnerChanged",
+					 G_CALLBACK (name_owner_changed_cb),
+					 daemon, NULL);
 }
 
 /**
@@ -502,8 +502,8 @@ syncdaemon_daemon_connect (SyncdaemonDaemon *daemon)
 #ifdef HAVE_GDBUS
 #else
 		if (!dbus_g_proxy_begin_call (DBUS_G_PROXY (proxy), "connect",
-					      connect_response_cb, daemon, NULL,
-					      G_TYPE_INVALID)) {
+						  connect_response_cb, daemon, NULL,
+						  G_TYPE_INVALID)) {
 			g_warning ("Call to 'connect' method failed");
 
 			return FALSE;
@@ -544,8 +544,8 @@ syncdaemon_daemon_disconnect (SyncdaemonDaemon *daemon)
 #ifdef HAVE_GDBUS
 #else
 		if (!dbus_g_proxy_begin_call (DBUS_G_PROXY (proxy), "disconnect",
-					      disconnect_response_cb, daemon, NULL,
-					      G_TYPE_INVALID)) {
+						  disconnect_response_cb, daemon, NULL,
+						  G_TYPE_INVALID)) {
 			g_warning ("Call to 'disconnect' method failed");
 
 			return FALSE;
@@ -850,8 +850,8 @@ syncdaemon_daemon_get_metadata (SyncdaemonDaemon *daemon, const gchar *path, gbo
 	interface = syncdaemon_daemon_get_filesystem_interface (daemon);
 	if (interface != NULL) {
 		return syncdaemon_filesystem_interface_get_metadata (SYNCDAEMON_FILESYSTEM_INTERFACE (interface),
-								     path,
-								     with_subtree_sync_check);
+									 path,
+									 with_subtree_sync_check);
 	}
 
 	return NULL;
@@ -920,7 +920,7 @@ gboolean
 syncdaemon_daemon_is_folder_enabled (SyncdaemonDaemon *daemon, const gchar *path, gboolean *is_root)
 {
 	const gchar *root;
-        gchar *root_with_slash;
+	gchar *root_with_slash;
 	gboolean managed = FALSE, is_subscribed = FALSE;
 	gchar *dirpath;
 	SyncdaemonInterface *interface;
@@ -959,18 +959,22 @@ syncdaemon_daemon_is_folder_enabled (SyncdaemonDaemon *daemon, const gchar *path
 		udfs = syncdaemon_folders_interface_get_folders (SYNCDAEMON_FOLDERS_INTERFACE (interface));
 		for (l = udfs; l != NULL; l = l->next) {
 			const gchar *tmp_path;
+			gchar *tmp_path_with_slash;
 			SyncdaemonFolderInfo *finfo = SYNCDAEMON_FOLDER_INFO (l->data);
 
 			tmp_path = syncdaemon_folder_info_get_path (finfo);
 			is_subscribed = syncdaemon_folder_info_get_subscribed (finfo);
+
+			tmp_path_with_slash = g_strconcat (tmp_path, G_DIR_SEPARATOR_S, NULL);
 			if (g_strcmp0 (path, tmp_path) == 0) {
 				managed = is_subscribed;
 				*is_root = TRUE;
 				break;
-			} else if (g_str_has_prefix (path, tmp_path)) {
+			} else if (g_str_has_prefix (path, tmp_path_with_slash)) {
 				managed = is_subscribed;
 				break;
 			}
+			g_free (tmp_path_with_slash);
 		}
 
 		g_slist_free (udfs);
