@@ -23,10 +23,10 @@ from contrib.testing.testcase import FakeMain, environ
 from tests.syncdaemon.test_vm import MetadataTestCase, BaseVolumeManagerTests
 from ubuntuone.storageprotocol import request
 from ubuntuone.syncdaemon.volume_manager import (
+    get_udf_path,
     LegacyShareFileShelf, _Share, Share, Shared, Root, UDF, _UDF,
     MetadataUpgrader, VMFileShelf,
 )
-from ubuntuone.platform.linux import get_udf_path, get_share_path
 
 
 class VolumesTests(BaseVolumeManagerTests):
@@ -41,31 +41,6 @@ class VolumesTests(BaseVolumeManagerTests):
                                        suggested_path.encode('utf-8')),
                           udf_path)
 
-    def test_get_share_path(self):
-        """Test for get_share_path."""
-        share_id = uuid.uuid4()
-        name = 'The little pretty share'
-        other_name = 'Dorian Grey'
-        share = self._create_share_volume(volume_id=share_id, name=name,
-                                          other_visible_name=other_name)
-        result = get_share_path(share)
-
-        expected = '%s (%s, %s)' % (name, other_name, share_id)
-        self.assertEqual(result, expected)
-
-    def test_get_share_path_visible_name_empty(self):
-        """Test for get_share_path."""
-        share_id = uuid.uuid4()
-        name = 'The little pretty share'
-        other_name = ''
-        share = self._create_share_volume(volume_id=share_id, name=name,
-                                          other_visible_name=other_name)
-        result = get_share_path(share)
-
-        expected = '%s (%s)' % (name, share_id)
-        self.assertEqual(result, expected)
-
-
 class MetadataOldLayoutTests(MetadataTestCase):
     """Tests for 'old' layouts and metadata upgrade"""
 
@@ -76,13 +51,6 @@ class MetadataOldLayoutTests(MetadataTestCase):
         self.new_root_dir = self.u1_dir
         self.new_shares_dir = self.mktemp('shares_dir')
 
-    def tearDown(self):
-        """Cleanup all the cruft."""
-        for path in [self.u1_dir, self.new_shares_dir]:
-            if path and os.path.exists(path):
-                self.rmtree(path)
-        MetadataTestCase.tearDown(self)
-
     def _build_layout_version_0(self):
         """Build the dir structure to mimic md v.0/None."""
         self.share_md_dir = os.path.join(self.tmpdir, 'data_dir', 'vm')
