@@ -178,6 +178,7 @@ class FakeMessaging(AbstractMessaging):
         self.messages_shown = {}
         self.messages_updated = {}
         self.callbacks = []
+
     # pylint: disable=R0913
     def show_message(self, sender, callback=None, message_time=None,
                      message_count=None, icon=None):
@@ -330,7 +331,6 @@ class FileDiscoveryBubbleTestCase(TestCase):
         aggregator.logger.setLevel(logging.DEBUG)
         self.addCleanup(aggregator.logger.removeHandler, self.handler)
 
-
     def get_notifications_shown(self):
         """The list of notifications shown."""
         return self.bubble.notification.notifications_shown
@@ -569,13 +569,13 @@ class ProgressBarTestCase(TestCase):
 
     def test_shown_when_progress_made(self):
         """The progress bar is shown when progress is made."""
-        self.bar.progress_made(50, 100)
+        self.bar.set_progress(0.5)
         self.assertTrue(self.bar.visible)
         self.assertTrue(self.bar.launcher.progress_visible)
 
     def test_progress_made_updates_counter(self):
         """Progress made updates the counter."""
-        self.bar.progress_made(50, 100)
+        self.bar.set_progress(0.5)
         self.assertEqual(self.bar.progress, 0.5)
 
     def test_no_timer_set_initially(self):
@@ -584,49 +584,49 @@ class ProgressBarTestCase(TestCase):
 
     def test_progress_made_sets_timer(self):
         """Progress made sets up a timer."""
-        self.bar.progress_made(50, 100)
+        self.bar.set_progress(0.5)
         self.assertNotEqual(self.bar.timer, None)
 
     def test_cleanup_resets_timer(self):
         """The cleanup method resets the timer."""
-        self.bar.progress_made(50, 100)
+        self.bar.set_progress(0.5)
         self.bar.cleanup()
         self.assertEqual(self.bar.timer, None)
 
     def test_progress_made_not_updated_initially(self):
         """Progress made is not updated initially."""
-        self.bar.progress_made(50, 100)
+        self.bar.set_progress(0.5)
         self.assertEqual(0, len(self.timeout_calls))
         self.assertEqual(0.0, self.bar.launcher.progress)
 
     def test_progress_made_updated_after_a_delay(self):
         """The progressbar is updated after a delay."""
-        self.bar.progress_made(50, 100)
+        self.bar.set_progress(0.5)
         self.clock.advance(aggregator.ProgressBar.updates_delay)
         self.assertIn(0.5, self.timeout_calls)
         self.assertEqual(0.5, self.bar.launcher.progress)
 
     def test_progress_updates_are_aggregated(self):
         """The progressbar is updated after a delay."""
-        self.bar.progress_made(50, 100)
+        self.bar.set_progress(0.5)
         self.clock.advance(aggregator.ProgressBar.updates_delay / 2)
-        self.bar.progress_made(60, 100)
+        self.bar.set_progress(0.6)
         self.clock.advance(aggregator.ProgressBar.updates_delay / 2)
         self.assertEqual(1, len(self.timeout_calls))
 
     def test_progress_updates_are_continuous(self):
         """The progressbar updates are continuous."""
-        self.bar.progress_made(50, 100)
+        self.bar.set_progress(0.5)
         self.clock.advance(aggregator.ProgressBar.updates_delay)
         self.assertEqual(0.5, self.bar.launcher.progress)
-        self.bar.progress_made(60, 100)
+        self.bar.set_progress(0.6)
         self.clock.advance(aggregator.ProgressBar.updates_delay)
         self.assertEqual(0.6, self.bar.launcher.progress)
         self.assertEqual(2, len(self.timeout_calls))
 
     def test_hidden_when_completed(self):
         """The progressbar is hidden when everything completes."""
-        self.bar.progress_made(50, 100)
+        self.bar.set_progress(0.5)
         self.bar.completed()
         self.assertFalse(self.bar.visible)
         self.assertFalse(self.bar.launcher.progress_visible)
@@ -634,7 +634,7 @@ class ProgressBarTestCase(TestCase):
     @defer.inlineCallbacks
     def test_progress_made_inhibits_logout_suspend(self):
         """Suspend and logout are inhibited when the progressbar is shown."""
-        self.bar.progress_made(50, 100)
+        self.bar.set_progress(0.5)
         expected = aggregator.session.INHIBIT_LOGOUT_SUSPEND
         inhibitor = yield self.bar.inhibitor_defer
         self.assertEqual(inhibitor.flags, expected)
@@ -642,7 +642,7 @@ class ProgressBarTestCase(TestCase):
     @defer.inlineCallbacks
     def test_completed_uninhibits_logout_suspend(self):
         """Suspend and logout are uninhibited when all has completed."""
-        self.bar.progress_made(50, 100)
+        self.bar.set_progress(0.5)
         d = self.bar.inhibitor_defer
         self.bar.completed()
         inhibitor = yield d
@@ -676,6 +676,9 @@ class FakeCommand(object):
 
     def __init__(self, path=''):
         self.path = path
+        self.share_id = path
+        self.node_id = path
+        self.deflated_size = 10000
 
 
 class FakeVolumeManager(object):
@@ -702,6 +705,7 @@ class FakeAggregator(object):
         self.clock = PatchedClock()
         self.files_uploading = []
         self.files_downloading = []
+        self.progress_events = []
 
     def queue_done(self):
         """The queue completed all operations."""
@@ -733,6 +737,11 @@ class FakeAggregator(object):
             self.files_uploading.remove(command)
         self.queued_commands.discard(command)
 
+    def progress_made(self, share_id, node_id, n_bytes_written, deflated_size):
+        """Progress made on up- or download."""
+        self.progress_events.append(
+            (share_id, node_id, n_bytes_written, deflated_size))
+
     def connection_made(self):
         """The client made the connection to the server."""
         self.connected = True
@@ -745,12 +754,10 @@ class FakeAggregator(object):
 class StatusFrontendTestCase(BaseTwistedTestCase):
     """Test the status frontend."""
 
-    def _fake_popunder(self):
-        self.popunders += 1
-
+    @defer.inlineCallbacks
     def setUp(self):
         """Initialize this test instance."""
-        BaseTwistedTestCase.setUp(self)
+        yield super(StatusFrontendTestCase, self).setUp()
         self.patch(aggregator, "StatusAggregator", FakeAggregator)
         self.patch(aggregator, "ToggleableNotification",
                    FakeNotificationSingleton())
@@ -772,11 +779,6 @@ class StatusFrontendTestCase(BaseTwistedTestCase):
                                                         is_public, public_url)
         self.assertEqual(
             1, len(self.status_frontend.notification.notifications_shown))
-        self.assertEqual(
-            (aggregator.UBUNTUONE_TITLE,
-             'A file was just made public at http://fake_public/url', None,
-             False),
-            self.status_frontend.notification.notifications_shown[0])
 
     def test_file_unpublished(self):
         """A file unpublished event is processed."""
@@ -789,10 +791,6 @@ class StatusFrontendTestCase(BaseTwistedTestCase):
                                                         is_public, public_url)
         self.assertEqual(
             1, len(self.status_frontend.notification.notifications_shown))
-        self.assertEqual(
-            (aggregator.UBUNTUONE_TITLE, 'A file is no longer published', None,
-             False),
-            self.status_frontend.notification.notifications_shown[0])
 
     def test_download_started(self):
         """A download was added to the queue."""
@@ -802,6 +800,15 @@ class StatusFrontendTestCase(BaseTwistedTestCase):
         qc = self.status_frontend.aggregator.queued_commands
         self.assertIn(fake_command, qc)
 
+    def test_download_started_with_no_deflated_size(self):
+        """A download of unknown size was added to the queue."""
+        self.patch(status_listener.action_queue, "Download", FakeCommand)
+        fake_command = FakeCommand()
+        fake_command.deflated_size = None
+        self.listener.handle_SYS_QUEUE_ADDED(fake_command)
+        qc = self.status_frontend.aggregator.queued_commands
+        self.assertIn(fake_command, qc)
+
     def test_download_finished(self):
         """A download was removed from the queue."""
         self.patch(status_listener.action_queue, "Download", FakeCommand)
@@ -812,15 +819,24 @@ class StatusFrontendTestCase(BaseTwistedTestCase):
         self.assertNotIn(fake_command, qc)
 
     def test_upload_started(self):
-        """A upload was added to the queue."""
+        """An upload was added to the queue."""
         self.patch(status_listener.action_queue, "Upload", FakeCommand)
         fake_command = FakeCommand()
         self.listener.handle_SYS_QUEUE_ADDED(fake_command)
         qc = self.status_frontend.aggregator.queued_commands
         self.assertIn(fake_command, qc)
 
+    def test_upload_started_with_no_deflated_size(self):
+        """An upload of unknown size was added to the queue."""
+        self.patch(status_listener.action_queue, "Upload", FakeCommand)
+        fake_command = FakeCommand()
+        fake_command.deflated_size = None
+        self.listener.handle_SYS_QUEUE_ADDED(fake_command)
+        qc = self.status_frontend.aggregator.queued_commands
+        self.assertIn(fake_command, qc)
+
     def test_upload_finished(self):
-        """A upload was removed from the queue."""
+        """An upload was removed from the queue."""
         self.patch(status_listener.action_queue, "Upload", FakeCommand)
         fake_command = FakeCommand()
         self.listener.handle_SYS_QUEUE_ADDED(fake_command)
@@ -828,6 +844,32 @@ class StatusFrontendTestCase(BaseTwistedTestCase):
         qc = self.status_frontend.aggregator.queued_commands
         self.assertNotIn(fake_command, qc)
 
+    def test_progress_made_on_upload(self):
+        """Progress was made on an uploading file."""
+        share_id = 'fake_share'
+        node_id = 'fake_node'
+        n_bytes_written = 100
+        deflated_size = 10000
+        self.listener.handle_AQ_UPLOAD_FILE_PROGRESS(
+            share_id, node_id, n_bytes_written, deflated_size)
+        pe = self.status_frontend.aggregator.progress_events
+        self.assertEqual(
+            [(share_id, node_id, n_bytes_written, deflated_size)], pe,
+            "progress_made was not called (exactly once) on aggregator.")
+
+    def test_progress_made_on_download(self):
+        """Progress was made on an downloading file."""
+        share_id = 'fake_share'
+        node_id = 'fake_node'
+        n_bytes_written = 200
+        deflated_size = 20000
+        self.listener.handle_AQ_DOWNLOAD_FILE_PROGRESS(
+            share_id, node_id, n_bytes_written, deflated_size)
+        pe = self.status_frontend.aggregator.progress_events
+        self.assertEqual(
+            [(share_id, node_id, n_bytes_written, deflated_size)], pe,
+            "progress_made was not called (exactly once) on aggregator.")
+
     def test_queue_done(self):
         """The queue is empty."""
         fake_command = FakeCommand()
@@ -845,11 +887,6 @@ class StatusFrontendTestCase(BaseTwistedTestCase):
         self.listener.handle_VM_SHARE_CREATED(SHARE_ID)
         self.assertEqual(
             1, len(self.status_frontend.notification.notifications_shown))
-        self.assertEqual(
-            (aggregator.UBUNTUONE_TITLE,
-             "New cloud folder available: '%s' shared by %s" % (
-                 'None', FAKE_SENDER), None, False),
-            self.status_frontend.notification.notifications_shown[0])
         msg = self.status_frontend.messaging.messages_shown[FAKE_SENDER]
         # msg did not receive a time argument
         self.assertEqual(None, msg[1])
@@ -875,22 +912,10 @@ class StatusFrontendTestCase(BaseTwistedTestCase):
         self.assertEqual(
             1, len(self.status_frontend.notification.notifications_shown))
         self.assertEqual(
-            (aggregator.UBUNTUONE_TITLE, "New cloud folder available: 'None'",
-             None, False),
-            self.status_frontend.notification.notifications_shown[0])
-        self.assertEqual(
-            1, len(self.status_frontend.messaging.messages_shown))
+            0, len(self.status_frontend.messaging.messages_shown))
         self.assertEqual(
             0, len(self.status_frontend.messaging.messages_updated))
-        msg = self.status_frontend.messaging.messages_shown[
-            aggregator.NEW_UDFS_SENDER]
-        # msg did not receive a time argument
-        self.assertEqual(None, msg[1])
-        # msg did receive a count argument
-        self.assertEqual(1, msg[2])
-        # call callback
-        msg[0]('fake_indicator')
-        self.assertEqual(1, len(self.status_frontend.messaging.callbacks))
+        self.assertEqual(0, len(self.status_frontend.messaging.callbacks))
 
     def test_two_new_udfs_available(self):
         """A new udf is available for subscription."""
@@ -901,27 +926,9 @@ class StatusFrontendTestCase(BaseTwistedTestCase):
         self.assertEqual(
             2, len(self.status_frontend.notification.notifications_shown))
         self.assertEqual(
-            (aggregator.UBUNTUONE_TITLE, "New cloud folder available: 'None'",
-             None, False),
-            self.status_frontend.notification.notifications_shown[0])
+            0, len(self.status_frontend.messaging.messages_shown))
         self.assertEqual(
-            (aggregator.UBUNTUONE_TITLE, "New cloud folder available: 'None'",
-             None, False),
-            self.status_frontend.notification.notifications_shown[1])
-        self.assertEqual(
-            1, len(self.status_frontend.messaging.messages_shown))
-        self.assertEqual(
-            1, len(self.status_frontend.messaging.messages_updated))
-        msg = self.status_frontend.messaging.messages_shown[
-            aggregator.NEW_UDFS_SENDER]
-        # msg did not receive a time argument
-        self.assertEqual(None, msg[1])
-        # msg did receive a count argument
-        self.assertEqual(1, msg[2])
-        self.assertEqual(
-            (aggregator.NEW_UDFS_SENDER, 1),
-            self.status_frontend.messaging.messages_updated[
-                aggregator.NEW_UDFS_SENDER])
+            0, len(self.status_frontend.messaging.messages_updated))
 
     def test_server_connection_lost(self):
         """The client connected to the server."""
@@ -949,7 +956,7 @@ class StatusFrontendTestCase(BaseTwistedTestCase):
         """Quota exceeded in udf."""
         mocker = Mocker()
         launcher = mocker.replace(
-            "ubuntuone.platform.unity.UbuntuOneLauncher")
+            "ubuntuone.platform.launcher.UbuntuOneLauncher")
         launcher()
         mock_launcher = mocker.mock()
         mocker.result(mock_launcher)
@@ -969,7 +976,7 @@ class StatusFrontendTestCase(BaseTwistedTestCase):
         """Quota exceeded in root."""
         mocker = Mocker()
         launcher = mocker.replace(
-            "ubuntuone.platform.unity.UbuntuOneLauncher")
+            "ubuntuone.platform.launcher.UbuntuOneLauncher")
         launcher()
         mock_launcher = mocker.mock()
         mocker.result(mock_launcher)
@@ -990,7 +997,7 @@ class StatusFrontendTestCase(BaseTwistedTestCase):
         """Quota exceeded in share."""
         mocker = Mocker()
         launcher = mocker.replace(
-            "ubuntuone.platform.unity.UbuntuOneLauncher")
+            "ubuntuone.platform.launcher.UbuntuOneLauncher")
         launcher()
         mock_launcher = mocker.mock()
         mocker.result(mock_launcher)
@@ -1194,27 +1201,27 @@ class StatusAggregatorTestCase(TestCase):
 
     def assertStatusReset(self):
         """Assert that the status is at zero."""
-        self.assertEqual(self.aggregator.download_total, 0)
-        self.assertEqual(self.aggregator.download_done, 0)
-        self.assertEqual(self.aggregator.upload_total, 0)
-        self.assertEqual(self.aggregator.upload_done, 0)
-        self.assertEqual(self.aggregator.total_counter, 0)
-        self.assertEqual(self.aggregator.done_counter, 0)
-        self.assertEqual(len(self.aggregator.files_uploading), 0)
-        self.assertEqual(len(self.aggregator.files_downloading), 0)
-        self.assertIdentical(self.aggregator.queue_done_timer, None)
+        self.assertEqual(0, self.aggregator.download_done)
+        self.assertEqual(0, self.aggregator.upload_done)
+        self.assertEqual(0, len(self.aggregator.files_uploading))
+        self.assertEqual(0, len(self.aggregator.files_downloading))
+        self.assertEqual({}, self.aggregator.progress)
+        self.assertEqual({}, self.aggregator.to_do)
+        self.assertIdentical(None, self.aggregator.queue_done_timer)
 
     def assertMiscCommandQueued(self, fc):
         """Assert that some command was queued."""
-        self.assertEqual(self.aggregator.total_counter, 1)
-        message = "queueing command (0/1): %s" % fc.__class__.__name__
+        self.assertEqual(len(self.aggregator.to_do), 1)
+        message = "queueing command (total: 1): %s" % fc.__class__.__name__
+        self.assertEqual(fc.deflated_size, sum(self.aggregator.to_do.values()))
         self.assertTrue(self.handler.check_debug(message))
         self.assertTrue(self.aggregator.progress_bar.visible)
 
     def assertMiscCommandUnqueued(self, fc):
         """Assert that some command was unqueued."""
-        self.assertEqual(self.aggregator.done_counter, 1)
-        message = "unqueueing command (1/1): %s" % fc.__class__.__name__
+        self.assertEqual(
+            1, self.aggregator.download_done + self.aggregator.upload_done)
+        message = "unqueueing command: %s" % fc.__class__.__name__
         self.assertTrue(self.handler.check_debug(message))
 
     def test_counters_start_at_zero(self):
@@ -1226,10 +1233,13 @@ class StatusAggregatorTestCase(TestCase):
         fc = FakeCommand(path='testfile.txt')
         self.assertEqual('', self.aggregator.downloading_filename)
         self.status_frontend.download_started(fc)
-        self.assertEqual(1, self.aggregator.download_total)
+        self.assertEqual(1, len(self.aggregator.files_downloading))
         self.assertEqual('testfile.txt', self.aggregator.downloading_filename)
         self.assertMiscCommandQueued(fc)
         self.assertEqual(1, self.fake_bubble.count)
+        self.assertEqual(
+            {(fc.share_id, fc.node_id): (fc.deflated_size)},
+            self.aggregator.to_do)
 
     def test_file_download_finished(self):
         """Test that a file has finished downloading."""
@@ -1238,16 +1248,22 @@ class StatusAggregatorTestCase(TestCase):
         self.status_frontend.download_finished(fc)
         self.assertEqual(self.aggregator.download_done, 1)
         self.assertMiscCommandUnqueued(fc)
+        self.assertEqual(
+            {(fc.share_id, fc.node_id): (fc.deflated_size)},
+            self.aggregator.progress)
 
     def test_file_upload_started(self):
         """Test that a file has started upload."""
         fc = FakeCommand(path='testfile.txt')
         self.assertEqual('', self.aggregator.uploading_filename)
         self.status_frontend.upload_started(fc)
-        self.assertEqual(1, self.aggregator.upload_total)
+        self.assertEqual(1, len(self.aggregator.files_uploading))
         self.assertEqual('testfile.txt', self.aggregator.uploading_filename)
         self.assertMiscCommandQueued(fc)
         self.assertEqual(1, self.fake_bubble.count)
+        self.assertEqual(
+            {(fc.share_id, fc.node_id): (fc.deflated_size)},
+            self.aggregator.to_do)
 
     def test_file_upload_finished(self):
         """Test that a file has finished uploading."""
@@ -1256,6 +1272,21 @@ class StatusAggregatorTestCase(TestCase):
         self.status_frontend.upload_finished(fc)
         self.assertEqual(self.aggregator.upload_done, 1)
         self.assertMiscCommandUnqueued(fc)
+        self.assertEqual(
+            {(fc.share_id, fc.node_id): (fc.deflated_size)},
+            self.aggregator.progress)
+
+    def test_progress_made(self):
+        """Progress on up and downloads is tracked."""
+        share_id = 'fake_share'
+        node_id = 'fake_node'
+        n_bytes_written = 200
+        deflated_size = 100000
+        self.aggregator.progress_made(
+            share_id, node_id, n_bytes_written, deflated_size)
+        self.assertEqual(
+            {(share_id, node_id): (n_bytes_written)},
+            self.aggregator.progress)
 
     def test_get_discovery_message(self):
         """Test the message that's shown on the discovery bubble."""
@@ -1464,7 +1495,7 @@ class HundredFeetTestCase(TestCase):
         clock.advance(aggregator.FileDiscoveryUpdateState.updates_timeout -
                       aggregator.FileDiscoveryUpdateState.updates_delay)
         sf.upload_finished(upload)
-        sf.upload_finished(download)
+        sf.download_finished(download)
         # the progress still is now 100%
         self.assertEqual(1.0, sf.aggregator.progress_bar.progress)
         sf.queue_done()
@@ -1478,7 +1509,7 @@ class HundredFeetTestCase(TestCase):
         self.patch(aggregator, "UbuntuOneLauncher", FakeLauncher)
         self.patch(aggregator.session, "Inhibitor", FakeInhibitor)
         clock = PatchedClock()
-        upload = FakeCommand()
+        upload = FakeCommand('upload.foo')
         sf = aggregator.StatusFrontend(clock=clock)
         sf.set_show_all_notifications(False)
 
@@ -1493,7 +1524,7 @@ class HundredFeetTestCase(TestCase):
         self.assertEqual(0, len(notifications_shown))
         clock.advance(aggregator.FileDiscoveryGatheringState.initial_delay)
         self.assertEqual(0, len(notifications_shown))
-        download = FakeCommand()
+        download = FakeCommand('download.bar')
         sf.download_started(download)
         self.assertEqual(0, len(notifications_shown))
         # the progress still is zero
@@ -1503,7 +1534,7 @@ class HundredFeetTestCase(TestCase):
         clock.advance(aggregator.FileDiscoveryUpdateState.updates_timeout -
                       aggregator.FileDiscoveryUpdateState.updates_delay)
         sf.upload_finished(upload)
-        sf.upload_finished(download)
+        sf.download_finished(download)
         # the progress still is now 100%
         self.assertEqual(1.0, sf.aggregator.progress_bar.progress)
         self.assertEqual(0, len(notifications_shown))
