@@ -20,17 +20,18 @@ from __future__ import with_statement
 
 import inspect
 import os
-import shutil
 import unittest
 
 from twisted.internet import defer, reactor
 
-from contrib.testing import testcase
+from contrib.testing.testcase import (
+    BaseTwistedTestCase,
+    FakeVolumeManager,
+    skip_if_win32_missing_fs_event,
+)
 from ubuntuone.syncdaemon import (filesystem_manager, event_queue,
                                   events_nanny, hash_queue, tritcask)
 
-TESTS_DIR = os.path.join(os.getcwd(), "tmp")
-
 
 class EventListener(object):
     """Store the events."""
@@ -51,7 +52,7 @@ class EventListener(object):
 def hash_hack_run(self):
     """Function to hack the hash class to avoid it processing the requests."""
     while True:
-        info = self.queue.get()
+        info, timestamp = self.queue.get()
         try:
             path, mdid = info
         except TypeError:
@@ -59,28 +60,23 @@ def hash_hack_run(self):
         self.hashing = path
 
 
-class DownloadFinishedTests(testcase.BaseTwistedTestCase):
+class DownloadFinishedTests(BaseTwistedTestCase):
     """Test the AQ Download Finished Nanny behaviour."""
     timeout = 2
 
+    @defer.inlineCallbacks
     def setUp(self):
         """set up the test."""
-        testcase.BaseTwistedTestCase.setUp(self)
-        try:
-            os.mkdir(TESTS_DIR)
-        except OSError:
-            # already there, remove it to clean and create again
-            shutil.rmtree(TESTS_DIR)
-            os.mkdir(TESTS_DIR)
-        self.usrdir = os.path.join(TESTS_DIR, "usrdir")
-        self.partials_dir = os.path.join(TESTS_DIR, "partials")
+        yield super(DownloadFinishedTests, self).setUp()
+        self.usrdir = self.mktemp("usrdir")
+        self.partials_dir = self.mktemp("partials")
 
         # hack hash queue
         self._original_hash_run = hash_queue._Hasher.run
         hash_queue._Hasher.run = hash_hack_run
 
         # create vm, fsm, eq, hq...
-        vm = testcase.FakeVolumeManager(self.usrdir)
+        vm = FakeVolumeManager(self.usrdir)
         db = tritcask.Tritcask(self.mktemp('tritcask'))
         self.addCleanup(db.shutdown)
         self.fsm = fsm = filesystem_manager.FileSystemManager(
@@ -97,11 +93,12 @@ class DownloadFinishedTests(testcase.BaseTwistedTestCase):
         fsm.create(self.tf, "")
         fsm.set_node_id(self.tf, "nodeid")
 
+    @defer.inlineCallbacks
     def tearDown(self):
         """tear down the test."""
         hash_queue._Hasher.run = self._original_hash_run
         self.hq.shutdown()
-        testcase.BaseTwistedTestCase.tearDown(self)
+        yield super(DownloadFinishedTests, self).tearDown()
 
     def insert_in_hq(self, path, node_id):
         """Inserts something in HQ and waits that thread."""
@@ -610,6 +607,7 @@ class DownloadFinishedTests(testcase.BaseTwistedTestCase):
         d.callback(None)
         return d
 
+    @skip_if_win32_missing_fs_event
     def test_dir_move_changeblock(self):
         """Move event affects the blocked stuff."""
         # create several files with tricky names
@@ -684,6 +682,7 @@ class DownloadFinishedTests(testcase.BaseTwistedTestCase):
         d.callback(None)
         return d
 
+    @skip_if_win32_missing_fs_event
     def test_complex(self):
         """Open, Open, Close, MoveFile, Open, MoveDir, Close, Close... test!"""
         # create a file inside a directory
@@ -862,11 +861,3 @@ class EventListenersTests(unittest.TestCase):
                 defined_args = inspect.getargspec(meth)[0]
                 self.assertEqual(defined_args[0], 'self')
                 self.assertEqual(set(defined_args[1:]), set(evtargs))
-
-
-def test_suite():
-    # pylint: disable-msg=C0111
-    return unittest.TestLoader().loadTestsFromName(__name__)
-
-if __name__ == "__main__":
-    unittest.main()
