@@ -22,35 +22,37 @@ import cPickle
 import os
 import hashlib
 import unittest
-import shutil
 
+from twisted.internet import defer
+
+from ubuntuone.devtools.testcase import skipIfOS
+from ubuntuone.platform import (
+    open_file,
+    path_exists,
+)
 from ubuntuone.syncdaemon.file_shelf import (
     FileShelf,
     CachedFileShelf,
     LRUCache,
     CacheInconsistencyError,
 )
-from contrib.testing import testcase
+from contrib.testing.testcase import BaseTwistedTestCase
 
 
 BROKEN_PICKLE = '\axb80\x02}q\x01(U\x01aU\x04testq\x02U\x01bU\x06brokenq\x03u.'
 
 
-class TestFileShelf(testcase.BaseTwistedTestCase):
+class TestFileShelf(BaseTwistedTestCase):
     """ Test the FileShelf """
     fileshelf_class = FileShelf
 
+    @defer.inlineCallbacks
     def setUp(self):
         """ Sets up a test. """
-        testcase.BaseTwistedTestCase.setUp(self)
+        yield super(TestFileShelf, self).setUp()
         self.path = self.mktemp('shelf')
         self.shelf = self.fileshelf_class(self.path)
 
-    def tearDown(self):
-        """Clean up the tests."""
-        shutil.rmtree(self.path)
-        testcase.BaseTwistedTestCase.tearDown(self)
-
     def test_bad_depth(self):
         """ test that the shelf reject invalid depth at creation time """
         self.assertRaises(ValueError, self.fileshelf_class, self.path, depth=-1)
@@ -60,7 +62,7 @@ class TestFileShelf(testcase.BaseTwistedTestCase):
         directory for the new file based shelf at creation time.
         """
         path = os.path.join(self.path, 'shelf_file')
-        open(path, 'w').close()
+        open_file(path, 'w').close()
         self.fileshelf_class(path)
         self.assertTrue(os.path.isdir(path))
 
@@ -77,13 +79,13 @@ class TestFileShelf(testcase.BaseTwistedTestCase):
             # test __setitem__
             shelf[key] = 'foo'
             key_path = os.path.join(path, *[key[i] for i in xrange(0, idx)])
-            self.assertTrue(os.path.exists(os.path.join(key_path, key)))
+            self.assertTrue(path_exists(os.path.join(key_path, key)))
             # test __getitem__
             self.assertEquals('foo', shelf[key])
             # test __delitem__
             del shelf[key]
             self.assertRaises(KeyError, shelf.__getitem__, key)
-            self.assertFalse(os.path.exists(os.path.join(key_path, key)))
+            self.assertFalse(path_exists(os.path.join(key_path, key)))
 
     def test_invalid_keys(self):
         """ test the exception raised when invalid keys are eused ('', None)"""
@@ -139,12 +141,12 @@ class TestFileShelf(testcase.BaseTwistedTestCase):
         """
         self.shelf['bad_file'] = {}
         path = self.shelf.key_file('bad_file')
-        open(path, 'w').close()
+        open_file(path, 'w').close()
         self.assertRaises(KeyError, self.shelf.__getitem__, 'bad_file')
 
         self.shelf['broken_pickle'] = {}
         path = self.shelf.key_file('broken_pickle')
-        with open(path, 'w') as f:
+        with open_file(path, 'w') as f:
             f.write(BROKEN_PICKLE)
         self.assertRaises(KeyError, self.shelf.__getitem__, 'broken_pickle')
 
@@ -152,30 +154,30 @@ class TestFileShelf(testcase.BaseTwistedTestCase):
         """test that each time a metadata file is updated a .old is kept"""
         self.shelf['bad_file'] = {'value':'old'}
         path = self.shelf.key_file('bad_file')
-        self.assertFalse(os.path.exists(path+'.old'))
+        self.assertFalse(path_exists(path+'.old'))
         self.assertEquals({'value':'old'}, self.shelf['bad_file'])
         # force the creation of the .old file
         self.shelf['bad_file'] = {'value':'new'}
-        self.assertTrue(os.path.exists(path+'.old'))
+        self.assertTrue(path_exists(path+'.old'))
         # check that the new value is there
         self.assertEquals({'value':'new'}, self.shelf['bad_file'])
         # write the current md file fwith 0 bytes
-        open(path, 'w').close()
+        open_file(path, 'w').close()
         # test that the old value is retrieved
         self.assertEquals({'value':'old'}, self.shelf['bad_file'])
 
         self.shelf['broken_pickle'] = {'value':'old'}
         path = self.shelf.key_file('broken_pickle')
         # check that .old don't exist
-        self.assertFalse(os.path.exists(path+'.old'))
+        self.assertFalse(path_exists(path+'.old'))
         # force the creation of the .old file
         self.shelf['broken_pickle'] = {'value':'new'}
         # check that .old exists
-        self.assertTrue(os.path.exists(path+'.old'))
+        self.assertTrue(path_exists(path+'.old'))
         # check that the new value is there
         self.assertEquals({'value':'new'}, self.shelf['broken_pickle'])
         # write random bytes to the md file
-        with open(path, 'w') as f:
+        with open_file(path, 'w') as f:
             f.write(BROKEN_PICKLE)
         # check that the old value is retrieved
         self.assertEquals({'value':'old'}, self.shelf['broken_pickle'])
@@ -184,10 +186,10 @@ class TestFileShelf(testcase.BaseTwistedTestCase):
         """test keys() with .old and .new files around"""
         self.shelf["foo"] = "bar"
         self.shelf["foo1"] = "bar1"
-        open(self.shelf.key_file('foo')+'.old', 'w').close()
-        open(self.shelf.key_file('foo1')+'.old', 'w').close()
-        open(self.shelf.key_file('foo')+'.new', 'w').close()
-        open(self.shelf.key_file('foo1')+'.new', 'w').close()
+        open_file(self.shelf.key_file('foo')+'.old', 'w').close()
+        open_file(self.shelf.key_file('foo1')+'.old', 'w').close()
+        open_file(self.shelf.key_file('foo')+'.new', 'w').close()
+        open_file(self.shelf.key_file('foo1')+'.new', 'w').close()
         self.assertEquals(set(['foo', 'foo1']), set(self.shelf.keys()))
 
     def test_corrupted_backup(self):
@@ -196,17 +198,17 @@ class TestFileShelf(testcase.BaseTwistedTestCase):
         # create the .old backup
         self.shelf["foo"] = "bar1"
         # write 0 bytes to both
-        open(self.shelf.key_file('foo')+'.old', 'w').close()
-        open(self.shelf.key_file('foo'), 'w').close()
+        open_file(self.shelf.key_file('foo')+'.old', 'w').close()
+        open_file(self.shelf.key_file('foo'), 'w').close()
         self.assertRaises(KeyError, self.shelf.__getitem__, 'foo')
 
     def test_endless_borken_backups(self):
         """test getitem  with a lot of files named .old.old.....old"""
         self.shelf["foo"] = "bar"
         path = self.shelf.key_file('foo')
-        open(self.shelf.key_file('foo'), 'w').close()
+        open_file(self.shelf.key_file('foo'), 'w').close()
         for _ in xrange(20):
-            open(path+'.old', 'w').close()
+            open_file(path+'.old', 'w').close()
             path=path+'.old'
         self.assertRaises(KeyError, self.shelf.__getitem__, 'foo')
 
@@ -217,13 +219,14 @@ class TestFileShelf(testcase.BaseTwistedTestCase):
         self.shelf["foo"] = "bar1"
         path = self.shelf.key_file('foo')
         # create a .new file (a hard reboot during the rename dance)
-        open(path+'.new', 'w').close()
+        open_file(path+'.new', 'w').close()
         # write 0 bytes to both
         del self.shelf['foo']
-        self.assertFalse(os.path.exists(path))
-        self.assertFalse(os.path.exists(path+'.old'), 'there is a .old file!')
-        self.assertFalse(os.path.exists(path+'.new'), 'there is a .new file!')
+        self.assertFalse(path_exists(path))
+        self.assertFalse(path_exists(path+'.old'), 'there is a .old file!')
+        self.assertFalse(path_exists(path+'.new'), 'there is a .new file!')
 
+    @skipIfOS('win32', 'Skipped because code is deprecated on Windows.')
     def test_custom_unpickle(self):
         """Test the _pickle and _unpikle methods."""
         self.mktemp('my_shelf')
@@ -259,36 +262,36 @@ class TestFileShelf(testcase.BaseTwistedTestCase):
         self.shelf['ok_key'] = {'status':'this is valid metadata'}
         self.shelf['bad_file'] = {}
         path = self.shelf.key_file('bad_file')
-        open(path, 'w').close()
+        open_file(path, 'w').close()
         self.assertRaises(KeyError, self.shelf.__getitem__, 'bad_file')
         self.assertEquals(1, len(list(self.shelf.iteritems())))
-        self.assertFalse(os.path.exists(path))
+        self.assertFalse(path_exists(path))
 
         self.shelf['broken_pickle'] = {}
         path = self.shelf.key_file('broken_pickle')
-        with open(path, 'w') as f:
+        with open_file(path, 'w') as f:
             f.write(BROKEN_PICKLE)
         self.assertRaises(KeyError, self.shelf.__getitem__, 'broken_pickle')
         self.assertEquals(1, len(list(self.shelf.iteritems())))
-        self.assertFalse(os.path.exists(path))
+        self.assertFalse(path_exists(path))
 
     def test_broken_metadata_items(self):
         """Test that broken metadata is ignored during iteritems."""
         self.shelf['ok_key'] = {'status':'this is valid metadata'}
         self.shelf['bad_file'] = {}
         path = self.shelf.key_file('bad_file')
-        open(path, 'w').close()
+        open_file(path, 'w').close()
         self.assertRaises(KeyError, self.shelf.__getitem__, 'bad_file')
         self.assertEquals(1, len(list(self.shelf.items())))
-        self.assertFalse(os.path.exists(path))
+        self.assertFalse(path_exists(path))
 
         self.shelf['broken_pickle'] = {}
         path = self.shelf.key_file('broken_pickle')
-        with open(path, 'w') as f:
+        with open_file(path, 'w') as f:
             f.write(BROKEN_PICKLE)
         self.assertRaises(KeyError, self.shelf.__getitem__, 'broken_pickle')
         self.assertEquals(1, len(list(self.shelf.items())))
-        self.assertFalse(os.path.exists(path))
+        self.assertFalse(path_exists(path))
 
 
 class CachedFileShelfTests(TestFileShelf):
@@ -313,15 +316,15 @@ class CachedFileShelfTests(TestFileShelf):
         """overrides parent test as we have the value in the cache."""
         self.shelf['bad_file'] = {'value':'old'}
         path = self.shelf.key_file('bad_file')
-        self.assertFalse(os.path.exists(path+'.old'))
+        self.assertFalse(path_exists(path+'.old'))
         self.assertEquals({'value':'old'}, self.shelf['bad_file'])
         # force the creation of the .old file
         self.shelf['bad_file'] = {'value':'new'}
-        self.assertTrue(os.path.exists(path+'.old'))
+        self.assertTrue(path_exists(path+'.old'))
         # check that the new value is there
         self.assertEquals({'value':'new'}, self.shelf['bad_file'])
         # write the current md file fwith 0 bytes
-        open(path, 'w').close()
+        open_file(path, 'w').close()
         # HERE IS THE DIFFERENCE with the parent tests
         # test that the new value is retrieved from the cache!
         self.assertEquals({'value':'new'}, self.shelf['bad_file'])
@@ -329,15 +332,15 @@ class CachedFileShelfTests(TestFileShelf):
         self.shelf['broken_pickle'] = {'value':'old'}
         path = self.shelf.key_file('broken_pickle')
         # check that .old don't exist
-        self.assertFalse(os.path.exists(path+'.old'))
+        self.assertFalse(path_exists(path+'.old'))
         # force the creation of the .old file
         self.shelf['broken_pickle'] = {'value':'new'}
         # check that .old exists
-        self.assertTrue(os.path.exists(path+'.old'))
+        self.assertTrue(path_exists(path+'.old'))
         # check that the new value is there
         self.assertEquals({'value':'new'}, self.shelf['broken_pickle'])
         # write random bytes to the md file
-        with open(path, 'w') as f:
+        with open_file(path, 'w') as f:
             f.write(BROKEN_PICKLE)
         # HERE IS THE DIFFERENCE with the parent tests
         # test that the new value is retrieved from the cache!
@@ -467,12 +470,3 @@ class LRUCacheTests(unittest.TestCase):
             cache['key2']
         except CacheInconsistencyError, e:
             self.fail(e)
-
-
-def test_suite():
-    # pylint: disable-msg=C0111
-    return unittest.TestLoader().loadTestsFromName(__name__)
-
-if __name__ == "__main__":
-    unittest.main()
-
