@@ -19,19 +19,23 @@
 
 import logging
 import os
-import shutil
 
 from twisted.internet import defer, reactor
-
-from ubuntuone.syncdaemon import main as main_mod
-from ubuntuone.clientdefs import VERSION
+from ubuntuone.devtools.handlers import MementoHandler
 
 from contrib.testing.testcase import (
     BaseTwistedTestCase, FAKED_CREDENTIALS,
 )
 from tests.platform import setup_main_test, get_main_params
-
-from ubuntuone.devtools.handlers import MementoHandler
+from ubuntuone.clientdefs import VERSION
+from ubuntuone.platform import (
+    is_link,
+    make_dir,
+    make_link,
+    path_exists,
+    remove_dir,
+)
+from ubuntuone.syncdaemon import main as main_mod
 
 
 def _get_main_common_params(testcase):
@@ -54,12 +58,22 @@ class FakeListener(object):
         """Something! :)"""
 
 
+class FakedExternalInterface(object):
+    """Do nothing."""
+
+    def __init__(self, *a, **kw):
+        self._called = []
+        self.connect = lambda *a, **kw: self._called.append(('connect', a, kw))
+        self.shutdown = lambda *a, **kw: None
+
+
 class MainTests(BaseTwistedTestCase):
     """ Basic tests to check main.Main """
 
+    @defer.inlineCallbacks
     def setUp(self):
         """ Sets up a test. """
-        BaseTwistedTestCase.setUp(self)
+        yield super(MainTests, self).setUp()
         self.root = self.mktemp('root')
         self.shares = self.mktemp('shares')
         self.data = self.mktemp('data')
@@ -67,41 +81,42 @@ class MainTests(BaseTwistedTestCase):
         # perform the extra setup steps according to the platform
         setup_main_test(self)
 
+        self.patch(main_mod, 'ExternalInterface', FakedExternalInterface)
+        # no event logger by default
+        self.patch(main_mod.event_logging, "get_listener", lambda *a: None)
+        # no status listener by default
+        self.patch(main_mod.status_listener, "get_listener", lambda *a: None)
+
         self.handler = MementoHandler()
         self.handler.setLevel(logging.DEBUG)
         self._logger = logging.getLogger('ubuntuone.SyncDaemon')
         self._logger.addHandler(self.handler)
+        self.addCleanup(self._logger.removeHandler, self.handler)
 
     def build_main(self, **kwargs):
-        """
-        Build and return a Main object, using reasonable defaults for
-        the tests, plus whatever extra kwargs are passed in.
+        """Build and return a Main object.
+
+        Use reasonable defaults for the tests, plus whatever extra kwargs are
+        passed in.
+
         """
         # get the params using the platform code to ensure they are correct
-        params =  get_main_params(self, _get_main_common_params(self))
+        params = get_main_params(self, _get_main_common_params(self))
         params.update(kwargs)
         m = main_mod.Main(**params)
+        self.addCleanup(m.shutdown)
         m.local_rescan = lambda *_: m.event_q.push('SYS_LOCAL_RESCAN_DONE')
         return m
 
-    def tearDown(self):
-        """Clean up the tests."""
-        self._logger.removeHandler(self.handler)
-        shutil.rmtree(self.root)
-        shutil.rmtree(self.shares)
-        shutil.rmtree(self.data)
-        BaseTwistedTestCase.tearDown(self)
-
     def test_main_initialization(self):
         """test that creating a Main instance works as expected."""
         main = self.build_main()
-        main.shutdown()
+        self.assertIsInstance(main, main_mod.Main)
 
     def test_main_start(self):
         """Test that Main.start works."""
         main = self.build_main()
         main.start()
-        main.shutdown()
 
     def test_main_restarts_on_critical_error(self):
         """Test that Main restarts when syncdaemon gets into UNKNOWN_ERROR."""
@@ -111,16 +126,17 @@ class MainTests(BaseTwistedTestCase):
         main.start()
         main.event_q.push('SYS_UNKNOWN_ERROR')
         self.assertTrue(self.restarted)
-        main.shutdown()
 
+    @defer.inlineCallbacks
     def test_shutdown_pushes_sys_quit(self):
         """When shutting down, the SYS_QUIT event is pushed."""
-        main = self.build_main()
+        params = get_main_params(self, _get_main_common_params(self))
+        main = main_mod.Main(**params)
         events = []
         self.patch(main.event_q, 'push',
                    lambda *a, **kw: events.append((a, kw)))
 
-        main.shutdown()
+        yield main.shutdown()
         expected = [(('SYS_USER_DISCONNECT',), {}), (('SYS_QUIT',), {})]
         self.assertEqual(expected, events)
 
@@ -150,95 +166,74 @@ class MainTests(BaseTwistedTestCase):
         main.start()
         main.event_q.push('SYS_NET_CONNECTED')
         main.event_q.push('SYS_USER_CONNECT', access_token='')
-        d0.addCallback(lambda _: main.shutdown())
         return d0
 
     def test_create_dirs_already_exists_dirs(self):
         """test that creating a Main instance works as expected."""
         link = os.path.join(self.root, 'Shared With Me')
-        self.assertFalse(os.path.exists(link))
-        self.assertTrue(os.path.exists(self.shares))
-        self.assertTrue(os.path.exists(self.root))
+        self.assertFalse(is_link(link))
+        self.assertTrue(path_exists(self.shares))
+        self.assertTrue(path_exists(self.root))
         main = self.build_main()
-        self.assertTrue(os.path.exists(main.shares_dir_link))
         # check that the shares link is actually a link
-        self.assertTrue(os.path.islink(main.shares_dir_link))
+        self.assertTrue(is_link(main.shares_dir_link))
         self.assertEquals(link, main.shares_dir_link)
-        main.shutdown()
 
     def test_create_dirs_already_exists_symlink_too(self):
         """test that creating a Main instance works as expected."""
         link = os.path.join(self.root, 'Shared With Me')
-        os.symlink(self.shares, link)
-        self.assertTrue(os.path.exists(link))
-        self.assertTrue(os.path.islink(link))
-        self.assertTrue(os.path.exists(self.shares))
-        self.assertTrue(os.path.exists(self.root))
+        make_link(self.shares, link)
+        self.assertTrue(is_link(link))
+        self.assertTrue(path_exists(self.shares))
+        self.assertTrue(path_exists(self.root))
         main = self.build_main()
         # check that the shares link is actually a link
-        self.assertTrue(os.path.islink(main.shares_dir_link))
-        main.shutdown()
+        self.assertTrue(is_link(main.shares_dir_link))
 
     def test_create_dirs_already_exists_but_not_symlink(self):
         """test that creating a Main instance works as expected."""
         link = os.path.join(self.root, 'Shared With Me')
-        os.makedirs(link)
-        self.assertTrue(os.path.exists(link))
-        self.assertFalse(os.path.islink(link))
-        self.assertTrue(os.path.exists(self.shares))
-        self.assertTrue(os.path.exists(self.root))
+        make_dir(link, recursive=True)
+        self.assertTrue(path_exists(link))
+        self.assertFalse(is_link(link))
+        self.assertTrue(path_exists(self.shares))
+        self.assertTrue(path_exists(self.root))
         main = self.build_main()
         # check that the shares link is actually a link
         self.assertEquals(main.shares_dir_link, link)
-        self.assertFalse(os.path.islink(main.shares_dir_link))
-        main.shutdown()
+        self.assertFalse(is_link(main.shares_dir_link))
 
     def test_create_dirs_none_exists(self):
         """test that creating a Main instance works as expected."""
-        link = os.path.join(self.root, 'Shared With Me')
         # remove the existing dirs
-        os.rmdir(self.root)
-        os.rmdir(self.shares)
+        remove_dir(self.root)
+        remove_dir(self.shares)
         main = self.build_main()
         # check that the shares link is actually a link
-        self.assertTrue(os.path.exists(link))
-        self.assertTrue(os.path.islink(main.shares_dir_link))
-        self.assertTrue(os.path.exists(self.shares))
-        self.assertTrue(os.path.exists(self.root))
-        main.shutdown()
+        self.assertTrue(is_link(main.shares_dir_link))
+        self.assertTrue(path_exists(self.shares))
+        self.assertTrue(path_exists(self.root))
 
     def test_connect_if_autoconnect_is_enabled(self):
         """If autoconnect option is enabled, connect the syncdaemon."""
         user_config = main_mod.config.get_user_config()
         orig = user_config.get_autoconnect()
         user_config.set_autoconnect(True)
-        self.addCleanup(lambda: user_config.set_autoconnect(orig))
-
-        self.connect_called = False
-        self.patch(main_mod.ubuntuone.platform.ExternalInterface, 'connect',
-                   lambda *a, **kw: setattr(self, 'connect_called', (a, kw)))
+        self.addCleanup(user_config.set_autoconnect, orig)
 
         main = self.build_main()
-        self.addCleanup(lambda: main.shutdown())
-
-        self.assertEqual(self.connect_called, ((main.external,),
-                                               {'autoconnecting': True}))
+        expected = [('connect', (), {'autoconnecting': True})]
+        self.assertEqual(main.external._called, expected)
 
     def test_dont_connect_if_autoconnect_is_disabled(self):
         """If autoconnect option is disabled, do not connect the syncdaemon."""
         user_config = main_mod.config.get_user_config()
         orig = user_config.get_autoconnect()
         user_config.set_autoconnect(False)
-        self.addCleanup(lambda: user_config.set_autoconnect(orig))
-
-        self.connect_called = False
-        self.patch(main_mod.ubuntuone.platform.ExternalInterface, 'connect',
-                   lambda *a, **kw: setattr(self, 'connect_called', True))
+        self.addCleanup(user_config.set_autoconnect, orig)
 
         main = self.build_main()
-        self.addCleanup(lambda: main.shutdown())
-
-        self.assertFalse(self.connect_called)
+        self.assertEqual(main.external._called, [])
 
     def _get_listeners(self, main):
         """Return the subscribed objects."""
@@ -250,17 +245,14 @@ class MainTests(BaseTwistedTestCase):
 
     def test_event_logger_starts_if_available(self):
         """The event logger is started if available."""
-        self.patch(main_mod.status_listener,
+        self.patch(main_mod.event_logging,
                    "get_listener", lambda *a: FakeListener())
         main = self.build_main()
-        self.addCleanup(lambda: main.shutdown())
         self.assertIn(main.eventlog_listener, self._get_listeners(main))
 
     def test_event_logger_not_started_if_not_available(self):
         """The event logger is not started if it's not available."""
-        self.patch(main_mod.event_logging, "get_listener", lambda *a: None)
         main = self.build_main()
-        self.addCleanup(lambda: main.shutdown())
         self.assertNotIn(main.eventlog_listener, self._get_listeners(main))
 
     def test_status_listener_is_installed(self):
@@ -268,43 +260,33 @@ class MainTests(BaseTwistedTestCase):
         self.patch(main_mod.status_listener,
                    "get_listener", lambda *a: FakeListener())
         main = self.build_main()
-        self.addCleanup(lambda: main.shutdown())
         self.assertIn(main.status_listener, self._get_listeners(main))
 
     def test_status_listener_not_installed_when_disabled(self):
         """The status listener is not started if it's not available."""
-        self.patch(main_mod.status_listener, "get_listener", lambda *a: None)
         main = self.build_main()
-        self.addCleanup(lambda: main.shutdown())
         self.assertNotIn(main.status_listener, self._get_listeners(main))
 
     def test_get_rootdir(self):
         """The get_rootdir returns the root dir."""
-        expected = '~/Ubuntu Test One'
+        expected = os.path.expanduser(os.path.join('~', 'Ubuntu Test One'))
         main = self.build_main(root_dir=expected)
-        self.addCleanup(lambda: main.shutdown())
-
         self.assertEqual(main.get_rootdir(), expected)
 
     def test_get_sharesdir(self):
         """The get_sharesdir returns the shares dir."""
-        expected = '~/Share it to Me'
+        expected = os.path.expanduser(os.path.join('~', 'Share it to Me'))
         main = self.build_main(shares_dir=expected)
-        self.addCleanup(lambda: main.shutdown())
-
         self.assertEqual(main.get_sharesdir(), expected)
 
     def test_get_sharesdirlink(self):
         """The get_sharesdirlink returns the shares dir link."""
         expected = 'Share it to Me'
         main = self.build_main(shares_symlink_name=expected)
-        self.addCleanup(lambda: main.shutdown())
-
         self.assertEqual(main.get_sharesdir_link(),
                          os.path.join(main.get_rootdir(), expected))
 
     def test_version_is_logged(self):
         """Test that the client version is logged."""
-        main = self.build_main()
-        self.addCleanup(lambda: main.shutdown())
+        self.build_main()
         self.assertTrue(self.handler.check_info("client version", VERSION))
