@@ -22,24 +22,27 @@ import contextlib
 import mmap
 import marshal
 import os
-import time
 import types
 import uuid
 import zlib
 
 from operator import attrgetter
+from twisted.internet import defer
 
 from contrib.testing.testcase import BaseTwistedTestCase
 from ubuntuone.devtools.handlers import MementoHandler
+from ubuntuone.syncdaemon import tritcask
 from ubuntuone.syncdaemon.tritcask import (
     TOMBSTONE,
     LIVE,
     HINT,
     INACTIVE,
     DEAD,
+    BROKEN,
     BadCrc,
     BadHeader,
     DataFile,
+    ImmutableDataFile,
     TempDataFile,
     DeadDataFile,
     HintFile,
@@ -48,6 +51,7 @@ from ubuntuone.syncdaemon.tritcask import (
     KeydirEntry,
     Keydir,
     TritcaskShelf,
+    WindowsTimer,
     header_size,
     crc32_size,
     crc32_struct,
@@ -58,6 +62,8 @@ from ubuntuone.syncdaemon.tritcask import (
     is_live,
     is_immutable,
     is_hint,
+    time,
+    timestamp,
     logger,
 )
 
@@ -65,16 +71,18 @@ from ubuntuone.syncdaemon.tritcask import (
 class BaseTestCase(BaseTwistedTestCase):
     """Base testcase for Tritcask."""
 
+    @defer.inlineCallbacks
     def setUp(self):
-        BaseTwistedTestCase.setUp(self)
+        yield super(BaseTestCase, self).setUp()
         self.base_dir = self.mktemp('data_dir')
         self.root_dir = self.mktemp('root_dir')
         self.memento = MementoHandler()
         logger.addHandler(self.memento)
+        self.addCleanup(logger.removeHandler, self.memento)
 
-    def tearDown(self):
-        logger.removeHandler(self.memento)
-        BaseTwistedTestCase.tearDown(self)
+        # reset this class attribute after each test, otherwise
+        # it stays dirty after some tests
+        self.addCleanup(setattr, DataFile, 'last_generated_id', 0)
 
     def build_data(self):
         """Build a random key/value."""
@@ -90,11 +98,34 @@ class DataFileTest(BaseTestCase):
 
     def test__get_next_file_id(self):
         """Test for _get_next_file_id method."""
-        curr = int(time.time() * 100000)
+        curr = int(timestamp() * 100000)
+
         self.assertTrue(curr <= int(self.file_class._get_next_file_id()))
-        curr = int(time.time() * 100000)
+        curr = int(timestamp() * 100000)
         self.assertTrue(curr <= int(self.file_class._get_next_file_id()))
 
+    def test_lightning_fast_virtualbox_sometimes_lies(self):
+        """Win on Virtualbox sometimes returns 0 in time.clock."""
+        test_items = 100
+        self.patch(time, "clock", lambda: 0)
+        file_ids = [self.file_class._get_next_file_id()
+                    for _ in range(test_items)]
+        # even though the clock is not moving, all ids must be different!
+        self.assertEqual(len(set(file_ids)), test_items)
+
+    def test_log_warning_repeated_timestamps(self):
+        """When the clock is not moving, we log a warning."""
+        expected_warning = "Repeated timestamps"
+        ts = timestamp() + 1.0
+        self.patch(tritcask, "timestamp", lambda: ts)
+
+        id1 = self.file_class._get_next_file_id()
+        self.assertFalse(self.memento.check_warning(expected_warning))
+
+        id2 = self.file_class._get_next_file_id()
+        self.assertNotEqual(id1, id2)
+        self.assertTrue(self.memento.check_warning(expected_warning))
+
     def test_exists(self):
         """Tests for exists method."""
         new_file = self.file_class(self.base_dir)
@@ -126,6 +157,7 @@ class DataFileTest(BaseTestCase):
         """Test for make_immutable method."""
         new_file = self.file_class(self.base_dir)
         new_file.fd.write('foo')
+        new_file.fd.flush()
         immutable_file = new_file.make_immutable()
         # the DataFile should be closed
         self.assertEqual(None, new_file.fd)
@@ -134,17 +166,18 @@ class DataFileTest(BaseTestCase):
     def test_iter_entries(self):
         """Test for iter_entries"""
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         for i in range(10):
             db.put(i, 'foo%d' % (i,), 'bar%s' % (i,))
         for i, entry in enumerate(db.live_file.iter_entries()):
             self.assertEqual(entry[4], i)
             self.assertEqual(entry[5], 'foo%d' % (i,))
             self.assertEqual(entry[6], 'bar%d' % (i,))
-        db.shutdown()
 
     def test_iter_entries_bad_crc(self):
         """Test that BadCrc during iter_entries is the same as EOF."""
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         for i in range(10):
             db.put(i, 'foo%d' % (i,), 'bar%s' % (i,))
         # write a different value -> random bytes
@@ -165,11 +198,11 @@ class DataFileTest(BaseTestCase):
         self.assertTrue(self.memento.check_warning(
             'the rest of the file will be ignored.'))
         self.assertTrue(db.live_file.has_bad_crc, 'has_bad_crc should be True.')
-        db.shutdown()
 
     def test_iter_entries_bad_header_unpack(self):
         """Test that unpack error during iter_entries is the same as EOF."""
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         for i in range(10):
             db.put(i, 'foo%d' % (i,), 'bar%s' % (i,))
         # truncate the file at the start of the header of the last value.
@@ -187,7 +220,6 @@ class DataFileTest(BaseTestCase):
         self.assertTrue(self.memento.check_warning('Found corrupted header on'))
         self.assertTrue(self.memento.check_warning(
             'the rest of the file will be ignored.'))
-        db.shutdown()
 
     def test__getitem__(self):
         """Test our slicing support."""
@@ -400,11 +432,11 @@ class ImmutableDataFileTest(DataFileTest):
         db.rotate()
         db.shutdown()
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         for i, entry in enumerate(db._immutable[file_id].iter_entries()):
             self.assertEqual(entry[4], i)
             self.assertEqual(entry[5], 'foo%d' % (i,))
             self.assertEqual(entry[6], 'bar%d' % (i,))
-        db.shutdown()
 
     def test_iter_entries_bad_crc(self):
         """Test that BadCrc during iter_entries is the same as EOF."""
@@ -422,6 +454,7 @@ class ImmutableDataFileTest(DataFileTest):
         db.rotate()
         db.shutdown()
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         entries = []
         for i, entry in enumerate(db._immutable[file_id].iter_entries()):
             entries.append(entry)
@@ -432,7 +465,6 @@ class ImmutableDataFileTest(DataFileTest):
         self.assertTrue(self.memento.check_warning('Found BadCrc on'))
         self.assertTrue(self.memento.check_warning(
             'the rest of the file will be ignored.'))
-        db.shutdown()
 
     def test_iter_entries_bad_header_unpack(self):
         """Test that unpack error during iter_entries is the same as EOF."""
@@ -446,6 +478,7 @@ class ImmutableDataFileTest(DataFileTest):
         db.rotate()
         db.shutdown()
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         entries = []
         for i, entry in enumerate(db._immutable[file_id].iter_entries()):
             entries.append(entry)
@@ -459,8 +492,6 @@ class ImmutableDataFileTest(DataFileTest):
         self.assertTrue(self.memento.check_warning(
             'the rest of the file will be ignored.'))
 
-        db.shutdown()
-
     def test__getitem__(self):
         """Test our slicing support."""
         rw_file = DataFile(self.base_dir)
@@ -623,18 +654,18 @@ class HintFileTest(BaseTestCase):
         #check that the hint matches the contents in the DB
         hint_file = immutable_file.get_hint_file()
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         for hint_entry in hint_file.iter_entries():
             self.assertEqual(6, len(hint_entry))
             self.assertTrue(db.get(hint_entry[2], hint_entry[-1]) is not None)
-        db.shutdown()
 
     def test_write(self):
         """Test the write method."""
         hint_file = HintFile(os.path.join(self.base_dir, 'hint_file'))
-        tstamp1 = time.time()
+        tstamp1 = timestamp()
         entry = HintEntry(tstamp1, len('foo'), 0, len('bar'), 100, 'foo')
         hint_file.write(entry)
-        tstamp2 = time.time()
+        tstamp2 = timestamp()
         entry2 = HintEntry(tstamp2, len('foo1'), 1, len('bar1'), 100, 'foo1')
         hint_file.write(entry2)
         hint_file.close()
@@ -666,7 +697,7 @@ class HintEntryTest(BaseTestCase):
 
     def test_header_property(self):
         """Test the header property."""
-        tstamp = time.time()
+        tstamp = timestamp()
         entry = HintEntry(tstamp, len('foo'), 0, len('bar'), 100, 'foo')
         self.assertEqual((entry.tstamp, entry.key_sz, entry.row_type,
                           entry.value_sz, entry.value_pos), entry.header)
@@ -678,13 +709,11 @@ class LowLevelTest(BaseTestCase):
     Test the on-disk format, writes and reads.
     """
 
+    @defer.inlineCallbacks
     def setUp(self):
-        BaseTestCase.setUp(self)
+        yield super(LowLevelTest, self).setUp()
         self.db = Tritcask(self.base_dir)
-
-    def tearDown(self):
-        BaseTestCase.tearDown(self)
-        self.db.shutdown()
+        self.addCleanup(self.db.shutdown)
 
     def test_get_file_id(self):
         """Test for _get_file_id function."""
@@ -727,6 +756,7 @@ class LowLevelTest(BaseTestCase):
         self.db.live_file.make_immutable()
         self.db.shutdown()
         self.db = Tritcask(self.base_dir)
+        self.addCleanup(self.db.shutdown)
         self.db.put(1, 'foo1', 'bar1')
         self.assertEqual(1, len(self.db._immutable))
         # read from the old file
@@ -755,9 +785,11 @@ class LowLevelTest(BaseTestCase):
         for rtype, key in self.db._keydir.keys():
             self.db.get(rtype, key)
         # check that we have 2 open files
-        self.assertEqual(1, len(self.db._immutable))
-        self.assertNotEqual(None, self.db.live_file)
-        self.db.shutdown()
+        try:
+            self.assertEqual(1, len(self.db._immutable))
+            self.assertNotEqual(None, self.db.live_file)
+        finally:
+            self.db.shutdown()
         self.assertEqual(None, self.db.live_file)
         self.assertEqual(0, len(self.db._immutable))
 
@@ -777,7 +809,6 @@ class LowLevelTest(BaseTestCase):
         self.assertEqual(2, len(files), files)
         self.assertIn(INACTIVE, files[0])
         self.assertIn(LIVE, files[1])
-        self.db.shutdown()
 
     def test_rotate_empty_live_file(self):
         """Test data file rotation with empty live file."""
@@ -787,7 +818,6 @@ class LowLevelTest(BaseTestCase):
         self.assertIn(LIVE, files[0])
         self.assertEqual(self.db.live_file.filename,
                          os.path.join(self.db.base_path, files[0]))
-        self.db.shutdown()
 
 
 class InitTest(BaseTestCase):
@@ -814,10 +844,10 @@ class InitTest(BaseTestCase):
             db.put(0, key, value)
         db.shutdown()
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         self.assertTrue(self.memento.check_debug(
             'loading entries from (build_hint=%s): %s'
             % (False, db.live_file.filename)))
-        db.shutdown()
 
     def test_initialize_bad_path(self):
         """Initialize with a invalid path."""
@@ -835,11 +865,57 @@ class InitTest(BaseTestCase):
             filenames.append(immutable_file.filename)
             immutable_file.close()
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         self.assertNotIn(db.live_file.filename, filenames)
         files = [fn.filename for fn in sorted(db._immutable.values(),
                                               key=attrgetter('filename'))]
         self.assertEqual(files, filenames)
-        db.shutdown()
+
+    def test_find_data_files_immutable_open_error(self):
+        """Test the _find_data_files method failing to open a file."""
+        data_file = DataFile(self.base_dir)
+        data_file.write(0, 'foo_0', 'bar_0')
+        immutable_file = data_file.make_immutable()
+        immutable_file.close()
+        # patch the open call and make it fail
+        def fail_open(filename, *a):
+            """Always fail."""
+            raise IOError("I'm a broken file.")
+        self.patch(ImmutableDataFile, '_open', fail_open)
+        db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
+        files = [fn.filename for fn in sorted(db._immutable.values(),
+                                              key=attrgetter('filename'))]
+        self.assertNotIn(immutable_file.filename, files)
+        # check the logs
+        msg = ("Failed to open %s, renaming it to: %s - error: %s" % \
+               (immutable_file.filename,
+                immutable_file.filename.replace(INACTIVE, BROKEN),
+                IOError("I'm a broken file")))
+        self.assertTrue(self.memento.check_warning(msg))
+
+    def test_find_data_files_live_open_error(self):
+        """Test the _find_data_files method failing to open a file."""
+        data_file = DataFile(self.base_dir)
+        data_file.write(0, 'foo_0', 'bar_0')
+        data_file.close()
+        # patch the open call and make it fail
+        orig_open = DataFile._open
+        def fail_open(filename, *a):
+            """Always fail only once."""
+            self.patch(DataFile, '_open', orig_open)
+            raise IOError("I'm a broken file.")
+        self.patch(DataFile, '_open', fail_open)
+        db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
+        self.assertNotEqual(data_file.filename, db.live_file.filename)
+        # check the logs
+        self.memento.debug = True
+        msg = ("Failed to open %s, renaming it to: %s - error: %s" % \
+               (data_file.filename,
+                data_file.filename.replace(LIVE, BROKEN),
+                IOError("I'm a broken file")))
+        self.assertTrue(self.memento.check_warning(msg))
 
     def test_build_keydir_on_init(self):
         """Test _build_keydir method."""
@@ -850,9 +926,9 @@ class InitTest(BaseTestCase):
         db.shutdown()
         old_keydir = db._keydir
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         # check that the keydir is the same.
         self.assertEqual(old_keydir, db._keydir)
-        db.shutdown()
 
     def test_build_keydir_with_bad_data(self):
         """Test _build_keydir method with a bad data file."""
@@ -869,12 +945,12 @@ class InitTest(BaseTestCase):
             fd.flush()
         old_keydir = db._keydir
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         # check that the keydir is the expected:
         # the new keydir should have entries from 0-8
         # remove the extra entry from the old keydir
         old_keydir.pop((9, 'foo9'))
         self.assertEqual(old_keydir, db._keydir)
-        db.shutdown()
 
     def test_build_keydir_with_hint(self):
         """Test _build_keydir using a hint file."""
@@ -889,8 +965,8 @@ class InitTest(BaseTestCase):
         Tritcask(self.base_dir).shutdown()
         # now create a new DB using the hint
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         self.assertEqual(old_keydir, db._keydir)
-        db.shutdown()
 
     def test_build_keydir_with_multiple_hint(self):
         """Test _build_keydir using a several hint files."""
@@ -912,8 +988,8 @@ class InitTest(BaseTestCase):
         Tritcask(self.base_dir).shutdown()
         # now create a new DB using the hint
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         self.assertEqual(old_keydir, db._keydir)
-        db.shutdown()
 
     def test_build_keydir_build_hint(self):
         """Test _build_keydir build the hints"""
@@ -925,13 +1001,13 @@ class InitTest(BaseTestCase):
         hint_filename = db.live_file.make_immutable().hint_filename
         db.shutdown()
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         self.assertTrue(os.path.exists(hint_filename))
         #check that the hint matches the contents in the DB
         hint_file = HintFile(hint_filename)
         for hint_entry in hint_file.iter_entries():
             self.assertEqual(6, len(hint_entry))
             self.assertTrue(db.get(hint_entry[2], hint_entry[-1]) is not None)
-        db.shutdown()
 
     def test_build_keydir_build_hint_only_for_immutable(self):
         """Test that _build_keydir build the hint only for immutable files."""
@@ -947,6 +1023,7 @@ class InitTest(BaseTestCase):
             db.put(i, 'foo%d' % (i,), 'bar%s' % (i,))
         db.shutdown()
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         self.assertFalse(os.path.exists(db.live_file.hint_filename))
         self.assertTrue(os.path.exists(hint_filename), os.listdir(db.base_path))
         #check that the hint matches the contents in the DB
@@ -954,7 +1031,6 @@ class InitTest(BaseTestCase):
         for hint_entry in hint_file.iter_entries():
             self.assertEqual(6, len(hint_entry))
             self.assertTrue(db.get(hint_entry[2], hint_entry[-1]) is not None)
-        db.shutdown()
 
     def test_build_keydir(self):
         """Test _build_keydir method."""
@@ -966,8 +1042,8 @@ class InitTest(BaseTestCase):
         # shutdown (close the files)
         db.shutdown()
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         self.assertEqual(old_keydir, db._keydir)
-        db.shutdown()
 
     def test_build_keydir_with_dead_rows(self):
         """Test _build_keydir method with TOMBSTONE rows."""
@@ -984,8 +1060,8 @@ class InitTest(BaseTestCase):
         # shutdown (close the files)
         db.shutdown()
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         self.assertEqual(old_keydir, db._keydir)
-        db.shutdown()
 
     def test_build_keydir_build_hint_with_dead_rows(self):
         """Test that _build_keydir build the hint with TOMBSTONE rows."""
@@ -1012,12 +1088,13 @@ class InitTest(BaseTestCase):
         # trigger a hint build of the last rotated file
         Tritcask(self.base_dir, dead_bytes_threshold=0.01).shutdown()
         db = Tritcask(self.base_dir, dead_bytes_threshold=0.01)
+        self.addCleanup(db.shutdown)
         self.assertEqual(sorted(old_keydir.keys()), sorted(db._keydir.keys()))
-        db.shutdown()
 
     def test_should_rotate(self):
         """Test should_rotate method."""
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         # add some data
         for i in range(10):
             db.put(i, 'foo%d' % (i,), 'bar%s' % (i,))
@@ -1027,11 +1104,11 @@ class InitTest(BaseTestCase):
             for i in range(10):
                 db.put(i, 'foo%d' % (i,), 'bar%s' % (i,))
         self.assertTrue(db.should_rotate())
-        db.shutdown()
 
     def test_should_merge(self):
         """Test should_merge method."""
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         # add some data
         for i in range(10):
             db.put(i, 'foo%d' % (i,), 'bar%s' % (i,))
@@ -1043,11 +1120,11 @@ class InitTest(BaseTestCase):
                 db.put(i, 'foo%d' % (i,), 'bar%s' % (i,))
             db.rotate()
         self.assertTrue(db.should_merge(db._immutable))
-        db.shutdown()
 
     def test_should_merge_max_files(self):
         """Test should_merge method based on max_im files ."""
         db = Tritcask(self.base_dir, max_immutable_files=10)
+        self.addCleanup(db.shutdown)
         # add some data
         # create 10 immutable files
         for j in range(10):
@@ -1059,7 +1136,29 @@ class InitTest(BaseTestCase):
         db.put(200, 'foo%d' % (200,), 'bar%s' % (200,))
         db.rotate()
         self.assertTrue(db.should_merge(db._immutable))
+
+    def test_should_merge_no_stats(self):
+        """Test should_merge method without stats for a data file."""
+        db = Tritcask(self.base_dir, max_immutable_files=10)
+        # add some data
+        for j in range(10):
+            db.put(j, 'foo%d' % (j,), 'bar%s' % (j,))
+        # rotate the file to create a immutable with all the live rows
+        db.rotate()
+        # delete everything
+        for j in range(10):
+            db.delete(j, 'foo%d' % (j,))
+        # rotate the file to create an immutable with all the tombstones
+        fid = db.live_file.file_id
+        db.rotate()
         db.shutdown()
+        # start with auto_merge=False
+        db = Tritcask(self.base_dir, auto_merge=False)
+        self.addCleanup(db.shutdown)
+        # check that we don't have any stats for the file with the tombstones
+        self.assertRaises(KeyError, db._keydir.get_stats, fid)
+        # check the should_merge works as expected
+        self.assertTrue(db.should_merge(db._immutable))
 
     def test__rotate_and_not_merge(self):
         """Test _rotate_and_merge method."""
@@ -1118,13 +1217,11 @@ class InitTest(BaseTestCase):
 
 class BasicTest(BaseTestCase):
 
+    @defer.inlineCallbacks
     def setUp(self):
-        BaseTestCase.setUp(self)
+        yield super(BasicTest, self).setUp()
         self.db = Tritcask(self.base_dir)
-
-    def tearDown(self):
-        BaseTestCase.tearDown(self)
-        self.db.shutdown()
+        self.addCleanup(self.db.shutdown)
 
     def test_put(self):
         """Basic test for the put method."""
@@ -1247,6 +1344,7 @@ class MergeTests(BaseTestCase):
         self._add_data(db, 100)
         db.shutdown()
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         keydir_to_merge = db._keydir.copy()
         # add data to the live file
         self._add_data(db, 100)
@@ -1276,6 +1374,7 @@ class MergeTests(BaseTestCase):
         self._add_data(db, 100)
         db.shutdown()
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         keydir_to_merge = db._keydir.copy()
         # add data to the live file
         self._add_data(db, 100)
@@ -1310,6 +1409,7 @@ class MergeTests(BaseTestCase):
         # at this moment we only have the live file
         # start a new Tritcask instance.
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         self.assertFalse(old_live_file.exists())
         self.assertIn(old_live_file.file_id, db._immutable)
 
@@ -1336,11 +1436,11 @@ class MergeTests(BaseTestCase):
         # at this moment we only have the live file
         # start a new Tritcask instance.
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         self.assertFalse(old_live_file.exists())
         self.assertFalse(old_live_file_1.exists())
         self.assertIn(old_live_file.file_id, db._immutable)
         self.assertIn(old_live_file_1.file_id, db._immutable)
-        db.shutdown()
 
     def test_merge_do_nothing_all_dead_entries_single_file(self):
         """Test possible merge of a immutable file with 100% dead data.
@@ -1358,36 +1458,42 @@ class MergeTests(BaseTestCase):
         # start tritcask without automerge and check
         db = Tritcask(self.base_dir, auto_merge=False)
         files = sorted(os.listdir(db.base_path))
-        self.assertEqual(2, len(files))
-        self.assertIn(INACTIVE, files[0])
-        self.assertIn(LIVE, files[1])
-        db.shutdown()
+        try:
+            self.assertEqual(2, len(files))
+            self.assertIn(INACTIVE, files[0])
+            self.assertIn(LIVE, files[1])
+        finally:
+            db.shutdown()
         # trigger the rotation
         # now we should have a single immutable_file (previous live one)
         db = Tritcask(self.base_dir)
-        self.assertEqual(1, len(db._immutable))
-        files = sorted(os.listdir(db.base_path))
-        self.assertEqual(3, len(files))
-        self.assertIn(DEAD, files[0])
-        self.assertIn(DEAD, files[1])
-        self.assertIn(LIVE, files[2])
-        db.shutdown()
+        try:
+            self.assertEqual(1, len(db._immutable))
+            files = sorted(os.listdir(db.base_path))
+            self.assertEqual(3, len(files))
+            self.assertIn(DEAD, files[0])
+            self.assertIn(DEAD, files[1])
+            self.assertIn(LIVE, files[2])
+        finally:
+            db.shutdown()
         # at this moment we have the live + imm_file files
         # start a new Tritcask instance.
         db = Tritcask(self.base_dir)
-        files = sorted(os.path.join(self.base_dir,f ) \
-                       for f in os.listdir(db.base_path))
-        #self.assertIn(imm_file.filename.replace(INACTIVE, DEAD), files)
-        # the immutable_file should be dead as there are no live entries
-        # there should be only 2 files, the dead and the live
-        # dead + dead_hint + live
-        files = sorted(os.listdir(db.base_path))
-        self.assertEqual(1, len(files), files)
-        #self.assertIn(DEAD, files[0])
-        #self.assertIn(DEAD, files[1])
-        #self.assertIn(HINT, files[1])
-        self.assertIn(LIVE, files[0])
-        db.shutdown()
+        try:
+            files = sorted(os.path.join(self.base_dir,f ) \
+                           for f in os.listdir(db.base_path))
+            #self.assertIn(imm_file.filename.replace(INACTIVE, DEAD), files)
+            # the immutable_file should be dead as there are no live entries
+            # there should be only 2 files, the dead and the live
+            # dead + dead_hint + live
+            files = sorted(os.listdir(db.base_path))
+            self.assertEqual(1, len(files), files)
+            #self.assertIn(DEAD, files[0])
+            #self.assertIn(DEAD, files[1])
+            #self.assertIn(HINT, files[1])
+            self.assertIn(LIVE, files[0])
+        finally:
+            db.shutdown()
         # start a new Tritcask to check everything is ok after the
         # merge.
         Tritcask(self.base_dir).shutdown()
@@ -1396,7 +1502,7 @@ class MergeTests(BaseTestCase):
         self.assertEqual(1, len(files), files)
 
     def test_merge_mixed_dead_entries(self):
-        """Test merge of 2 immutable files one with 100% dead data.
+        """Test merge of 2 immutable files one with 100%% dead data.
 
         The expected side effect is that the file should be marked as dead.
         """
@@ -1406,21 +1512,26 @@ class MergeTests(BaseTestCase):
         for i, k in enumerate(db._keydir.keys()):
             if i <= 100:
                 db.delete(*k)
-        db = Tritcask(self.base_dir, auto_merge=False)
-        self.assertEqual(1, len(db._immutable))
-        imm_file_1 = db._immutable.values()[0]
         db.shutdown()
+        db = Tritcask(self.base_dir, auto_merge=False)
+        try:
+            self.assertEqual(1, len(db._immutable))
+            imm_file_1 = db._immutable.values()[0]
+        finally:
+            db.shutdown()
         db = Tritcask(self.base_dir)
         # the "empty" immutable_file should be dead as there are no live entries
         # but it's also removed from the _immutable dict
         files = sorted(os.path.join(self.base_dir,f ) \
                        for f in os.listdir(db.base_path))
-        self.assertIn(imm_file_1.filename.replace(INACTIVE, DEAD), files)
-        self._add_data(db, 100)
-        for i, k in enumerate(db._keydir.keys()):
-            if i % 2:
-                db.delete(*k)
-        db.shutdown()
+        try:
+            self.assertIn(imm_file_1.filename.replace(INACTIVE, DEAD), files)
+            self._add_data(db, 100)
+            for i, k in enumerate(db._keydir.keys()):
+                if i % 2:
+                    db.delete(*k)
+        finally:
+            db.shutdown()
         # at this moment we only have the live file
         # start a new Tritcask instance.
         db = Tritcask(self.base_dir)
@@ -1429,13 +1540,15 @@ class MergeTests(BaseTestCase):
         #    - the new immutable_file
         #    - the current live file
         files = sorted(os.listdir(db.base_path))
-        self.assertEqual(4, len(files), files)
-        self.assertIn(DEAD, files[0])
-        self.assertIn(INACTIVE, files[1])
-        self.assertIn(INACTIVE, files[2])
-        self.assertIn(HINT, files[2])
-        self.assertIn(LIVE, files[3])
-        db.shutdown()
+        try:
+            self.assertEqual(4, len(files), files)
+            self.assertIn(DEAD, files[0])
+            self.assertIn(INACTIVE, files[1])
+            self.assertIn(INACTIVE, files[2])
+            self.assertIn(HINT, files[2])
+            self.assertIn(LIVE, files[3])
+        finally:
+            db.shutdown()
         # start a new Tritcask to check everything is ok after the
         # merge.
         Tritcask(self.base_dir).shutdown()
@@ -1463,20 +1576,23 @@ class MergeTests(BaseTestCase):
         # trigger the rotation
         # now we should have a single immutable_file (previous live one)
         db = Tritcask(self.base_dir)
-        self.assertEqual(1, len(db._immutable))
-        dead_file = db._immutable.values()[0]
-        self.assertIn(DEAD, dead_file.filename)
-        files = sorted(os.listdir(db.base_path))
-        self.assertEqual(4, len(files))
-        self.assertIn(DEAD, files[0])
-        self.assertIn(INACTIVE, files[1])
-        self.assertIn(HINT, files[2])
-        self.assertIn(LIVE, files[3])
-        self.assertEqual(db.live_file.size, 0)
-        db.shutdown()
+        try:
+            self.assertEqual(1, len(db._immutable))
+            dead_file = db._immutable.values()[0]
+            self.assertIn(DEAD, dead_file.filename)
+            files = sorted(os.listdir(db.base_path))
+            self.assertEqual(4, len(files))
+            self.assertIn(DEAD, files[0])
+            self.assertIn(INACTIVE, files[1])
+            self.assertIn(HINT, files[2])
+            self.assertIn(LIVE, files[3])
+            self.assertEqual(db.live_file.size, 0)
+        finally:
+            db.shutdown()
         # at this moment we have the live + imm_file files
         # start a new Tritcask instance.
         db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
         # the dead file should be fone
         # there should be only 2 files, the immutable and the live
         # immutable + hint + live
@@ -1489,7 +1605,7 @@ class MergeTests(BaseTestCase):
         live_id = int(db.live_file.file_id)
         imm_id = int(db._immutable.keys()[0])
         self.assertTrue(live_id > imm_id, "%d <= %d" % (live_id, imm_id))
-        db.shutdown()
+
 
 class KeydirStatsTests(BaseTestCase):
     """Tests for the Keydir stats handling."""
@@ -1499,11 +1615,11 @@ class KeydirStatsTests(BaseTestCase):
         keydir = Keydir()
         file_id = DataFile._get_next_file_id()
         for i in range(10):
-            keydir[(0, str(uuid.uuid4()))] = KeydirEntry(file_id, time.time(),
+            keydir[(0, str(uuid.uuid4()))] = KeydirEntry(file_id, timestamp(),
                                                     len(str(uuid.uuid4())), i+10)
         file_id_1 = DataFile._get_next_file_id()
         for i in range(20):
-            keydir[(0, str(uuid.uuid4()))] = KeydirEntry(file_id_1, time.time(),
+            keydir[(0, str(uuid.uuid4()))] = KeydirEntry(file_id_1, timestamp(),
                                                     len(str(uuid.uuid4())), i+10)
         entry_size = len(str(uuid.uuid4()))*2 + header_size + crc32_size
         self.assertEqual(entry_size*10, keydir._stats[file_id]['live_bytes'])
@@ -1514,11 +1630,11 @@ class KeydirStatsTests(BaseTestCase):
         keydir = Keydir()
         file_id = DataFile._get_next_file_id()
         key = str(uuid.uuid4())
-        entry = KeydirEntry(file_id, time.time(), 1, 1)
+        entry = KeydirEntry(file_id, timestamp(), 1, 1)
         keydir[(0, key)] = entry
         base_size = len(key) + header_size + crc32_size
         self.assertEqual(keydir._stats[file_id]['live_bytes'], base_size + 1)
-        new_entry = KeydirEntry(file_id, time.time(), 2, 2)
+        new_entry = KeydirEntry(file_id, timestamp(), 2, 2)
         keydir[(0, key)] = new_entry
         self.assertEqual(keydir._stats[file_id]['live_bytes'], base_size + 2)
 
@@ -1530,12 +1646,12 @@ class KeydirStatsTests(BaseTestCase):
         keydir = Keydir()
         file_id = DataFile._get_next_file_id()
         key = str(uuid.uuid4())
-        entry = KeydirEntry(file_id, time.time(), 10, 1)
+        entry = KeydirEntry(file_id, timestamp(), 10, 1)
         keydir[(0, key)] = entry
         base_size = len(key) + header_size + crc32_size
         self.assertEqual(keydir._stats[file_id]['live_bytes'], base_size + 10)
         new_file_id = DataFile._get_next_file_id()
-        new_entry = KeydirEntry(new_file_id, time.time(), 5, 10)
+        new_entry = KeydirEntry(new_file_id, timestamp(), 5, 10)
         keydir[(0, key)] = new_entry
         self.assertEqual(keydir._stats[new_file_id]['live_bytes'],
                          base_size + 5)
@@ -1547,14 +1663,14 @@ class KeydirStatsTests(BaseTestCase):
         file_id = DataFile._get_next_file_id()
         for i in range(10):
             key = str(uuid.uuid4())
-            keydir[(0, key)] = KeydirEntry(file_id, time.time(),
+            keydir[(0, key)] = KeydirEntry(file_id, timestamp(),
                                            len(str(uuid.uuid4())), i+10)
             if i % 2:
                 keydir.remove((0, key))
         file_id_1 = DataFile._get_next_file_id()
         for i in range(20):
             key = str(uuid.uuid4())
-            keydir[(0, key)] = KeydirEntry(file_id_1, time.time(),
+            keydir[(0, key)] = KeydirEntry(file_id_1, timestamp(),
                                            len(str(uuid.uuid4())), i+10)
             if i % 2:
                 keydir.remove((0, key))
@@ -1578,12 +1694,18 @@ class KeydirStatsTests(BaseTestCase):
         keydir = Keydir()
         file_id = DataFile._get_next_file_id()
         for i in range(10):
-            keydir[(0, str(uuid.uuid4()))] = KeydirEntry(file_id, time.time(),
+            keydir[(0, str(uuid.uuid4()))] = KeydirEntry(file_id, timestamp(),
                                                  len(str(uuid.uuid4())), i+10)
         self.assertEqual(keydir._stats[file_id], keydir.get_stats(file_id))
         self.assertTrue(keydir._stats[file_id] is not \
                         keydir.get_stats(file_id))
 
+    def test_get_stats_missing(self):
+        """Test get_stats with a missing file_id."""
+        keydir = Keydir()
+        file_id = DataFile._get_next_file_id()
+        self.assertRaises(KeyError, keydir.get_stats, file_id)
+
 
 class TritcaskShelfTests(BaseTestCase):
     """Tests for TritcaskShelf."""
@@ -1659,7 +1781,7 @@ class TritcaskShelfTests(BaseTestCase):
         shelf = MarshalShelf(0, db)
         shelf['foo'] = 'bar'
         self.assertIn('bar', shelf['foo'])
-        self.assertEquals(db.get(0, 'foo'), marshal.dumps('bar'))
+        self.assertEqual(db.get(0, 'foo'), marshal.dumps('bar'))
 
     def test_keys(self):
         """Test for the keys method."""
@@ -1683,3 +1805,10 @@ class TritcaskShelfTests(BaseTestCase):
         shelf['foo_a'] = 'bar_a'
         self.assertEqual(11, len(shelf))
 
+class WindowsTimerTests(BaseTwistedTestCase):
+    """Tests for the windows timer."""
+
+    def test_initial_value(self):
+        """Test that the initial value is > 0."""
+        timer = WindowsTimer()
+        self.assertTrue(int(timer.time()) > 0)
