@@ -17,8 +17,10 @@
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
 """DBUS interface module."""
 
-import dbus.service
 import logging
+import warnings
+
+import dbus.service
 
 from dbus import DBusException
 from xml.etree import ElementTree
@@ -26,9 +28,11 @@ from xml.etree import ElementTree
 from twisted.internet import defer
 from twisted.python.failure import Failure
 from ubuntuone.syncdaemon.interfaces import IMarker
-from ubuntuone.platform.linux.credentials import (
+from ubuntuone.platform.credentials.linux import (
     DBUS_BUS_NAME, DBUS_CREDENTIALS_PATH,
     DBUS_CREDENTIALS_IFACE)
+from ubuntuone.platform.linux.launcher import UbuntuOneLauncher
+
 from ubuntuone.syncdaemon.interaction_interfaces import (
     bool_str,
     get_share_dict,
@@ -41,30 +45,43 @@ from ubuntuone.syncdaemon.interaction_interfaces import (
     SyncdaemonPublicFiles,
     SyncdaemonService,
     SyncdaemonShares,
-    SyncdaemonStatus
-)
+    SyncdaemonStatus)
 
 # Disable the "Invalid Name" check here, as we have lots of DBus style names
 # pylint: disable-msg=C0103
 
 DBUS_IFACE_NAME = 'com.ubuntuone.SyncDaemon'
 DBUS_IFACE_SYNC_NAME = DBUS_IFACE_NAME + '.SyncDaemon'
-DBUS_IFACE_STATUS_NAME = DBUS_IFACE_NAME+'.Status'
-DBUS_IFACE_EVENTS_NAME = DBUS_IFACE_NAME+'.Events'
-DBUS_IFACE_FS_NAME = DBUS_IFACE_NAME+'.FileSystem'
-DBUS_IFACE_SHARES_NAME = DBUS_IFACE_NAME+'.Shares'
-DBUS_IFACE_CONFIG_NAME = DBUS_IFACE_NAME+'.Config'
-DBUS_IFACE_FOLDERS_NAME = DBUS_IFACE_NAME+'.Folders'
+DBUS_IFACE_STATUS_NAME = DBUS_IFACE_NAME + '.Status'
+DBUS_IFACE_EVENTS_NAME = DBUS_IFACE_NAME + '.Events'
+DBUS_IFACE_FS_NAME = DBUS_IFACE_NAME + '.FileSystem'
+DBUS_IFACE_SHARES_NAME = DBUS_IFACE_NAME + '.Shares'
+DBUS_IFACE_CONFIG_NAME = DBUS_IFACE_NAME + '.Config'
+DBUS_IFACE_FOLDERS_NAME = DBUS_IFACE_NAME + '.Folders'
 DBUS_IFACE_PUBLIC_FILES_NAME = DBUS_IFACE_NAME + '.PublicFiles'
+DBUS_IFACE_LAUNCHER_NAME = DBUS_IFACE_NAME + '.Launcher'
 
 # NetworkManager State constants
 NM_STATE_UNKNOWN = 0
-NM_STATE_ASLEEP = 1
-NM_STATE_CONNECTING = 2
-NM_STATE_CONNECTED = 3
-NM_STATE_DISCONNECTED = 4
+NM_STATE_ASLEEP_OLD = 1
+NM_STATE_ASLEEP = 10
+NM_STATE_CONNECTING_OLD = 2
+NM_STATE_CONNECTING = 40
+NM_STATE_CONNECTED_OLD = 3
+NM_STATE_CONNECTED_LOCAL = 50
+NM_STATE_CONNECTED_SITE = 60
+NM_STATE_CONNECTED_GLOBAL = 70
+NM_STATE_DISCONNECTED_OLD = 4
+NM_STATE_DISCONNECTED = 20
 # NM state -> events mapping
-NM_STATE_EVENTS = {NM_STATE_CONNECTED: 'SYS_NET_CONNECTED',
+# Note that the LOCAL and SITE mappings are *not* a typo.  Local and site links
+# are not enough to connect to one.ubuntu.com, so we treat them as if we were
+# not connected.
+NM_STATE_EVENTS = {NM_STATE_CONNECTED_OLD: 'SYS_NET_CONNECTED',
+                   NM_STATE_CONNECTED_LOCAL: 'SYS_NET_DISCONNECTED',
+                   NM_STATE_CONNECTED_SITE: 'SYS_NET_DISCONNECTED',
+                   NM_STATE_CONNECTED_GLOBAL: 'SYS_NET_CONNECTED',
+                   NM_STATE_DISCONNECTED_OLD: 'SYS_NET_DISCONNECTED',
                    NM_STATE_DISCONNECTED: 'SYS_NET_DISCONNECTED'}
 
 logger = logging.getLogger("ubuntuone.SyncDaemon.DBus")
@@ -179,12 +196,19 @@ class Status(DBusExposedObject):
         logger.debug('called current_downloads')
         return self.syncdaemon_status.current_downloads()
 
-    @dbus.service.method(DBUS_IFACE_STATUS_NAME, out_signature='a(sa{ss})')
+    @dbus.service.method(DBUS_IFACE_STATUS_NAME,
+                         in_signature='s', out_signature='t')
+    def free_space(self, vol_id):
+        """Return the free space for the given volume."""
+        logger.debug('free_space for volume %r', vol_id)
+        return self.syncdaemon_status.free_space(vol_id)
+
+    @dbus.service.method(DBUS_IFACE_STATUS_NAME, out_signature='a(ssa{ss})')
     def waiting(self):
         """Return a list of the operations in action queue."""
         logger.debug('called waiting')
         commands = self.syncdaemon_status.waiting()
-        for op, data  in commands:
+        for op, op_id, data in commands:
             sanitize_dict(data)
         return commands
 
@@ -192,9 +216,11 @@ class Status(DBusExposedObject):
     def waiting_metadata(self):
         """Return a list of the operations in the meta-queue.
 
-        As we don't have meta-queue anymore, this is faked.  This method
+        As we don't have meta-queue anymore, this is faked. This method
         is deprecated, and will go away in a near future.
+
         """
+        warnings.warn('Use "waiting" method instead.', DeprecationWarning)
         logger.debug('called waiting_metadata')
         commands = self.syncdaemon_status.waiting_metadata()
         for op, data  in commands:
@@ -207,7 +233,9 @@ class Status(DBusExposedObject):
 
         As we don't have content-queue anymore, this is faked.  This method
         is deprecated, and will go away in a near future.
+
         """
+        warnings.warn('Use "waiting" method instead.', DeprecationWarning)
         logger.debug('called waiting_content')
         commands = self.syncdaemon_status.waiting_content()
         for data in commands:
@@ -265,14 +293,20 @@ class Status(DBusExposedObject):
         """Fire a signal to notify that the content queue has changed.
 
         This signal is deprecated, and will go away in a near future.
+
         """
+        msg = 'Connect to RequestQueueAdded/RequestQueueRemoved instead.'
+        warnings.warn(msg, DeprecationWarning)
 
     @dbus.service.signal(DBUS_IFACE_STATUS_NAME)
     def MetaQueueChanged(self):
         """Fire a signal to notify that the meta queue has changed.
 
         This signal is deprecated, and will go away in a near future.
+
         """
+        msg = 'Connect to RequestQueueAdded/RequestQueueRemoved instead.'
+        warnings.warn(msg, DeprecationWarning)
 
     @dbus.service.signal(DBUS_IFACE_STATUS_NAME, signature='ssa{ss}')
     def RequestQueueAdded(self, op_name, op_id, data):
@@ -536,7 +570,6 @@ class FileSystem(DBusExposedObject):
         return self.syncdaemon_filesystem.get_metadata_and_quick_tree_synced(
                                                                         path)
 
-
     @dbus.service.method(DBUS_IFACE_FS_NAME,
                          in_signature='', out_signature='aa{ss}')
     def get_dirty_nodes(self):
@@ -906,10 +939,26 @@ class Config(DBusExposedObject):
         """Return the autoconnect config value."""
         return self.syncdaemon_config.autoconnect_enabled()
 
+    @dbus.service.method(DBUS_IFACE_CONFIG_NAME)
+    def enable_autoconnect(self):
+        """Enable syncdaemon autoconnect."""
+        self.syncdaemon_config.set_autoconnect_enabled(True)
+
+    @dbus.service.method(DBUS_IFACE_CONFIG_NAME)
+    def disable_autoconnect(self):
+        """Disable syncdaemon autoconnect."""
+        self.syncdaemon_config.set_autoconnect_enabled(False)
+
     @dbus.service.method(DBUS_IFACE_CONFIG_NAME,
                          in_signature='b', out_signature='')
     def set_autoconnect_enabled(self, enabled):
-        """Enable syncdaemon autoconnect."""
+        """Enable syncdaemon autoconnect.
+
+        This method is deprecated.
+
+        """
+        msg = 'Use enable_autoconnect/disable_autoconnect instead.'
+        warnings.warn(msg, DeprecationWarning)
         self.syncdaemon_config.set_autoconnect_enabled(enabled)
 
     @dbus.service.method(DBUS_IFACE_CONFIG_NAME,
@@ -1076,6 +1125,22 @@ class Folders(DBusExposedObject):
         """Notify an error while unsubscribing from a user defined folder."""
 
 
+class Launcher(DBusExposedObject):
+    """A DBus interface to interact with the launcher icon."""
+
+    def __init__(self, bus_name):
+        """Create the instance."""
+        self.path = '/launcher'
+        DBusExposedObject.__init__(self, bus_name=bus_name,
+                                   path=self.path)
+
+    @dbus.service.method(DBUS_IFACE_LAUNCHER_NAME)
+    def unset_urgency(self):
+        """Unset urgency on the launcher."""
+        launcher = UbuntuOneLauncher()
+        launcher.set_urgent(False)
+
+
 class PublicFiles(DBusExposedObject):
     """A DBus interface for handling public files."""
 
@@ -1193,6 +1258,7 @@ class DBusInterface(object):
                              self.volume_manager)
         self.folders = Folders(self.busName, self.volume_manager,
                                self.fs_manager)
+        self.launcher = Launcher(self.busName)
         self.public_files = PublicFiles(
             self.busName, self.fs_manager, self.action_queue)
         self.config = Config(self.busName, self)
@@ -1212,7 +1278,7 @@ class DBusInterface(object):
         if DBusInterface.test:
             # testing under sync; just do it
             logger.debug('using the fake NetworkManager')
-            self.connection_state_changed(NM_STATE_CONNECTED)
+            self.connection_state_changed(NM_STATE_CONNECTED_GLOBAL)
         else:
             def error_handler(error):
                 """Handle errors from NM."""
@@ -1222,7 +1288,7 @@ class DBusInterface(object):
                 # If we get an error back from NetworkManager, we should
                 # just try to connect anyway; it probably means that
                 # NetworkManager is down or broken or something.
-                self.connection_state_changed(NM_STATE_CONNECTED)
+                self.connection_state_changed(NM_STATE_CONNECTED_GLOBAL)
             try:
                 nm = self.system_bus.get_object(
                     'org.freedesktop.NetworkManager',
@@ -1235,7 +1301,7 @@ class DBusInterface(object):
                     # NetworkManager isn't running.
                     logger.warn("Unable to connect to NetworkManager. "
                                   "Assuming we have network.")
-                    self.connection_state_changed(NM_STATE_CONNECTED)
+                    self.connection_state_changed(NM_STATE_CONNECTED_GLOBAL)
                 else:
                     raise
             else:
@@ -1268,6 +1334,7 @@ class DBusInterface(object):
         if self.send_events:
             self.event_queue.unsubscribe(self.all_events_sender)
         self.folders.remove_from_connection()
+        self.launcher.remove_from_connection()
         # remove the NM's StateChanged signal receiver
         self.system_bus.remove_signal_receiver(self.connection_state_changed,
                                signal_name='StateChanged',
