@@ -20,32 +20,22 @@
 import itertools
 import operator
 import os
-import sys
+
 
 import gettext
 
 from twisted.internet import reactor, defer
 
 from ubuntuone.clientdefs import GETTEXT_PACKAGE
-from ubuntuone.logger import basic_formatter, logging
+from ubuntuone.status.logger import logger
 from ubuntuone.platform import session
 from ubuntuone.platform.notification import Notification
 from ubuntuone.platform.messaging import Messaging
-from ubuntuone.platform.unity import UbuntuOneLauncher, DummyLauncher
+from ubuntuone.platform.launcher import UbuntuOneLauncher, DummyLauncher
 
 ONE_DAY = 24 * 60 * 60
 Q_ = lambda string: gettext.dgettext(GETTEXT_PACKAGE, string)
 
-LOG_LEVEL = logging.DEBUG
-logger = logging.getLogger('ubuntuone.status')
-logger.setLevel(LOG_LEVEL)
-
-if os.environ.get('DEBUG_STATUS'):
-    debug_handler = logging.StreamHandler(sys.stderr)
-    debug_handler.setFormatter(basic_formatter)
-    debug_handler.setLevel(LOG_LEVEL)
-    logger.addHandler(debug_handler)
-
 UBUNTUONE_TITLE = Q_("Ubuntu One")
 NEW_UDFS_SENDER = Q_("New cloud folder(s) available")
 FINAL_COMPLETED = Q_("File synchronization completed.")
@@ -74,7 +64,7 @@ def files_being_uploaded(filename, files_uploading):
         "filename": filename, "other_files": other_files}
     return gettext.dngettext(
         GETTEXT_PACKAGE,
-        "'%(filename)s' and %(other_files)d other file is being "
+        "'%(filename)s' and %(other_files)d other file are being "
         "uploaded to your personal cloud.",
         "'%(filename)s' and %(other_files)d other files are being "
         "uploaded to your personal cloud.", other_files) % format_args
@@ -91,7 +81,7 @@ def files_being_downloaded(filename, files_downloading):
         "filename": filename, "other_files": other_files}
     return gettext.dngettext(
         GETTEXT_PACKAGE,
-        "'%(filename)s' and %(other_files)d other file is being "
+        "'%(filename)s' and %(other_files)d other file are being "
         "downloaded to your computer.",
         "'%(filename)s' and %(other_files)d other files are being "
         "downloaded to your computer.", other_files) % format_args
@@ -108,7 +98,7 @@ def files_were_uploaded(filename, upload_done):
         'filename': filename, 'other_files': other_files}
     return gettext.dngettext(
         GETTEXT_PACKAGE,
-        "'%(filename)s' and %(other_files)d other file was uploaded to "
+        "'%(filename)s' and %(other_files)d other file were uploaded to "
         "your personal cloud.",
         "'%(filename)s' and %(other_files)d other files were uploaded "
         "to your personal cloud.", other_files) % format_args
@@ -125,16 +115,12 @@ def files_were_downloaded(filename, download_done):
         'filename': filename, 'other_files': other_files}
     return gettext.dngettext(
         GETTEXT_PACKAGE,
-        "'%(filename)s' and %(other_files)d other file was "
+        "'%(filename)s' and %(other_files)d other file were "
         "downloaded to your computer.",
         "'%(filename)s' and %(other_files)d other files were "
         "downloaded to your computer.", other_files) % format_args
 
 
-def files_were_made_public(no_of_files):
-    """Get the i18n string for files that were made public."""
-
-
 class ToggleableNotification(object):
     """A controller for notifications that can be turned off."""
 
@@ -554,10 +540,9 @@ class ProgressBar(object):
         self.launcher.set_progress(self.progress)
         logger.debug("progressbar updated: %f", self.progress)
 
-    def progress_made(self, steps_done, steps_total):
-        """Steps amount changed. Set up a timer if one not ticking."""
-        assert steps_total > 0
-        self.progress = float(steps_done) / float(steps_total)
+    def set_progress(self, progress):
+        """Steps amount changed. Set up a timer if there isn't one ticking."""
+        self.progress = progress
         if not self.visible:
             self.visible = True
             self.launcher.show_progressbar()
@@ -625,6 +610,8 @@ class StatusAggregator(object):
         self.reset()
         self.progress_bar = ProgressBar(clock=self.clock)
         self.finished_delay = 10
+        self.progress = {}
+        self.to_do = {}
 
     def get_notification(self):
         """Create a new toggleable notification object."""
@@ -633,11 +620,7 @@ class StatusAggregator(object):
     # pylint: disable=W0201
     def reset(self):
         """Reset all counters and notifications."""
-        self.total_counter = 0
-        self.done_counter = 0
         self.download_done = 0
-        self.download_total = 0
-        self.upload_total = 0
         self.upload_done = 0
         self.files_uploading = []
         self.uploading_filename = ''
@@ -655,6 +638,8 @@ class StatusAggregator(object):
         if self.final_status_bubble:
             self.final_status_bubble.cleanup()
         self.final_status_bubble = FinalStatusBubble(self)
+        self.progress = {}
+        self.to_do = {}
     # pylint: enable=W0201
 
     def get_discovery_message(self):
@@ -699,7 +684,7 @@ class StatusAggregator(object):
 
     def queue_done(self):
         """Queue is finished."""
-        if not self.total_counter:
+        if not self.to_do:
             return
         if self.queue_done_timer is None:
             logger.debug("queue done callback added")
@@ -712,7 +697,10 @@ class StatusAggregator(object):
 
     def update_progressbar(self):
         """Update the counters of the progressbar."""
-        self.progress_bar.progress_made(self.done_counter, self.total_counter)
+        if len(self.to_do) > 0:
+            progress = float(
+                sum(self.progress.values())) / sum(self.to_do.values())
+            self.progress_bar.set_progress(progress)
 
     def download_started(self, command):
         """A download just started."""
@@ -720,16 +708,18 @@ class StatusAggregator(object):
             self.queue_done_timer.cleanup()
             self.queue_done_timer = None
         self.files_downloading.append(command)
-        self.download_total += 1
-        self.total_counter += 1
+        if command.deflated_size is not None:
+            self.to_do[
+                (command.share_id, command.node_id)] = command.deflated_size
         # pylint: disable=W0201
         if not self.downloading_filename:
             self.downloading_filename = self.files_downloading[0].path.split(
                 os.path.sep)[-1]
         # pylint: enable=W0201
         self.update_progressbar()
-        logger.debug("queueing command (%d/%d): %s", self.done_counter,
-                     self.total_counter, command.__class__.__name__)
+        logger.debug(
+            "queueing command (total: %d): %s",
+            len(self.to_do), command.__class__.__name__)
         self.file_discovery_bubble.new_file_found()
 
     def download_finished(self, command):
@@ -737,9 +727,10 @@ class StatusAggregator(object):
         if command in self.files_downloading:
             self.files_downloading.remove(command)
         self.download_done += 1
-        self.done_counter += 1
-        logger.debug("unqueueing command (%d/%d): %s", self.done_counter,
-                     self.total_counter, command.__class__.__name__)
+        if command.deflated_size is not None:
+            self.progress[
+                (command.share_id, command.node_id)] = command.deflated_size
+        logger.debug("unqueueing command: %s", command.__class__.__name__)
         self.update_progressbar()
 
     def upload_started(self, command):
@@ -748,16 +739,18 @@ class StatusAggregator(object):
             self.queue_done_timer.cleanup()
             self.queue_done_timer = None
         self.files_uploading.append(command)
-        self.upload_total += 1
-        self.total_counter += 1
+        if command.deflated_size is not None:
+            self.to_do[
+                (command.share_id, command.node_id)] = command.deflated_size
         # pylint: disable=W0201
         if not self.uploading_filename:
             self.uploading_filename = self.files_uploading[0].path.split(
                 os.path.sep)[-1]
         # pylint: enable=W0201
         self.update_progressbar()
-        logger.debug("queueing command (%d/%d): %s", self.done_counter,
-                     self.total_counter, command.__class__.__name__)
+        logger.debug(
+            "queueing command (total: %d): %s", len(self.to_do),
+            command.__class__.__name__)
         self.file_discovery_bubble.new_file_found()
 
     def upload_finished(self, command):
@@ -765,9 +758,19 @@ class StatusAggregator(object):
         if command in self.files_uploading:
             self.files_uploading.remove(command)
         self.upload_done += 1
-        self.done_counter += 1
-        logger.debug("unqueueing command (%d/%d): %s", self.done_counter,
-                     self.total_counter, command.__class__.__name__)
+        if command.deflated_size is not None:
+            self.progress[
+                (command.share_id, command.node_id)] = command.deflated_size
+        logger.debug("unqueueing command: %s", command.__class__.__name__)
+        self.update_progressbar()
+
+    def progress_made(self, share_id, node_id, n_bytes_written, deflated_size):
+        """Progress made on up- or download."""
+        if n_bytes_written is not None:
+            # if we haven't gotten the total size yet, set it now
+            if deflated_size and (share_id, node_id) not in self.to_do:
+                self.to_do[(share_id, node_id)] = deflated_size
+            self.progress[(share_id, node_id)] = n_bytes_written
         self.update_progressbar()
 
     def connection_lost(self):
@@ -787,7 +790,6 @@ class StatusFrontend(object):
         self.aggregator = StatusAggregator(clock=clock)
         self.notification = self.aggregator.get_notification()
         self.messaging = Messaging()
-        self.udf_message = None
         self.quota_timer = None
 
     def file_published(self, public_url):
@@ -817,6 +819,11 @@ class StatusFrontend(object):
         """A file upload was unqueued."""
         self.aggregator.upload_finished(command)
 
+    def progress_made(self, share_id, node_id, n_bytes_written, deflated_size):
+        """Progress made on up- or download."""
+        self.aggregator.progress_made(
+            share_id, node_id, n_bytes_written, deflated_size)
+
     def queue_done(self):
         """The queue is empty."""
         self.aggregator.queue_done()
@@ -831,21 +838,9 @@ class StatusFrontend(object):
         """A new udf is available for subscription."""
         if udf.subscribed:
             return
-        if self.udf_message is not None:
-            self.messaging.update_count(self.udf_message, 1)
-            return
-        self.udf_message = self.messaging.show_message(
-            NEW_UDFS_SENDER, callback=self._reset_udf_message_callback,
-            message_count=1)
         self.notification.send_notification(
             UBUNTUONE_TITLE, UDFAvailableStatus(udf=udf).one())
 
-    def _reset_udf_message_callback(self, indicator, message_time=None):
-        """A callback wrapper that resets the udf_message to None."""
-        self.udf_message = None
-        callback = self.messaging.create_callback()
-        callback(indicator, message_time=message_time)
-
     def server_connection_lost(self):
         """The client lost the connection to the server."""
         logger.debug("server connection lost")
