@@ -42,6 +42,8 @@ from urllib import urlencode
 from urllib2 import urlopen, Request, HTTPError
 from urlparse import urljoin
 
+import OpenSSL.SSL
+
 from zope.interface import implements
 from twisted.internet import reactor, defer, threads, task
 from twisted.internet import error as twisted_errors
@@ -49,6 +51,8 @@ from twisted.names import client as dns_client
 from twisted.python.failure import Failure, DefaultException
 
 from oauth import oauth
+from ubuntuone import clientdefs
+from ubuntuone.platform import platform, remove_file
 from ubuntuone.storageprotocol import protocol_pb2, content_hash
 from ubuntuone.storageprotocol import errors as protocol_errors
 from ubuntuone.storageprotocol.client import (
@@ -70,7 +74,6 @@ EREGEX = "^.+\\@(\\[?)[a-zA-Z0-9\\-\\.]+\\.([a-zA-Z]{2,3}|[0-9]{1,3})(\\]?)$"
 # progress threshold to emit a download/upload progress event: 64Kb
 TRANSFER_PROGRESS_THRESHOLD = 64*1024*1024
 
-
 def passit(func):
     """Pass the value on for the next deferred, while calling func with it."""
 
@@ -231,11 +234,29 @@ class NamedTemporaryFile(object):
 
     def __init__(self):
         fileno, self.name = tempfile.mkstemp()
-        self._fd = os.fdopen(fileno, 'r+w')
+
+        # build a file object from the descriptor; note that this will *not*
+        # create a new file descriptor at the OS level
+        self._fh = os.fdopen(fileno, 'w+b')
 
     def __getattr__(self, attr):
-        """Proxy everything else (other than .name) on to self._fd."""
-        return getattr(self._fd, attr)
+        """Proxy everything else (other than .name) on to self._fh."""
+        return getattr(self._fh, attr)
+
+
+def sanitize_message(message):
+    """Remove bytes and magic hash, return arguments to log()."""
+    if message.type == protocol_pb2.Message.BYTES:
+        return ('start - processMessage: id: %s, type: %s',
+                message.id, message.type)
+    elif message.type == protocol_pb2.Message.PUT_CONTENT:
+        lines = [ line for line in str(message).split("\n")
+                if not line.strip().startswith("magic_hash:") ]
+        return ('start - processMessage: %s',
+                " ".join(lines))
+    else:
+        return ('start - processMessage: %s',
+                str(message).replace("\n", " "))
 
 
 class LoggingStorageClient(ThrottlingStorageClient):
@@ -248,18 +269,16 @@ class LoggingStorageClient(ThrottlingStorageClient):
         ThrottlingStorageClient.__init__(self)
         self.log = logging.getLogger('ubuntuone.SyncDaemon.StorageClient')
         # configure the handler level to be < than DEBUG
-        self.log.setLevel(TRACE)
         self.log_trace = partial(self.log.log, TRACE)
 
+    def log_message(self, message):
+        """Log the messages in the trace log."""
+        if self.log.isEnabledFor(TRACE):
+            self.log_trace(*sanitize_message(message))
+
     def processMessage(self, message):
         """Wrapper that logs the message and result."""
-        # don't log the full message if it's of type BYTES
-        if message.type == protocol_pb2.Message.BYTES:
-            self.log_trace('start - processMessage: id: %s, type: %s',
-                           message.id, message.type)
-        else:
-            self.log_trace('start - processMessage: %s',
-                          str(message).replace("\n", " "))
+        self.log_message(message)
         if message.id in self.requests:
             req = self.requests[message.id]
             req.deferred.addCallbacks(self.log_success, self.log_error)
@@ -283,52 +302,82 @@ class LoggingStorageClient(ThrottlingStorageClient):
     def sendMessage(self, message):
         """Wrapper that logs the message and result."""
         # don't log the full message if it's of type BYTES
-        if message.type == protocol_pb2.Message.BYTES:
-            self.log_trace('start - sendMessage: id: %s, type: %s',
-                           message.id, message.type)
-        else:
-            self.log_trace('start - sendMessage: %s',
-                          str(message).replace("\n", " "))
+        self.log_message(message)
         result = ThrottlingStorageClient.sendMessage(self, message)
         self.log_trace('end - sendMessage: id: %s', message.id)
         return result
 
 
+class PingManager(object):
+    """Handle the ping/pong with the server."""
+
+    _ping_delay = 600  # 10 minutes
+    _timeout_delay = 180  # 3 minutes
+
+    def __init__(self, client):
+        self.client = client
+        self._loop = task.LoopingCall(self._do_ping)
+        self._loop.start(self._ping_delay, now=False)
+        self._timeout_call = None
+        self._running = True
+
+    @defer.inlineCallbacks
+    def _do_ping(self):
+        """Ping the server just to use the network."""
+        self.client.log.trace("Sending ping")
+        self._timeout_call = reactor.callLater(self._timeout_delay,
+                                               self._disconnect)
+        req = yield self.client.ping()
+        self.client.log.debug("Ping! rtt: %.3f segs", req.rtt)
+        self._timeout_call.cancel()
+
+    def _disconnect(self):
+        """Never got the pong, disconnect."""
+        self.stop()
+        self.client.log.info("No Pong response, disconnecting the client")
+        self.client.transport.loseConnection()
+
+    def _stop(self):
+        """Really stop all calls."""
+        self._loop.stop()
+        if self._timeout_call is not None and self._timeout_call.active():
+            self._timeout_call.cancel()
+
+    def stop(self):
+        """Stop all the calls if still running."""
+        if self._running:
+            self._running = False
+            self._stop()
+
+
 class ActionQueueProtocol(LoggingStorageClient):
     """This is the Action Queue version of the StorageClient protocol."""
 
     factory = None
-    _looping_ping = None
-    _ping_delay = 600  # 10 minutes
 
     def __init__(self):
         LoggingStorageClient.__init__(self)
         user_config = config.get_user_config()
         self.max_payload_size = user_config.get_max_payload_size()
+        self.ping_manager = None
 
     def connectionMade(self):
         """A new connection was made."""
         self.log.info('Connection made.')
         LoggingStorageClient.connectionMade(self)
         self.factory.event_queue.push('SYS_CONNECTION_MADE')
-        if self._looping_ping is None:
-            self._looping_ping = task.LoopingCall(self._do_ping)
-            self._looping_ping.start(self._ping_delay, now=False)
+        if self.ping_manager is not None:
+            self.ping_manager.stop()
+        self.ping_manager = PingManager(self)
 
     def connectionLost(self, reason):
         """The connection was lost."""
         self.log.info('Connection lost, reason: %s.', reason)
-        if self._looping_ping is not None:
-            self._looping_ping.stop()
-            self._looping_ping = None
+        if self.ping_manager is not None:
+            self.ping_manager.stop()
+            self.ping_manager = None
         LoggingStorageClient.connectionLost(self, reason)
 
-    @defer.inlineCallbacks
-    def _do_ping(self):
-        """Ping the server just to use the network."""
-        req = yield self.ping()
-        self.log.debug("Ping! rtt: %.3f segs", req.rtt)
-
 
 class Marker(str):
     """A uuid4-based marker class."""
@@ -397,10 +446,7 @@ class ZipQueue(object):
             upload.log.debug('compressing: %r', filename)
             # we need to compress the file completely to figure out its
             # compressed size. So streaming is out :(
-            if upload.tempfile_factory is None:
-                f = NamedTemporaryFile()
-            else:
-                f = upload.tempfile_factory()
+            upload.tempfile = f = NamedTemporaryFile()
             zipper = zlib.compressobj()
             magic_hasher = content_hash.magic_hash_factory()
             while not upload.cancelled:
@@ -414,10 +460,11 @@ class ZipQueue(object):
                 f.write(zipper.compress(data))
                 magic_hasher.update(data)
             upload.deflated_size = f.tell()
-            # close the compressed file (thus, if you actually want to stream
-            # it out, it must have a name so it can be reopnened)
-            f.close()
-            upload.tempfile = f
+
+            # keep the file open, but reset its position
+            # to zero, ready to be read later
+            f.seek(0)
+
             upload.magic_hash = magic_hasher.content_hash()
         except Exception, e: # pylint: disable-msg=W0703
             failed = True
@@ -894,7 +941,8 @@ class ActionQueue(ThrottlingStorageClientFactory, object):
             event = event_error
             self.event_queue.push(event_error, error=str(failure))
         except (twisted_errors.ConnectionLost,
-                twisted_errors.ConnectionDone), failure:
+                twisted_errors.ConnectionDone,
+                OpenSSL.SSL.Error), failure:
             # connection ended, just don't do anything: the SYS_CONNECTION_ETC
             # will be sent by normal client/protocol mechanisms, and logging
             # will be done later in this function.
@@ -981,12 +1029,14 @@ class ActionQueue(ThrottlingStorageClientFactory, object):
     @defer.inlineCallbacks
     def authenticate(self):
         """Authenticate against the server using stored credentials."""
+        metadata = {'version':clientdefs.VERSION,
+                    'platform':platform}
         authenticate_d = self._send_request_and_handle_errors(
             request=self.client.oauth_authenticate,
             request_error=protocol_errors.AuthenticationFailedError,
             event_error='SYS_AUTH_ERROR', event_ok='SYS_AUTH_OK',
             # XXX: handle self.token is None or self.consumer is None?
-            args=(self.consumer, self.token)
+            args=(self.consumer, self.token, metadata)
         )
         req = yield authenticate_d
 
@@ -1083,12 +1133,11 @@ class ActionQueue(ThrottlingStorageClientFactory, object):
                         path, fileobj_factory).go()
 
     def upload(self, share_id, node_id, previous_hash, hash, crc32,
-               size, path, fileobj_factory, tempfile_factory=None,
-               upload_id=None):
+               size, path, fileobj_factory, upload_id=None):
         """See .interfaces.IContentQueue."""
         return Upload(self.queue, share_id, node_id, previous_hash,
                       hash, crc32, size, path, fileobj_factory,
-                      tempfile_factory, upload_id=upload_id).go()
+                      upload_id=upload_id).go()
 
     def _cancel_op(self, share_id, node_id, cmdclass):
         """Generalized form of cancel_upload and cancel_download."""
@@ -2345,7 +2394,7 @@ class Upload(ActionQueueCommand):
     """Upload stuff to a file."""
 
     __slots__ = ('share_id', 'node_id', 'previous_hash', 'hash', 'crc32',
-                 'size', 'fileobj_factory', 'tempfile_factory', 'magic_hash',
+                 'size', 'fileobj_factory', 'magic_hash',
                  'deflated_size', 'tempfile', 'upload_req', 'tx_semaphore',
                  'n_bytes_written_last', 'path', 'n_bytes_written', 'upload_id')
 
@@ -2357,8 +2406,7 @@ class Upload(ActionQueueCommand):
     possible_markers = 'node_id',
 
     def __init__(self, request_queue, share_id, node_id, previous_hash, hash,
-                 crc32, size, path, fileobj_factory, tempfile_factory,
-                 upload_id=None):
+                 crc32, size, path, fileobj_factory, upload_id=None):
         super(Upload, self).__init__(request_queue)
         self.share_id = share_id
         self.node_id = node_id
@@ -2367,7 +2415,6 @@ class Upload(ActionQueueCommand):
         self.crc32 = crc32
         self.size = size
         self.fileobj_factory = fileobj_factory
-        self.tempfile_factory = tempfile_factory
         self.upload_id = upload_id
         self.tempfile = None
         self.path = path
@@ -2463,8 +2510,6 @@ class Upload(ActionQueueCommand):
                                            node_id=self.node_id,
                                            hash=self.hash)
 
-        if getattr(self.tempfile, 'name', None) is not None:
-            self.tempfile = open(self.tempfile.name)
         f = UploadProgressWrapper(self.tempfile, self)
 
         # access here the magic hash value, don't log anywhere, and
@@ -2500,9 +2545,8 @@ class Upload(ActionQueueCommand):
 
     def handle_success(self, request):
         """It worked! Push the event."""
-        # remove the temporary file if have one
-        if getattr(self.tempfile, 'name', None) is not None:
-            os.unlink(self.tempfile.name)
+        # remove the temporary file
+        remove_file(self.tempfile.name)
 
         # send the event
         d = dict(share_id=self.share_id, node_id=self.node_id, hash=self.hash,
@@ -2519,9 +2563,7 @@ class Upload(ActionQueueCommand):
 
     def handle_failure(self, failure):
         """It didn't work! Push the event."""
-        if getattr(self.tempfile, 'name', None) is not None:
-            os.unlink(self.tempfile.name)
-
+        remove_file(self.tempfile.name)
         self.action_queue.event_queue.push('AQ_UPLOAD_ERROR',
                                            error=failure.getErrorMessage(),
                                            share_id=self.share_id,
