@@ -19,12 +19,24 @@
 """storage shelf using a files tree."""
 
 from __future__ import with_statement
-from UserDict import DictMixin
+
 import cPickle
 import os
 import stat
 import errno
+
 from collections import deque
+from UserDict import DictMixin
+
+from ubuntuone.platform import (
+    make_dir,
+    open_file,
+    path_exists,
+    remove_file,
+    rename,
+    stat_path,
+    walk,
+)
 
 
 class FileShelf(object, DictMixin):
@@ -51,16 +63,16 @@ class FileShelf(object, DictMixin):
         creates it
         """
         try:
-            stat_result = os.stat(path)
+            stat_result = stat_path(path)
             # is a regular file?
             if stat.S_ISREG(stat_result.st_mode):
-                os.unlink(path)
-                os.makedirs(path)
+                remove_file(path)
+                make_dir(path, True)
             # else, the dir is already there
         except OSError, e:
             if e.errno == errno.ENOENT:
                 # the file or dir don't exist
-                os.makedirs(path)
+                make_dir(path, True)
             else:
                 raise
 
@@ -79,13 +91,13 @@ class FileShelf(object, DictMixin):
 
     def has_key(self, key):
         """ True if the the shelf has the key """
-        return os.path.exists(self.key_file(key))
+        return path_exists(self.key_file(key))
 
     def keys(self):
         """ returns a iterator over the keys """
         # pylint: disable-msg=W0612
         splitext = os.path.splitext
-        for dirpath, dirnames, filenames in os.walk(self._path):
+        for dirpath, dirnames, filenames in walk(self._path):
             for filename in filenames:
                 # just in case a .new file was left around
                 ext = splitext(filename)[1]
@@ -123,14 +135,14 @@ class FileShelf(object, DictMixin):
 
         """
         try:
-            with open(path, "rb") as fd:
+            with open_file(path, "rb") as fd:
                 data = self._unpickle(fd)
         except (EOFError, cPickle.UnpicklingError, ValueError):
             # the metadata is broked, try to get .old version if it's available
             old_path = path+'.old'
             # only search for a single .old in the name
             if os.path.splitext(path)[1] != '.old' and \
-               os.path.exists(old_path):
+               path_exists(old_path):
                 return self._load_pickle(key, old_path)
             else:
                 raise KeyError(key)
@@ -153,24 +165,24 @@ class FileShelf(object, DictMixin):
         new_path = path+".new"
         old_path = path+".old"
         self._check_and_create_dirs(os.path.dirname(path))
-        with open(new_path, "wb") as fh:
+        with open_file(new_path, "wb") as fh:
             self._pickle(value, fh, protocol=2)
             fh.flush()
-        if os.path.exists(path):
-            os.rename(path, old_path)
-        os.rename(new_path, path)
+        if path_exists(path):
+            rename(path, old_path)
+        rename(new_path, path)
 
     def __delitem__(self, key):
         """ delitem backed by the file storage """
         path = self.key_file(key)
         try:
-            os.remove(self.key_file(key))
+            remove_file(self.key_file(key))
         except OSError:
             raise KeyError(key)
         # also delete backup files
         for path in [path+'.old', path+'.new']:
             try:
-                os.remove(path)
+                remove_file(path)
             except OSError:
                 # ignore any OSError
                 pass
