@@ -20,7 +20,6 @@ from __future__ import with_statement
 
 import os
 import time
-import shutil
 import functools
 import itertools
 import logging
@@ -35,19 +34,24 @@ from ubuntuone.syncdaemon.interfaces import IMarker
 from ubuntuone.syncdaemon.marker import MDMarker
 from ubuntuone.syncdaemon.tritcask import TritcaskShelf
 from ubuntuone.platform import (
+    listdir,
     make_dir,
+    normpath,
     move_to_trash,
-    open_file,
     path_exists,
     remove_dir,
     remove_file,
+    remove_tree,
     rename,
+    recursive_move,
     set_dir_readonly,
     set_dir_readwrite,
     set_file_readonly,
     set_file_readwrite,
     stat_path,
+    walk,
 )
+from ubuntuone.platform import open_file as os_open
 
 METADATA_VERSION = "6"
 
@@ -334,7 +338,7 @@ class FileSystemManager(object):
         # get the metadata version
         self._version_file = os.path.join(data_dir, "metadata_version")
         if path_exists(self._version_file):
-            with open(self._version_file) as fh:
+            with os_open(self._version_file) as fh:
                 md_version = fh.read().strip()
         else:
             md_version = None
@@ -380,7 +384,8 @@ class FileSystemManager(object):
                     self._get_share(mdobj["share_id"])
                 except VolumeDoesNotExist:
                     # oops, the share is gone!, invalidate this mdid
-                    log_warning('Share %r disappeared! deleting mdid: %s', mdobj['share_id'], mdid)
+                    log_warning('Share %r disappeared! deleting mdid: %s',
+                                mdobj['share_id'], mdid)
                     del self.old_fs[mdid]
                     return False
                 else:
@@ -407,7 +412,7 @@ class FileSystemManager(object):
         for key, value in old_trash.iteritems():
             self.trash[key] = value
         # delete the old trash
-        shutil.rmtree(self._trash_dir)
+        remove_tree(self._trash_dir)
 
     def _migrate_movelimbo_to_tritcask(self):
         """Migrate move limbo from FileShelf to Tritcask."""
@@ -416,7 +421,7 @@ class FileSystemManager(object):
         for key, value in old_move_limbo.iteritems():
             self.move_limbo[key] = value
         # delete the old move limbo
-        shutil.rmtree(self._movelimbo_dir)
+        remove_tree(self._movelimbo_dir)
 
     def _load_metadata_None(self, old_version):
         """Loads metadata from when it wasn't even versioned."""
@@ -457,11 +462,11 @@ class FileSystemManager(object):
         self._migrate_trash_to_tritcask()
         self._migrate_movelimbo_to_tritcask()
         # set new version
-        with open(self._version_file, "w") as fh:
+        with os_open(self._version_file, "w") as fh:
             fh.write(METADATA_VERSION)
             os.fsync(fh.fileno())
         # remove the old metadata
-        shutil.rmtree(self.old_fs._path)
+        remove_tree(self.old_fs._path)
 
     def _load_metadata_1(self, old_version):
         """Loads metadata from version 1."""
@@ -500,11 +505,11 @@ class FileSystemManager(object):
         self._migrate_trash_to_tritcask()
         self._migrate_movelimbo_to_tritcask()
         # set new version
-        with open(self._version_file, "w") as fh:
+        with os_open(self._version_file, "w") as fh:
             fh.write(METADATA_VERSION)
             os.fsync(fh.fileno())
         # remove the old metadata
-        shutil.rmtree(self.old_fs._path)
+        remove_tree(self.old_fs._path)
 
     def _load_metadata_2(self, old_version):
         """Loads metadata from version 2."""
@@ -535,11 +540,11 @@ class FileSystemManager(object):
         self._migrate_trash_to_tritcask()
         self._migrate_movelimbo_to_tritcask()
         # set new version
-        with open(self._version_file, "w") as fh:
+        with os_open(self._version_file, "w") as fh:
             fh.write(METADATA_VERSION)
             os.fsync(fh.fileno())
         # remove the old metadata
-        shutil.rmtree(self.old_fs._path)
+        remove_tree(self.old_fs._path)
 
     def _load_metadata_3(self, old_version):
         """Loads metadata from version 3."""
@@ -564,11 +569,11 @@ class FileSystemManager(object):
         self._migrate_trash_to_tritcask()
         self._migrate_movelimbo_to_tritcask()
         # set new version
-        with open(self._version_file, "w") as fh:
+        with os_open(self._version_file, "w") as fh:
             fh.write(METADATA_VERSION)
             os.fsync(fh.fileno())
         # remove the old metadata
-        shutil.rmtree(self.old_fs._path)
+        remove_tree(self.old_fs._path)
 
     def _load_metadata_4(self, old_version):
         """Loads metadata from version 4."""
@@ -590,11 +595,11 @@ class FileSystemManager(object):
         self._migrate_trash_to_tritcask()
         self._migrate_movelimbo_to_tritcask()
         # set new version
-        with open(self._version_file, "w") as fh:
+        with os_open(self._version_file, "w") as fh:
             fh.write(METADATA_VERSION)
             os.fsync(fh.fileno())
         # remove the old metadata
-        shutil.rmtree(self.old_fs._path)
+        remove_tree(self.old_fs._path)
 
     def _load_metadata_5(self, old_version):
         """Loads metadata of last version."""
@@ -611,11 +616,11 @@ class FileSystemManager(object):
         self._migrate_trash_to_tritcask()
         self._migrate_movelimbo_to_tritcask()
         # set new version
-        with open(self._version_file, "w") as fh:
+        with os_open(self._version_file, "w") as fh:
             fh.write(METADATA_VERSION)
             os.fsync(fh.fileno())
         # remove the old metadata
-        shutil.rmtree(self.old_fs._path)
+        remove_tree(self.old_fs._path)
 
     def _load_metadata_updated(self):
         """Loads metadata of last version."""
@@ -654,7 +659,7 @@ class FileSystemManager(object):
         if not path.strip():
             raise ValueError("Empty paths are not allowed (got %r)" % path)
 
-        path = os.path.normpath(path)
+        path = normpath(path)
         if path in self._idx_path:
             raise ValueError("The path %r is already created!" % path)
 
@@ -682,7 +687,7 @@ class FileSystemManager(object):
 
     def set_node_id(self, path, node_id):
         """Sets the node_id to a md object."""
-        path = os.path.normpath(path)
+        path = normpath(path)
         mdid = self._idx_path[path]
         mdobj = self.fs[mdid]
         self._set_node_id(mdobj, node_id, path)
@@ -769,7 +774,7 @@ class FileSystemManager(object):
 
     def get_by_path(self, path):
         """Returns the md object according to the path."""
-        path = os.path.normpath(path)
+        path = normpath(path)
         mdid = self._idx_path[path]
         mdobj = self.fs[mdid]
         return _MDObject(**mdobj)
@@ -797,7 +802,7 @@ class FileSystemManager(object):
         """Set some values to the md object with that path."""
         if "mdid" in kwargs:
             raise ValueError("The mdid is forbidden to set externally")
-        path = os.path.normpath(path)
+        path = normpath(path)
         mdid = self._idx_path[path]
         self.set_by_mdid(mdid, **kwargs)
 
@@ -810,8 +815,8 @@ class FileSystemManager(object):
 
     def move_file(self, new_share_id, path_from, path_to):
         """Move a file/dir from one point to the other."""
-        path_from = os.path.normpath(path_from)
-        path_to = os.path.normpath(path_to)
+        path_from = normpath(path_from)
+        path_to = normpath(path_to)
         mdid = self._idx_path[path_from]
         mdobj = self.fs[mdid]
 
@@ -828,7 +833,7 @@ class FileSystemManager(object):
             with contextlib.nested(from_context, to_context):
                 self.eq.add_to_mute_filter(expected_event, path_from=path_from,
                                            path_to=path_to)
-                shutil.move(path_from, path_to)
+                recursive_move(path_from, path_to)
         except IOError, e:
             # file was not yet created
             self.eq.rm_from_mute_filter(expected_event,
@@ -839,8 +844,8 @@ class FileSystemManager(object):
 
     def moved(self, new_share_id, path_from, path_to):
         """Change the metadata of a moved file."""
-        path_from = os.path.normpath(path_from)
-        path_to = os.path.normpath(path_to)
+        path_from = normpath(path_from)
+        path_to = normpath(path_to)
         mdid = self._idx_path.pop(path_from)
         log_debug("move_file: mdid=%r path_from=%r path_to=%r",
                                                     mdid, path_from, path_to)
@@ -891,7 +896,7 @@ class FileSystemManager(object):
 
     def delete_metadata(self, path):
         """Delete the metadata."""
-        path = os.path.normpath(path)
+        path = normpath(path)
         mdid = self._idx_path[path]
         mdobj = self.fs[mdid]
         log_debug("delete metadata: path=%r mdid=%r", path, mdid)
@@ -915,7 +920,7 @@ class FileSystemManager(object):
                 raise DirectoryNotRemovable()
 
         # check disk searching for previous conflicts
-        for (dirpath, dirnames, filenames) in os.walk(path):
+        for (dirpath, dirnames, filenames) in walk(path):
             for fname in filenames + dirnames:
                 if fname.endswith(self.CONFLICT_SUFFIX):
                     logger("Conflicting dir on remove because of previous "
@@ -927,7 +932,7 @@ class FileSystemManager(object):
     def delete_file(self, path):
         """Delete a file/dir and the metadata."""
         # adjust the metadata
-        path = os.path.normpath(path)
+        path = normpath(path)
         mdid = self._idx_path[path]
         mdobj = self.fs[mdid]
         log_debug("delete: path=%r mdid=%r", path, mdid)
@@ -941,19 +946,21 @@ class FileSystemManager(object):
 
         try:
             if is_dir:
-                if os.listdir(path):
+                if listdir(path):
                     # not empty, need to check if we can delete it
                     subtree = self._delete_dir_tree(path=path)
                     for p, is_dir in subtree:
-                        filter_name = "FS_DIR_DELETE" if is_dir else "FS_FILE_DELETE"
+                        filter_name = "FS_DIR_DELETE" if is_dir \
+                                                      else "FS_FILE_DELETE"
                         self.eq.add_to_mute_filter(filter_name, path=p)
                         self.delete_metadata(p)
 
-                    with self._enable_share_write(mdobj['share_id'], path, recursive=True):
+                    with self._enable_share_write(mdobj['share_id'], path,
+                                                  recursive=True):
                         if self.user_config.get_use_trash():
                             move_to_trash(path)
                         else:
-                            shutil.rmtree(path)
+                            remove_tree(path)
                 else:
                     # empty, just delete it
                     with self._enable_share_write(mdobj['share_id'], path):
@@ -1080,9 +1087,10 @@ class FileSystemManager(object):
             while True:
                 try:
                     # don't alert EQ, partials are in other directory, not watched
-                    open(partial_path, "w").close()
+                    os_open(partial_path, "w").close()
                 except IOError, e:
-                    if e.errno == errno.ENAMETOOLONG:
+                    # linux will give you too long, windows will say invalid
+                    if e.errno in (errno.ENAMETOOLONG, errno.EINVAL):
                         trim += 1
                         partial_path = self._get_partial_path(mdobj, trim=trim)
                     else:
@@ -1100,7 +1108,7 @@ class FileSystemManager(object):
 
         mdobj = self.fs[mdid]
         partial_path = self._get_partial_path(mdobj)
-        return open(partial_path, "w")
+        return os_open(partial_path, "wb")
 
     def get_partial(self, node_id, share_id):
         """Get a read-only fd to a partial file."""
@@ -1110,7 +1118,7 @@ class FileSystemManager(object):
                              "not partial!" % (share_id, node_id))
 
         partial_path = self._get_partial_path(self.fs[mdid])
-        fd = open(partial_path, "r")
+        fd = os_open(partial_path, "rb")
         return fd
 
     def commit_partial(self, node_id, share_id, local_hash):
@@ -1132,7 +1140,7 @@ class FileSystemManager(object):
         with self._enable_share_write(share_id, path):
             self.eq.add_to_mute_filter("FS_FILE_CREATE", path=path)
             self.eq.add_to_mute_filter("FS_FILE_CLOSE_WRITE", path=path)
-            shutil.move(partial_path, path)
+            recursive_move(partial_path, path)
         mdobj["local_hash"] = local_hash
         mdobj["info"]["last_downloaded"] = time.time()
         mdobj["info"]["is_partial"] = False
@@ -1171,7 +1179,7 @@ class FileSystemManager(object):
     def _get_mdid_from_args(self, kwargs, parent):
         """Parse the kwargs and gets the mdid."""
         if len(kwargs) == 1 and "path" in kwargs:
-            path = os.path.normpath(kwargs["path"])
+            path = normpath(kwargs["path"])
             return self._idx_path[path]
         if len(kwargs) == 1 and "mdid" in kwargs:
             return kwargs["mdid"]
@@ -1188,7 +1196,7 @@ class FileSystemManager(object):
     def has_metadata(self, **kwargs):
         """Return True if there's metadata for a given object."""
         if len(kwargs) == 1 and "path" in kwargs:
-            path = os.path.normpath(kwargs["path"])
+            path = normpath(kwargs["path"])
             return path in self._idx_path
         if len(kwargs) == 1 and "mdid" in kwargs:
             return kwargs["mdid"] in self.fs
@@ -1247,7 +1255,7 @@ class FileSystemManager(object):
 
     def dir_content(self, path):
         """Return the content of the directory in a server-comparable way."""
-        path = os.path.normpath(path)
+        path = normpath(path)
         mdid = self._idx_path[path]
         mdobj = self.fs[mdid]
         if not mdobj["is_dir"]:
@@ -1269,7 +1277,8 @@ class FileSystemManager(object):
         if mdobj["is_dir"]:
             raise ValueError("You can only open files, not directories.")
 
-        return open_file(self.get_abspath(mdobj['share_id'], mdobj['path']))
+        return os_open(self.get_abspath(mdobj['share_id'], mdobj['path']),
+                       'rb')
 
     def _share_relative_path(self, share_id, path):
         """Return the relative path from the share_id."""
@@ -1301,7 +1310,7 @@ class FileSystemManager(object):
             # the relaitve path is the fullpath
             return share_path
         else:
-            return os.path.join(share_path, path)
+            return os.path.abspath(os.path.join(share_path, path))
 
     def _enable_share_write(self, share_id, path, recursive=False):
         """Helper to create a EnableShareWrite context manager."""
@@ -1526,7 +1535,7 @@ class EnableShareWrite(object):
 
         # if needed, change the whole subtree
         if self.recursive:
-            for dirpath, dirnames, filenames in os.walk(self.path, topdown=False):
+            for dirpath, dirnames, filenames in walk(self.path, topdown=False):
                 for dname in dirnames:
                     path = os.path.join(dirpath, dname)
                     set_dir_readwrite(path)
@@ -1554,7 +1563,7 @@ class EnableShareWrite(object):
                 set_file_readonly(self.path)
 
         # restore all saved ones
-        exists = os.path.exists
+        exists = path_exists
         for path, isdir in self._changed_nodes[::-1]:
             if exists(path):
                 if isdir:
@@ -1575,4 +1584,3 @@ def get_stat(path):
             return None
         else:
             raise
-
