@@ -22,6 +22,7 @@ import uuid
 
 from ubuntuone.syncdaemon import config
 from ubuntuone.syncdaemon.action_queue import Download, Upload
+from ubuntuone.platform import normpath
 
 logger = logging.getLogger("ubuntuone.SyncDaemon.InteractionInterfaces")
 
@@ -115,6 +116,10 @@ class SyncdaemonStatus(object):
                 current_downloads.append(entry)
         return current_downloads
 
+    def free_space(self, vol_id):
+        """Return the free space for the given volume."""
+        return self.main.vm.get_free_space(str(vol_id))
+
     def waiting(self):
         """Return a list of the operations in action queue."""
         logger.debug('called waiting')
@@ -122,7 +127,7 @@ class SyncdaemonStatus(object):
         for cmd in self.action_queue.queue.waiting:
             operation = cmd.__class__.__name__
             data = cmd.to_dict()
-            waiting.append((operation, data))
+            waiting.append((operation, str(id(cmd)), data))
         return waiting
 
     def waiting_metadata(self):
@@ -266,6 +271,7 @@ class SyncdaemonFileSystem(object):
             dirty_nodes.append(self._mdobj_dict(mdobj))
         return dirty_nodes
 
+
 class SyncdaemonShares(object):
     """An interface to interact with shares."""
 
@@ -597,8 +603,8 @@ class SyncdaemonFolders(object):
     def create(self, path):
         """Create a user defined folder in the specified path."""
         logger.debug('Folders.create: %r', path)
-        path = os.path.normpath(path)
-        self.vm.create_udf(path.encode('utf-8'))
+        path = normpath(path.encode('utf-8'))
+        self.vm.create_udf(path)
 
     def delete(self, folder_id):
         """Delete the folder specified by folder_id"""
@@ -642,6 +648,7 @@ class SyncdaemonFolders(object):
         """Refresh the volumes list, requesting it to the server."""
         self.vm.refresh_volumes()
 
+
 class SyncdaemonPublicFiles(object):
     """A DBus interface for handling public files."""
 
@@ -698,6 +705,7 @@ class SyncdaemonEvents(object):
         str_args = dict((str(k), str(v)) for k, v in args.items())
         self.event_queue.push(str(event_name), **str_args)
 
+
 class SyncdaemonService(object):
     """ The Daemon dbus interface. """
 
@@ -928,6 +936,7 @@ class SyncdaemonEventListener(object):
 
     def handle_SYS_STATE_CHANGED(self, state):
         """Handle SYS_STATE_CHANGED."""
+        logger.debug('emitting state changed: %r', state)
         self.interface.status.emit_status_changed(state)
 
     def handle_SV_FREE_SPACE(self, share_id, free_bytes):
