@@ -24,14 +24,14 @@ import os
 import zlib
 
 from ubuntuone.logger import (
-    LOGFOLDER,
     _DEBUG_LOG_LEVEL,
-    Logger,
-    DayRotatingFileHandler,
-    CustomRotatingFileHandler,
-    MultiFilter,
     basic_formatter,
+    CustomRotatingFileHandler,
+    DayRotatingFileHandler,
+    Logger,
+    MultiFilter,
 )
+from ubuntuone.platform.xdg_base_directory import ubuntuone_log_dir
 # api compatibility imports
 from ubuntuone import logger
 from ubuntuone.platform import get_filesystem_logger, setup_filesystem_logging
@@ -39,7 +39,8 @@ DebugCapture = logger.DebugCapture
 NOTE = logger.NOTE
 TRACE = logger.TRACE
 
-# pylint: disable-msg=C0103
+
+# pylint: disable=C0103
 class mklog(object):
     """
     Create a logger that keeps track of the method where it's being
@@ -121,11 +122,12 @@ class mklog(object):
             self.debug('traceback follows:\n\n' + failure.getTraceback(), '')
             return failure
         return callback, errback
-# pylint: enable-msg=C0103
-LOGFILENAME = os.path.join(LOGFOLDER, 'syncdaemon.log')
-EXLOGFILENAME = os.path.join(LOGFOLDER, 'syncdaemon-exceptions.log')
-INVALIDLOGFILENAME = os.path.join(LOGFOLDER, 'syncdaemon-invalid-names.log')
-BROKENLOGFILENAME = os.path.join(LOGFOLDER, 'syncdaemon-broken-nodes.log')
+
+# pylint: enable=C0103
+LOGFILENAME = os.path.join(ubuntuone_log_dir, 'syncdaemon.log')
+EXLOGFILENAME = os.path.join(ubuntuone_log_dir, 'syncdaemon-exceptions.log')
+INVALIDLOGFILENAME = os.path.join(ubuntuone_log_dir, 'syncdaemon-invalid-names.log')
+BROKENLOGFILENAME = os.path.join(ubuntuone_log_dir, 'syncdaemon-broken-nodes.log')
 
 
 root_logger = logging.getLogger("ubuntuone.SyncDaemon")
@@ -153,7 +155,7 @@ def init():
     # add the exception handler to the root logger
     logging.getLogger('').addHandler(exception_handler)
     root_logger.addHandler(exception_handler)
-    
+
     # hook twisted.python.log with standard logging
     from twisted.python import log
     observer = log.PythonLoggingObserver('twisted')
@@ -162,9 +164,9 @@ def init():
     twisted_logger.propagate = False
     twisted_logger.setLevel(logging.ERROR)
     twisted_logger.addHandler(root_handler)
-    twisted_logger.addHandler(exception_handler)   
-    
-    # set the filesystem logging    
+    twisted_logger.addHandler(exception_handler)
+
+    # set the filesystem logging
     setup_filesystem_logging(filesystem_logger, root_handler)
 
     # invalid filenames log
@@ -174,7 +176,7 @@ def init():
     invnames_handler.setFormatter(basic_formatter)
     invnames_handler.setLevel(logging.INFO)
     invnames_logger.addHandler(invnames_handler)
-    
+
     # broken nodes log
     brokennodes_logger = logging.getLogger("ubuntuone.SyncDaemon.BrokenNodes")
     brokennodes_logger.setLevel(_DEBUG_LOG_LEVEL)
@@ -221,7 +223,7 @@ def set_debug(dest):
     if 'file' in dest:
         # setup the existing loggers in debug
         root_handler.setLevel(_DEBUG_LOG_LEVEL)
-        logfile = os.path.join(LOGFOLDER, 'syncdaemon-debug.log')
+        logfile = os.path.join(ubuntuone_log_dir, 'syncdaemon-debug.log')
         root_handler.baseFilename = os.path.abspath(logfile)
         # don't cap the file size
         set_max_bytes(0)
@@ -252,8 +254,8 @@ def set_server_debug(dest):
     logger = logging.getLogger("storage.server")
     logger.setLevel(5) # this shows server messages
     if 'file' in dest:
-        handler = DayRotatingFileHandler(filename=os.path.join(LOGFOLDER,
-                                                'syncdaemon-debug.log'))
+        filename = os.path.join(ubuntuone_log_dir, 'syncdaemon-debug.log')
+        handler = DayRotatingFileHandler(filename=filename)
         handler.setFormatter(basic_formatter)
         handler.setLevel(5) # this shows server messages
         logger.addHandler(handler)
