@@ -55,6 +55,7 @@ LIVE = '.live'
 INACTIVE = '.inactive'
 HINT = '.hint'
 DEAD = '.dead'
+BROKEN = '.broken'
 
 VERSION = 'v1'
 FILE_SUFFIX = '.tritcask-%s.data' % VERSION
@@ -73,6 +74,7 @@ class BadCrc(Exception):
 class BadHeader(Exception):
     """A Exception for Bad header value."""
 
+
 TritcaskEntry = namedtuple('TritcaskEntry', ['crc32', 'tstamp', 'key_sz',
                          'value_sz', 'row_type', 'key', 'value', 'value_pos'])
 
@@ -138,9 +140,33 @@ def is_dead(filename):
     return DEAD in filename
 
 
+class WindowsTimer(object):
+    """A simmple class to get a more precise timestamp in windows."""
+
+    def __init__(self):
+        # start counting from now (and start the clock)
+        self.start = time.time() + time.clock()
+
+    def time(self):
+        """Return a float timestamp.
+
+        The timestamp is start_time + time.clock().
+        """
+        return self.start + time.clock()
+
+
+if sys.platform == 'win32':
+    _timer = WindowsTimer()
+    timestamp = _timer.time
+else:
+    timestamp = time.time
+
+
 class DataFile(object):
     """Class that encapsulates data file handling."""
 
+    last_generated_id = 0
+
     def __init__(self, base_path, filename=None):
         """Create a DataFile instance.
 
@@ -148,22 +174,28 @@ class DataFile(object):
         """
         self.has_bad_crc = False
         self.has_bad_data = False
+        self.fd = None
         if filename is None:
             filename = self._get_next_file_id() + LIVE + FILE_SUFFIX
         self.filename = os.path.join(base_path, filename)
         self.file_id = _get_file_id(filename)
         self.hint_filename = os.path.join(base_path, filename + HINT)
-        self.fd = None
         self._open()
 
     @property
     def size(self):
         return os.stat(self.filename).st_size
 
-    @staticmethod
-    def _get_next_file_id():
+    @classmethod
+    def _get_next_file_id(cls):
         """Return the next file id."""
-        return str(int(time.time() * 100000))
+        generated_id = int(timestamp() * 1000000)
+        if generated_id <= cls.last_generated_id:
+            logger.warning('Repeated timestamps, probably running under '
+                           'VirtualBox. Using workaround.')
+            generated_id = cls.last_generated_id + 1
+        cls.last_generated_id = generated_id
+        return str(generated_id)
 
     @property
     def has_hint(self):
@@ -236,7 +268,7 @@ class DataFile(object):
         """Write an entry to the file."""
         key_sz = len(key)
         value_sz = len(value)
-        tstamp = time.time()
+        tstamp = timestamp()
         header = header_struct.pack(tstamp, key_sz, value_sz, row_type)
         crc32 = crc32_struct.pack(zlib.crc32(header+key+value))
         if EXTRA_SEEK:
@@ -323,7 +355,12 @@ class ImmutableDataFile(DataFile):
         if self.fmmap:
             self.fmmap.close()
             self.fmmap = None
-        super(ImmutableDataFile, self).close()
+        # I don't call parent close, as I'm a read-only file and there is no
+        # need to call flush nor fsync (and windows hate it and dies in
+        # horrible ways)
+        if self.fd:
+            self.fd.close()
+            self.fd = None
 
     def iter_entries(self):
         """Return a generator for the entries in the mmaped file."""
@@ -383,6 +420,7 @@ class TempDataFile(DataFile):
 
     def delete(self):
         """Delete this file and the hint if exists."""
+        self.close()
         os.unlink(self.filename)
         if self.has_hint:
             os.unlink(self.hint_filename)
@@ -513,7 +551,6 @@ class Keydir(dict):
         return self._stats[file_id].copy()
 
 
-
 class Tritcask(object):
     """Implementation of a bitcask-like (row_type,key)/value store.
 
@@ -604,25 +641,30 @@ class Tritcask(object):
         # now check the default rotation policy
         try:
             live_file_stats = self._keydir.get_stats(self.live_file.file_id)
-            return (live_file_stats['live_bytes'] / self.live_file.size) \
-                    < self.dead_bytes_threshold
         except KeyError:
             # no info for the live file
             return False
+        else:
+            return (live_file_stats['live_bytes'] / self.live_file.size) \
+                    < self.dead_bytes_threshold
 
     def should_merge(self, immutable_files):
         """Check if the immutable_files should be merged."""
         if not immutable_files:
             return False
-        file_ids = sorted(immutable_files.keys())
-        live_bytes = sum([self._keydir.get_stats(file_id)['live_bytes'] \
-                          for file_id in file_ids])
+        live_bytes = 0
+        for file_id in sorted(immutable_files.keys()):
+            try:
+                stats = self._keydir.get_stats(file_id)
+            except KeyError:
+                # no stats for this data file
+                pass
+            else:
+                live_bytes += stats['live_bytes']
         total_bytes = sum([f.size for f in immutable_files.values()])
         merge_dead = (live_bytes / total_bytes) < self.dead_bytes_threshold
-        if merge_dead:
-            return True
-        # now check if we should merge using the max_immutable_files setting.
-        if len(self._immutable) > self.max_immutable_files:
+        # check if we should merge using the max_immutable_files setting.
+        if merge_dead or len(self._immutable) > self.max_immutable_files:
             return True
         # shouldn't merge.
         return False
@@ -645,23 +687,46 @@ class Tritcask(object):
         """Collect the files we need to work with."""
         logger.debug("lookingup data files")
         dead_files = 0
+        broken_files = 0
+        # becuase listdir appends / at the end of the path if there is not
+        # os.path.sep at the end we are going to add it, otherwhise on windows
+        # we will have wrong paths formed
+        base_path = self.base_path
+        if not base_path.endswith(os.path.sep):
+            base_path += os.path.sep
         files = os.listdir(self.base_path)
         for filename in files:
+            # first check for hint and dead
             if is_hint(filename):
                 continue
-            if is_live(filename):
-                self.live_file = DataFile(self.base_path, filename)
-            elif is_immutable(filename):
-                # it's an immutable file
-                data_file = ImmutableDataFile(self.base_path, filename)
-                self._immutable[data_file.file_id] = data_file
             elif is_dead(filename):
                 # a dead file...let's remove it
                 dead_files += 1
                 DeadDataFile(self.base_path, filename).delete()
+                continue
+            # if it's a live or immutable file try to open it, but if it's
+            # "broken", just rename it and continue
+            try:
+                if is_live(filename):
+                    self.live_file = DataFile(self.base_path, filename)
+                elif is_immutable(filename):
+                    # it's an immutable file
+                    data_file = ImmutableDataFile(self.base_path, filename)
+                    self._immutable[data_file.file_id] = data_file
+            except IOError, e:
+                # oops, failed to open the file..discard it
+                broken_files += 1
+                orig = os.path.join(self.base_path, filename)
+                # get the kind of the file so we can rename it
+                kind = LIVE if is_live(filename) else INACTIVE
+                dest = orig.replace(kind, BROKEN)
+                logger.warning("Failed to open %s, renaming it to: %s - "
+                               "error: %s", orig, dest, e)
+                # rename it to "broken"
+                os.rename(orig, dest)
         # immutable files + live
-        logger.info("found %s data files and %s dead files",
-                    len(self._immutable)+1, dead_files)
+        logger.info("found %s data files, %s dead and %s broken files",
+                    len(self._immutable)+1, dead_files, broken_files)
 
     def rotate(self, create_file=True):
         """Rotate the live file only if it already exsits."""
@@ -887,4 +952,3 @@ class TritcaskShelf(object, DictMixin):
     def _serialize(self, value):
         """Serialize value to string using protocol."""
         return cPickle.dumps(value, protocol=cPickle.HIGHEST_PROTOCOL)
-
