@@ -23,36 +23,49 @@ import functools
 import cPickle
 import logging
 import os
+import re
 import sys
+import stat
+
 from itertools import ifilter
 
+from twisted.internet import defer
 from ubuntuone.storageprotocol import request
 from ubuntuone.storageprotocol.volumes import (
     ShareVolume,
     UDFVolume,
     RootVolume,
 )
+
 from ubuntuone.syncdaemon.marker import MDMarker
 from ubuntuone.syncdaemon.interfaces import IMarker
 from ubuntuone.syncdaemon import file_shelf, config
 from ubuntuone.syncdaemon.tritcask import TritcaskShelf
-
-from ubuntuone.platform import (
-    VMMetadataUpgraderMixIn,
-    allow_writes,
+from ubuntuone.syncdaemon.vm_helper import (
     create_shares_link,
+    get_share_dir_name,
     get_udf_path,
-    get_share_path,
-    get_udf_path_name,
+    get_udf_suggested_path,
+)
+from ubuntuone.platform import (
+    allow_writes,
+    get_path_list,
+    is_link,
+    listdir,
+    make_dir,
+    normpath,
+    path_exists,
+    read_link,
+    recursive_move,
+    remove_file,
+    remove_dir,
+    remove_tree,
+    rename,
     set_dir_readonly,
     set_dir_readwrite,
-    path_exists,
-    make_dir,
+    walk,
 )
 
-from twisted.internet import defer
-
-
 # tritcask row types
 SHARE_ROW_TYPE = 3
 SHARED_ROW_TYPE = 4
@@ -72,7 +85,7 @@ class _Share(object):
         if path is None:
             self.path = None
         else:
-            self.path = os.path.normpath(path)
+            self.path = normpath(path)
         self.id = str(share_id)
         self.access_level = access_level
         self.accepted = accepted
@@ -151,7 +164,7 @@ class Share(Volume):
         if path is None:
             self.path = None
         else:
-            self.path = os.path.normpath(path)
+            self.path = normpath(path)
         self.name = name
         self.other_username = other_username
         self.other_visible_name = other_visible_name
@@ -254,6 +267,8 @@ class Shared(Share):
 class Root(Volume):
     """A volume representing the root."""
 
+    subscribed = True
+
     def __init__(self, volume_id=request.ROOT, node_id=None, path=None,
                  free_bytes=None, generation=None):
         """Create the Root."""
@@ -304,11 +319,15 @@ class UDF(Volume):
 
     @property
     def ancestors(self):
-        """Calculate all the ancestors for this UDF's path."""
-        user_home = os.path.abspath(os.path.expanduser('~'))
-        path_list = os.path.abspath(self.path).split(os.path.sep)
+        """Calculate all the ancestors for this UDF's path.
+
+        Return a list of paths that are 'syncdaemon' valid.
+
+        """
+        user_home = os.path.expanduser('~')
+        path_list = get_path_list(self.path)
         common_prefix = os.path.commonprefix([self.path, user_home])
-        common_list = common_prefix.split(os.path.sep)
+        common_list = get_path_list(common_prefix)
 
         result = [user_home]
         for p in path_list[len(common_list):-1]:
@@ -398,11 +417,11 @@ class VolumeManager(object):
 
         # create the metadata directories
         if not path_exists(self._shares_dir):
-            os.makedirs(self._shares_dir)
+            make_dir(self._shares_dir, recursive=True)
         if not path_exists(self._shared_dir):
-            os.makedirs(self._shared_dir)
+            make_dir(self._shared_dir, recursive=True)
         if not path_exists(self._udfs_dir):
-            os.makedirs(self._udfs_dir)
+            make_dir(self._udfs_dir, recursive=True)
 
         # load the metadata
         self.shares = VMTritcaskShelf(SHARE_ROW_TYPE, self.m.db)
@@ -622,7 +641,7 @@ class VolumeManager(object):
         """
         self.log.debug('_handle_new_volume: volume %r.', volume)
         if isinstance(volume, ShareVolume):
-            dir_name = get_share_path(volume)
+            dir_name = get_share_dir_name(volume)
             path = os.path.join(self.m.shares_dir, dir_name)
             share = Share.from_share_volume(volume, path)
             autosubscribe = config.get_user_config().get_share_autosubscribe()
@@ -658,7 +677,7 @@ class VolumeManager(object):
             self.log.debug('share %r: id=%s, name=%r', a_share.direction,
                            share_id, a_share.name)
             if a_share.direction == "to_me":
-                dir_name = get_share_path(a_share)
+                dir_name = get_share_dir_name(a_share)
                 path = os.path.join(self.m.shares_dir, dir_name)
                 share = Share.from_response(a_share, path)
                 shares.append(share.volume_id)
@@ -740,7 +759,7 @@ class VolumeManager(object):
             self.log.debug("New share notification, share_id: %s",
                            info.share_id)
             # XXX: request a refresh of the shares list(?)
-            dir_name = get_share_path(info)
+            dir_name = get_share_dir_name(info)
             path = os.path.join(self.m.shares_dir, dir_name)
             share = Share.from_notify_holder(info, path)
             self.add_share(share)
@@ -932,6 +951,7 @@ class VolumeManager(object):
             self.shares[share.volume_id] = share
             self.m.event_q.push('VM_SHARE_CHANGED', share_id=share.volume_id)
 
+    @defer.inlineCallbacks
     def _create_share_dir(self, share):
         """ Creates the share root dir, and set the permissions. """
         # XXX: verterok: This is going to be moved into fsm
@@ -940,9 +960,9 @@ class VolumeManager(object):
             with allow_writes(os.path.dirname(share.path)):
                 make_dir(share.path)
         # add the watch after the mkdir
-        if share.can_write() and not self.m.event_q.has_watch(share.path):
+        if share.can_write():
              self.log.debug('adding inotify watch to: %s', share.path)
-             self.m.event_q.add_watch(share.path)
+             yield self._add_watch(share.path)
         # if it's a ro share, change the perms
         if not share.can_write():
             set_dir_readonly(share.path)
@@ -977,11 +997,15 @@ class VolumeManager(object):
         for a_path, _ in self.m.fs.get_paths_starting_with(path):
             self.m.fs.delete_metadata(a_path)
 
+    def _add_watch(self, path):
+        """Add a inotify watch to path."""
+        return self.m.event_q.add_watch(path)
+
     def _remove_watch(self, path):
         """Remove the inotify watch from path."""
         try:
             self.m.event_q.rm_watch(path)
-        except (ValueError, RuntimeError, TypeError), e:
+        except (ValueError, RuntimeError, TypeError, KeyError), e:
             # pyinotify has an ugly error management, if we can call
             # it that, :(. We handle this here because it's possible
             # and correct that the path is not there anymore
@@ -996,7 +1020,7 @@ class VolumeManager(object):
 
     def create_share(self, path, username, name, access_level):
         """ create a share for the specified path, username, name """
-        self.log.debug('create share(%r, %s, %s, %s)',
+        self.log.debug('create share(%r, %r, %r, %r)',
                        path, username, name, access_level)
         mdobj = self.m.fs.get_by_path(path)
         mdid = mdobj.mdid
@@ -1012,6 +1036,7 @@ class VolumeManager(object):
                       other_username=username, other_visible_name=None,
                       node_id=node_id)
         self.marker_share_map[marker] = share
+        # XXX: unicode boundary! username, name should be unicode
         self.m.action_q.create_share(node_id, username, name,
                                      access_level, marker, abspath)
 
@@ -1132,11 +1157,12 @@ class VolumeManager(object):
         it's inside the root.
 
         """
+        new_path = path + os.path.sep
         volumes = itertools.chain([self.shares[request.ROOT]],
                                    self.udfs.values())
         for volume in volumes:
             vol_path = volume.path + os.path.sep
-            if path.startswith(vol_path) or vol_path.startswith(path):
+            if new_path.startswith(vol_path) or vol_path.startswith(new_path):
                 return True
         return False
 
@@ -1144,7 +1170,7 @@ class VolumeManager(object):
         """Request the creation of a UDF to AQ."""
         self.log.debug('create udf: %r', path)
         # check if path is the realpath, bail out if not
-        if os.path.realpath(path) != path:
+        if is_link(path):
             self.m.event_q.push('VM_UDF_CREATE_ERROR', path=path,
                                 error="UDFs can not be a symlink")
             return
@@ -1156,7 +1182,7 @@ class VolumeManager(object):
             return
 
         try:
-            udf_path, udf_name = get_udf_path_name(path)
+            suggested_path = get_udf_suggested_path(path)
         except ValueError, e:
             self.m.event_q.push('VM_UDF_CREATE_ERROR', path=path,
                                 error="INVALID_PATH: %s" % (e,))
@@ -1166,16 +1192,16 @@ class VolumeManager(object):
                 if marker in self.marker_udf_map:
                     # ignore this request
                     self.log.warning('Duplicated create_udf request for '
-                                     'path (ingoring it!): %r', udf_path)
+                                     'path (ingoring it!): %r', path)
                     return
-                udf = UDF(None, None,
-                          # suggested_path is always unicode
-                          os.path.join(udf_path, udf_name).decode('utf-8'),
+                udf = UDF(volume_id=None, node_id=None,
+                          suggested_path=suggested_path,
                           # always subscribed since it's a local request
-                          path, subscribed=True)
+                          path=path, subscribed=True)
                 self.marker_udf_map[marker] = udf
-                self.m.action_q.create_udf(udf_path.decode('utf-8'),
-                                           udf_name.decode('utf-8'), marker)
+                # XXX: unicode boundary! parameters should be unicode
+                server_path, udf_name = suggested_path.rsplit(u'/', 1)
+                self.m.action_q.create_udf(server_path, udf_name, marker)
             except Exception, e:
                 self.m.event_q.push('VM_UDF_CREATE_ERROR', path=path,
                                     error="UNKNOWN_ERROR: %s" % (e,))
@@ -1388,7 +1414,7 @@ class VolumeManager(object):
             self.udfs[volume_id] = vol
 
 
-class MetadataUpgrader(VMMetadataUpgraderMixIn):
+class MetadataUpgrader(object):
     """A class that loads old metadata and migrate it."""
 
     def __init__(self, data_dir, shares_md_dir, shared_md_dir, udfs_md_dir,
@@ -1416,12 +1442,12 @@ class MetadataUpgrader(VMMetadataUpgraderMixIn):
 
     def _get_md_version(self):
         """Returns the current md_version"""
-        if not os.path.exists(self._data_dir):
+        if not path_exists(self._data_dir):
             # first run, the data dir don't exist. No metadata to upgrade
             md_version = VolumeManager.METADATA_VERSION
-            os.makedirs(self._data_dir)
+            make_dir(self._data_dir, recursive=True)
             self.update_metadata_version()
-        elif os.path.exists(self._version_file):
+        elif path_exists(self._version_file):
             with open(self._version_file) as fh:
                 md_version = fh.read().strip()
             if not md_version:
@@ -1436,18 +1462,359 @@ class MetadataUpgrader(VMMetadataUpgraderMixIn):
 
     def update_metadata_version(self):
         """Write the version of the metadata."""
-        if not os.path.exists(os.path.dirname(self._version_file)):
-            os.makedirs(os.path.dirname(self._version_file))
+        if not path_exists(os.path.dirname(self._version_file)):
+            make_dir(os.path.dirname(self._version_file), recursive=True)
         with open(self._version_file, 'w') as fd:
             fd.write(VolumeManager.METADATA_VERSION)
             # make sure the data get to disk
             fd.flush()
             os.fsync(fd.fileno())
 
+    def _guess_metadata_version(self):
+        """Try to guess the metadata version based on current metadata
+        and layout, fallbacks to md_version = None if can't guess it.
+
+        """
+        md_version = None
+        if path_exists(self._shares_md_dir) \
+           and path_exists(self._shared_md_dir):
+            # we have shares and shared dirs
+            # md_version >= 1
+            old_root_dir = os.path.abspath(os.path.join(self._root_dir, 'My Files'))
+            old_share_dir = os.path.abspath(os.path.join(self._root_dir,
+                                                 'Shared With Me'))
+            if path_exists(old_share_dir) and path_exists(old_root_dir) \
+               and not is_link(old_share_dir):
+                # md >= 1 and <= 3
+                # we have a My Files dir, 'Shared With Me' isn't a
+                # symlink and ~/.local/share/ubuntuone/shares doesn't
+                # exists.
+                # md_version <= 3, set it to 2 as it will migrate
+                # .conflict to .u1conflict, and we don't need to upgrade
+                # from version 1 any more as the LegacyShareFileShelf
+                # takes care of that.
+                md_version = '2'
+            else:
+                try:
+                    target = read_link(self._shares_dir_link)
+                except OSError:
+                    target = None
+                if is_link(self._shares_dir_link) \
+                   and normpath(target) == os.path.abspath(self._shares_dir_link):
+                    # broken symlink, md_version = 4
+                    md_version = '4'
+                else:
+                    # md_version >= 5
+                    shelf = LegacyShareFileShelf(self._shares_md_dir)
+                    # check a pickled value to check if it's in version
+                    # 5 or 6
+                    md_version = '5'
+                    versions = {'5':0, '6':0}
+                    for key in shelf:
+                        share = shelf[key]
+                        if isinstance(share, _Share):
+                            versions['5'] += 1
+                        else:
+                            versions['6'] += 1
+                    if versions['5'] > 0:
+                        md_version = '5'
+                    elif versions['6'] > 0:
+                        md_version = '6'
+        else:
+            # this is metadata 'None'
+            md_version = None
+        return md_version
+
+    def _upgrade_metadata_None(self, md_version):
+        """Upgrade the shelf layout, for *very* old clients."""
+        from ubuntuone.syncdaemon.volume_manager import LegacyShareFileShelf
+        self.log.debug('Upgrading the share shelf layout')
+        # the shelf already exists, and don't have a .version file
+        # first backup the old data
+        backup = os.path.join(self._data_dir, '0.bkp')
+        if not path_exists(backup):
+            make_dir(backup, recursive=True)
+        # pylint: disable-msg=W0612
+        # filter 'shares' and 'shared' dirs, in case we are in the case of
+        # missing version but existing .version file
+        filter_known_dirs = lambda d: d != os.path.basename(self._shares_md_dir)\
+                and d != os.path.basename(self._shared_md_dir)
+        for dirname, dirs, files in walk(self._data_dir):
+            if dirname == self._data_dir:
+                for dir in filter(filter_known_dirs, dirs):
+                    if dir != os.path.basename(backup):
+                        recursive_move(os.path.join(dirname, dir),
+                                    os.path.join(backup, dir))
+        # regenerate the shelf using the new layout using the backup as src
+        old_shelf = LegacyShareFileShelf(backup)
+        if not path_exists(self._shares_dir):
+            make_dir(self._shares_dir, recursive=True)
+        new_shelf = LegacyShareFileShelf(self._shares_md_dir)
+        for key, share in old_shelf.iteritems():
+            new_shelf[key] = share
+        # now upgrade to metadata 2
+        self._upgrade_metadata_2(md_version)
+
+    def _upgrade_metadata_1(self, md_version):
+        """Upgrade to version 2.
+
+        Upgrade all pickled Share to the new package/module layout.
+
+        """
+        from ubuntuone.syncdaemon.volume_manager import LegacyShareFileShelf
+        self.log.debug('upgrading share shelfs from metadata 1')
+        shares = LegacyShareFileShelf(self._shares_md_dir)
+        for key, share in shares.iteritems():
+            shares[key] = share
+        shared = LegacyShareFileShelf(self._shared_md_dir)
+        for key, share in shared.iteritems():
+            shared[key] = share
+        # now upgrade to metadata 3
+        self._upgrade_metadata_2(md_version)
+
+    def _upgrade_metadata_2(self, md_version):
+        """Upgrade to version 3
+
+        Renames foo.conflict files to foo.u1conflict, foo.conflict.N
+        to foo.u1conflict.N, foo.partial to .u1partial.foo, and
+        .partial to .u1partial.
+
+        """
+        from ubuntuone.platform.linux import allow_writes
+        self.log.debug('upgrading from metadata 2 (bogus)')
+        for top in self._root_dir, self._shares_dir:
+            for dirpath, dirnames, filenames in walk(top):
+                with allow_writes(dirpath):
+                    for names in filenames, dirnames:
+                        self._upgrade_names(dirpath, names)
+        self._upgrade_metadata_3(md_version)
+
+    def _upgrade_names(self, dirpath, names):
+        """Do the actual renaming for _upgrade_metadata_2."""
+        for pos, name in enumerate(names):
+            new_name = name
+            if re.match(r'.*\.partial$|\.u1partial(?:\..+)?', name):
+                if name == '.partial':
+                    new_name = '.u1partial'
+                else:
+                    new_name = re.sub(r'^(.+)\.partial$',
+                                      r'.u1partial.\1', name)
+                if new_name != name:
+                    while os.path.lexists(os.path.join(dirpath, new_name)):
+                        # very, very strange
+                        self.log.warning('Found a .partial and .u1partial'
+                                         ' for the same file: %s!', new_name)
+                        new_name += '.1'
+            elif re.search(r'\.(?:u1)?conflict(?:\.\d+)?$', name):
+                new_name = re.sub(r'^(.+)\.conflict((?:\.\d+)?)$',
+                                  r'\1.u1conflict\2', name)
+                if new_name != name:
+                    while os.path.lexists(os.path.join(dirpath, new_name)):
+                        m = re.match(r'(.*\.u1conflict)((?:\.\d+)?)$', new_name)
+                        base, num = m.groups()
+                        if not num:
+                            num = '.1'
+                        else:
+                            num = '.' + str(int(num[1:])+1)
+                        new_name = base + num
+            if new_name != name:
+                old_path = os.path.join(dirpath, name)
+                new_path = os.path.join(dirpath, new_name)
+                self.log.debug('renaming %r to %r', old_path, new_path)
+                rename(old_path, new_path)
+                names[pos] = new_name
+
+
+    def _upgrade_metadata_3(self, md_version):
+        """Upgrade to version 4 (new layout!)
+
+        move "~/Ubuntu One/Shared With" Me to XDG_DATA/ubuntuone/shares
+        move "~/Ubuntu One/My Files" contents to "~/Ubuntu One"
+
+        """
+        from ubuntuone.syncdaemon.volume_manager import LegacyShareFileShelf
+        self.log.debug('upgrading from metadata 3 (new layout)')
+        old_share_dir = os.path.join(self._root_dir, 'Shared With Me')
+        old_root_dir = os.path.join(self._root_dir, 'My Files')
+        # change permissions
+        set_dir_readwrite(self._root_dir)
+
+        def move(src, dst):
+            """Move a file/dir taking care if it's read-only."""
+            prev_mode = stat.S_IMODE(os.stat(src).st_mode)
+            set_dir_readwrite(src)
+            recursive_move(src, dst)
+            os.chmod(dst, prev_mode)
+
+        # update the path's in metadata and move the folder
+        if path_exists(old_share_dir) and not is_link(old_share_dir):
+            set_dir_readwrite(old_share_dir)
+            if not path_exists(os.path.dirname(self._shares_dir)):
+                make_dir(os.path.dirname(self._shares_dir), recursive=True)
+            self.log.debug('moving shares dir from: %r to %r',
+                           old_share_dir, self._shares_dir)
+            for path in listdir(old_share_dir):
+                src = os.path.join(old_share_dir, path)
+                dst = os.path.join(self._shares_dir, path)
+                move(src, dst)
+            remove_dir(old_share_dir)
+
+        # update the shares metadata
+        shares = LegacyShareFileShelf(self._shares_md_dir)
+        for key, share in shares.iteritems():
+            if share.path is not None:
+                if share.path == old_root_dir:
+                    share.path = share.path.replace(old_root_dir,
+                                                    self._root_dir)
+                else:
+                    share.path = share.path.replace(old_share_dir,
+                                                    self._shares_dir)
+                shares[key] = share
+
+        shared = LegacyShareFileShelf(self._shared_md_dir)
+        for key, share in shared.iteritems():
+            if share.path is not None:
+                share.path = share.path.replace(old_root_dir, self._root_dir)
+            shared[key] = share
+        # move the My Files contents, taking care of dir/files with the same
+        # name in the new root
+        if path_exists(old_root_dir):
+            self.log.debug('moving My Files contents to the root')
+            # make My Files rw
+            set_dir_readwrite(old_root_dir)
+            path_join = os.path.join
+            for relpath in listdir(old_root_dir):
+                old_path = path_join(old_root_dir, relpath)
+                new_path = path_join(self._root_dir, relpath)
+                if path_exists(new_path):
+                    recursive_move(new_path, new_path+'.u1conflict')
+                if relpath == 'Shared With Me':
+                    # remove the Shared with Me symlink inside My Files!
+                    self.log.debug('removing shares symlink from old root')
+                    remove_file(old_path)
+                else:
+                    self.log.debug('moving %r to %r', old_path, new_path)
+                    move(old_path, new_path)
+            self.log.debug('removing old root: %r', old_root_dir)
+            remove_dir(old_root_dir)
+
+        # fix broken symlink (md_version 4)
+        self._upgrade_metadata_4(md_version)
+
+    def _upgrade_metadata_4(self, md_version):
+        """Upgrade to version 5 (fix the broken symlink!)."""
+        self.log.debug('upgrading from metadata 4 (broken symlink!)')
+        if is_link(self._shares_dir_link):
+            target = read_link(self._shares_dir_link)
+            if normpath(target) == self._shares_dir_link:
+                # the symnlink points to itself
+                self.log.debug('removing broken shares symlink: %r -> %r',
+                               self._shares_dir_link, target)
+                remove_file(self._shares_dir_link)
+        self._upgrade_metadata_5(md_version)
+
+    def _upgrade_metadata_5(self, md_version):
+        """Upgrade to version 6 (plain dict storage)."""
+        from ubuntuone.syncdaemon.volume_manager import VMFileShelf, LegacyShareFileShelf, UDF
+        self.log.debug('upgrading from metadata 5')
+        bkp_dir = os.path.join(os.path.dirname(self._data_dir), '5.bkp')
+        new_md_dir = os.path.join(os.path.dirname(self._data_dir), 'md_6.new')
+        new_shares_md_dir = os.path.join(new_md_dir, 'shares')
+        new_shared_md_dir = os.path.join(new_md_dir, 'shared')
+        new_udfs_md_dir = os.path.join(new_md_dir, 'udfs')
+        try:
+            # upgrade shares
+            old_shares = LegacyShareFileShelf(self._shares_md_dir)
+            shares = VMFileShelf(new_shares_md_dir)
+            for key, share in old_shares.iteritems():
+                shares[key] = self._upgrade_share_to_volume(share)
+            # upgrade shared folders
+            old_shared = LegacyShareFileShelf(self._shared_md_dir)
+            shared = VMFileShelf(new_shared_md_dir)
+            for key, share in old_shared.iteritems():
+                shared[key] = self._upgrade_share_to_volume(share, shared=True)
+            # upgrade the udfs
+            old_udfs = LegacyShareFileShelf(self._udfs_md_dir)
+            udfs = VMFileShelf(new_udfs_md_dir)
+            for key, udf in old_udfs.iteritems():
+                udfs[key] = UDF(udf.id, udf.node_id, udf.suggested_path,
+                                udf.path, udf.subscribed)
+            # move md dir to bkp
+            rename(self._data_dir, bkp_dir)
+            # move new to md dir
+            rename(new_md_dir, self._data_dir)
+            self._upgrade_metadata_6(md_version)
+        except Exception:
+            # something bad happend, remove partially upgraded metadata
+            remove_tree(new_md_dir)
+            raise
+
+    def _upgrade_share_to_volume(self, share, shared=False):
+        """Upgrade from _Share to new Volume hierarchy."""
+        from ubuntuone.syncdaemon.volume_manager import VMFileShelf, Root, Share, Shared
+        def upgrade_share_dict(share):
+            """Upgrade share __dict__ to be compatible with the
+            new Share.__init__.
+
+            """
+            if 'subtree' in share.__dict__:
+                share.node_id = share.__dict__.pop('subtree')
+            if 'id' in share.__dict__:
+                share.volume_id = share.__dict__.pop('id')
+            if 'free_bytes' in share.__dict__:
+                share.free_bytes = share.__dict__.pop('free_bytes')
+            else:
+                share.free_bytes = None
+            return share
+
+        if isinstance(share, dict):
+            # oops, we have mixed metadata. fix it!
+            clazz = VMFileShelf.classes[share[VMFileShelf.TYPE]]
+            share_dict = share.copy()
+            del share_dict[VMFileShelf.TYPE]
+            return clazz(**share_dict)
+        elif share.path == self._root_dir or share.id == '':
+            # handle the root special case
+            return Root(volume_id=request.ROOT,
+                        node_id=share.subtree, path=share.path)
+        else:
+            share = upgrade_share_dict(share)
+            if shared:
+                return Shared(**share.__dict__)
+            else:
+                return Share(**share.__dict__)
+
+    def _upgrade_metadata_6(self, md_version):
+        """Upgrade to version 7, tritcask!."""
+        from ubuntuone.syncdaemon.volume_manager import (
+            VMFileShelf, VMTritcaskShelf,
+            SHARE_ROW_TYPE, SHARED_ROW_TYPE, UDF_ROW_TYPE
+        )
+        self.log.debug('upgrading from metadata 6')
+        old_shares = VMFileShelf(self._shares_md_dir)
+        old_shared = VMFileShelf(self._shared_md_dir)
+        old_udfs = VMFileShelf(self._udfs_md_dir)
+        shares = VMTritcaskShelf(SHARE_ROW_TYPE, self.db)
+        shared = VMTritcaskShelf(SHARED_ROW_TYPE, self.db)
+        udfs = VMTritcaskShelf(UDF_ROW_TYPE, self.db)
+        for share_id, share in old_shares.iteritems():
+            shares[share_id] = share
+        for share_id, share in old_shared.iteritems():
+            shared[share_id] = share
+        for udf_id, udf in old_udfs.iteritems():
+            udfs[udf_id] = udf
+        # update the metadata version
+        self.update_metadata_version()
+        # now delete the old metadata
+        remove_tree(self._shares_md_dir)
+        remove_tree(self._shared_md_dir)
+        remove_tree(self._udfs_md_dir)
+
 
 class VMFileShelf(file_shelf.CachedFileShelf):
-    """Custom file shelf that allow request.ROOT as key, it's replaced
-    by the string: root_node_id.
+    """Custom file shelf.
+
+    Allow request.ROOT as key, it's replaced by the string: root_node_id.
 
     """
 
@@ -1494,8 +1861,7 @@ class VMFileShelf(file_shelf.CachedFileShelf):
 
 
 class LegacyShareFileShelf(VMFileShelf):
-    """A FileShelf capable of replacing pickled classes
-    with a different class.
+    """A FileShelf capable of replacing pickled classes with a different class.
 
     upgrade_map attribute is a dict of (module, name):class
 
@@ -1595,4 +1961,3 @@ class VMTritcaskShelf(TritcaskShelf):
     def _serialize(self, value):
         """Serialize value to string using protocol."""
         return cPickle.dumps(value.__dict__, protocol=2)
-
